@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: downloader-view.c,v 1.13 2003/05/25 18:59:59 xan Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,6 +29,7 @@
 #include "ephy-embed-utils.h"
 #include "ephy-file-helpers.h"
 #include "ephy-embed-shell.h"
+#include "ephy-stock-icons.h"
 
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkliststore.h>
@@ -69,7 +72,6 @@ struct DownloaderViewPrivate
 	GtkWidget *details_remaining;
 	GtkWidget *details_progress;
 	GtkWidget *details_button;
-	GtkWidget *keep_open_check;
 
 	GtkWidget *open_button;
 	GtkWidget *pause_button;
@@ -88,6 +90,7 @@ typedef struct
 	gchar *source;
 	gchar *dest;
 	DownloadStatus status;
+	gboolean download_started;
 
 	GtkTreeRowReference *ref;
 } DownloadDetails;
@@ -97,6 +100,7 @@ typedef struct
 	gboolean is_paused;
 	gboolean can_abort;
 	gboolean can_open;
+	gboolean can_pause;
 	DownloaderViewPrivate *priv;
 } ControlsInfo;
 
@@ -104,9 +108,7 @@ enum
 {
 	PROP_WINDOW,
 	PROP_TREEVIEW,
-	PROP_KEEP_OPEN,
 	PROP_DETAILS_FRAME,
-	PROP_DETAILS_SEPARATOR,
 	PROP_DETAILS_TABLE,
 	PROP_DETAILS_STATUS,
 	PROP_DETAILS_ELAPSED,
@@ -123,9 +125,7 @@ EphyDialogProperty properties [] =
 {
 	{ PROP_WINDOW, "download_manager_dialog", NULL, PT_NORMAL, NULL},
         { PROP_TREEVIEW, "clist", NULL, PT_NORMAL, NULL },
-	{ PROP_KEEP_OPEN, "keep_open_check", CONF_DOWNLOADING_KEEP_OPEN, PT_NORMAL, NULL },
 	{ PROP_DETAILS_FRAME, "details_frame", NULL, PT_NORMAL, NULL },
-	{ PROP_DETAILS_SEPARATOR, "details_separator", NULL, PT_NORMAL, NULL },
 	{ PROP_DETAILS_TABLE, "details_table", NULL, PT_NORMAL, NULL },
 	{ PROP_DETAILS_STATUS, "details_status", NULL, PT_NORMAL, NULL },
 	{ PROP_DETAILS_ELAPSED, "details_elapsed", NULL, PT_NORMAL, NULL },
@@ -210,11 +210,13 @@ format_time (gchar *buffer, glong time)
 
 	if (hours)
 	{
-		sprintf (buffer, "%u:%02u.%02u", hours, mins, secs);
+		/* Hours, Minutes, Seconds */
+		sprintf (buffer, _("%u:%02u.%02u"), hours, mins, secs);
 	}
 	else
 	{
-		sprintf (buffer, "%02u.%02u", mins, secs);
+		/* Minutes, Seconds */
+		sprintf (buffer, _("%02u.%02u"), mins, secs);
 	}
 }
 
@@ -328,6 +330,8 @@ controls_info_foreach (GtkTreeModel *model,
 	info->is_paused |= (details->status == DOWNLOAD_STATUS_PAUSED);
 	info->can_abort |= (details->status != DOWNLOAD_STATUS_COMPLETED);
 	info->can_open |= (details->status == DOWNLOAD_STATUS_COMPLETED);
+	info->can_pause |= ((details->status != DOWNLOAD_STATUS_COMPLETED) &&
+			   (details->download_started == TRUE));
 }
 
 static void
@@ -342,7 +346,7 @@ downloader_view_update_controls (DownloaderViewPrivate *priv)
 	info->priv = priv;
 
 	/* initial conditions */
-	info->is_paused = info->can_abort = info->can_open = FALSE;
+	info->is_paused = info->can_abort = info->can_open = info->can_pause = FALSE;
 
 	if (selection)
 	{
@@ -354,8 +358,7 @@ downloader_view_update_controls (DownloaderViewPrivate *priv)
 
 	/* setup buttons */
 	gtk_widget_set_sensitive (priv->open_button, info->can_open);
-	gtk_widget_set_sensitive (priv->pause_button, info->can_abort);
-	/* As long as we can abort, we can pause/resume */
+	gtk_widget_set_sensitive (priv->pause_button, info->can_pause);
 	gtk_widget_set_sensitive (priv->abort_button, info->can_abort);
 
 	if (info->is_paused)
@@ -475,9 +478,22 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 			    COL_FILENAME, details->filename,
 			    -1);
 
+	/* Pause Activation */
+
+	if (details->download_started == FALSE &&
+	    details->status != DOWNLOAD_STATUS_COMPLETED &&
+	    details->size_total != -1)
+	{
+		details->download_started = TRUE;
+		downloader_view_update_controls (priv);
+	}
+		
 	/* Progress */
 	if (details->status == DOWNLOAD_STATUS_COMPLETED)
+	{
 		details->progress = 1;
+	}
+	
 	sprintf (buffer, "%.1f%%",
 		 details->progress > 0 ?
 		 details->progress * 100.0 :
@@ -515,7 +531,7 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	{
 		sprintf (buffer,
 			 details->progress > 0 ?
-		 	 "00.00" :
+		 	 _("00.00") :
 		 	 _("Unknown"));
 	}
 	
@@ -573,6 +589,7 @@ downloader_view_add_download (DownloaderView *dv,
 	details->size_total = -1;
 	details->size_done = -1;
 	details->progress = -1;
+	details->download_started = FALSE;
 	dv->priv->show_details = FALSE;
 
 	g_hash_table_insert (dv->priv->details_hash,
@@ -608,7 +625,6 @@ downloader_view_remove_download (DownloaderView *dv,
 {
 	DownloadDetails *details;
 	GtkTreeIter iter;
-	GValue keep_open = {0, };
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
@@ -623,17 +639,7 @@ downloader_view_remove_download (DownloaderView *dv,
 	g_hash_table_remove (dv->priv->details_hash,
 			     persist_object);
 
-        ephy_dialog_get_value (EPHY_DIALOG(dv), PROP_KEEP_OPEN, &keep_open);
-
-	if (!g_value_get_boolean (&keep_open) &&
-	    g_hash_table_size (dv->priv->details_hash) == 0)
-	{
-		g_object_unref (dv);
-	}
-	else
-	{
-		ensure_selected_row (dv);
-	}
+	ensure_selected_row (dv);
 }
 
 void
@@ -674,7 +680,6 @@ downloader_view_set_download_status (DownloaderView *dv,
 {
 	DownloadDetails *details;
 	GtkTreeIter iter;
-	GValue keep_open = {0, };
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
@@ -689,13 +694,10 @@ downloader_view_set_download_status (DownloaderView *dv,
 	downloader_view_set_download_info (dv->priv, details, &iter);
 	downloader_view_update_controls (dv->priv);
 
-        ephy_dialog_get_value (EPHY_DIALOG(dv), PROP_KEEP_OPEN, &keep_open);
-
-	if (status == DOWNLOAD_STATUS_COMPLETED &&
-	    !g_value_get_boolean (&keep_open))
+/*	if (status == DOWNLOAD_STATUS_COMPLETED)
 	{
 		downloader_view_remove_download (dv, persist_object);
-	}
+	}*/
 }
 
 static void
@@ -707,7 +709,7 @@ downloader_view_build_ui (DownloaderView *dv)
 	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
 	GtkWidget *details_table;
-	const char *icon_path;
+	GdkPixbuf *icon;
 	EphyDialog *d = EPHY_DIALOG (dv);
 
 	ephy_dialog_construct (d,
@@ -722,7 +724,6 @@ downloader_view_build_ui (DownloaderView *dv)
 	priv->details_elapsed = ephy_dialog_get_control (d, PROP_DETAILS_ELAPSED);
 	priv->details_remaining = ephy_dialog_get_control (d, PROP_DETAILS_REMAINING);
 	priv->details_progress = ephy_dialog_get_control (d, PROP_DETAILS_PROGRESS);
-	priv->keep_open_check = ephy_dialog_get_control (d, PROP_KEEP_OPEN);
 	priv->details_button = ephy_dialog_get_control (d, PROP_DETAILS_BUTTON);
 	priv->open_button = ephy_dialog_get_control (d, PROP_OPEN_BUTTON);
 	priv->pause_button = ephy_dialog_get_control (d, PROP_PAUSE_BUTTON);
@@ -731,8 +732,9 @@ downloader_view_build_ui (DownloaderView *dv)
 
 	/* create file and location details labels */
 	priv->details_location = ephy_ellipsizing_label_new ("");
-	gtk_table_attach_defaults (GTK_TABLE(details_table), priv->details_location,
-				   1, 2, 0, 1);
+	gtk_table_attach_defaults (GTK_TABLE(details_table),
+				   priv->details_location,
+				   1, 2, 1, 2);
 	gtk_misc_set_alignment (GTK_MISC(priv->details_location), 0, 0);
 	gtk_label_set_selectable (GTK_LABEL(priv->details_location), TRUE);
 	gtk_widget_show (priv->details_location);
@@ -801,8 +803,11 @@ downloader_view_build_ui (DownloaderView *dv)
 
 	priv->model = GTK_TREE_MODEL (liststore);
 
-	icon_path =  ephy_file ("epiphany-download.png");
-	gtk_window_set_icon_from_file (GTK_WINDOW(priv->window), icon_path, NULL);
+	icon = gtk_widget_render_icon (GTK_WIDGET (priv->window),
+				       EPHY_STOCK_DOWNLOAD,
+				       GTK_ICON_SIZE_MENU,
+				       NULL);
+	gtk_window_set_icon (GTK_WINDOW(priv->window), icon);
 }
 
 void
@@ -898,11 +903,14 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	GValue val = {0, };
 	gpointer *persist_object;
 	DownloadDetails *details = NULL;
+	GtkWidget *details_button;
 	GtkWidget *details_frame;
 	DownloaderViewPrivate *priv= dv->priv;
 
+	details_button = ephy_dialog_get_control (EPHY_DIALOG(dv),
+						  PROP_DETAILS_BUTTON);
 	details_frame = ephy_dialog_get_control (EPHY_DIALOG(dv),
-						   PROP_DETAILS_FRAME);
+						 PROP_DETAILS_FRAME);
 
 	if (get_selected_row (priv, &iter))
 	{
@@ -913,6 +921,7 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 					       persist_object);
 		g_return_if_fail (details);
 
+		gtk_widget_set_sensitive (details_button, TRUE);
 		gtk_widget_set_sensitive (details_frame, TRUE);
 
 		downloader_view_update_details (priv, details);
@@ -929,6 +938,8 @@ downloader_treeview_selection_changed_cb (GtkTreeSelection *selection,
 			 0);
 
 		gtk_widget_set_sensitive (details_frame, FALSE);
+		if (!gtk_tree_model_get_iter_first (priv->model, &iter))
+			gtk_widget_set_sensitive (details_button, FALSE);
 	}
 }
 
@@ -997,23 +1008,17 @@ download_dialog_details_cb (GtkToggleButton *button,
 			    DownloaderView *dv)
 {
 	GtkWidget *details_frame;
-	GtkWidget *details_separator;
 
 	details_frame = ephy_dialog_get_control (EPHY_DIALOG(dv),
 						   PROP_DETAILS_FRAME);
-	details_separator = ephy_dialog_get_control (EPHY_DIALOG(dv),
-						       PROP_DETAILS_SEPARATOR);
-
 	if (gtk_toggle_button_get_active (button))
 	{
 		gtk_widget_show (GTK_WIDGET (details_frame));
-		gtk_widget_show (GTK_WIDGET (details_separator));
 		dv->priv->show_details = TRUE;
 	}
 	else
 	{
 		gtk_widget_hide (GTK_WIDGET (details_frame));
-		gtk_widget_hide (GTK_WIDGET (details_separator));
 		dv->priv->show_details = FALSE;
 	}
 
