@@ -26,6 +26,12 @@
 
 #include <string.h>
 
+enum
+{
+	NEW_WINDOW,
+	LAST_SIGNAL
+};
+
 struct EphyEmbedSinglePrivate
 {
 	EphyHistory *global_history;
@@ -42,6 +48,7 @@ static void
 ephy_embed_single_finalize (GObject *object);
 
 static GObjectClass *parent_class = NULL;
+static guint ephy_embed_single_signals[LAST_SIGNAL] = { 0 };
 
 GType
 ephy_embed_single_get_type (void)
@@ -79,6 +86,18 @@ ephy_embed_single_class_init (EphyEmbedSingleClass *klass)
 	parent_class = (GObjectClass *) g_type_class_peek_parent (klass);
 
         object_class->finalize = ephy_embed_single_finalize;
+
+	ephy_embed_single_signals[NEW_WINDOW] =
+                g_signal_new ("new_window_orphan",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (EphyEmbedSingleClass, new_window),
+                              NULL, NULL,
+                              ephy_marshal_VOID__POINTER_INT,
+                              G_TYPE_NONE,
+                              2,
+                              G_TYPE_POINTER,
+			      G_TYPE_INT);
 }
 
 static void
@@ -148,20 +167,21 @@ ephy_embed_single_load_proxy_autoconf (EphyEmbedSingle *shell,
 }
 
 gresult
-ephy_embed_single_get_charset_titles (EphyEmbedSingle *shell,
-				     const char *group,
-				     GList **charsets)
+ephy_embed_single_get_encodings (EphyEmbedSingle *shell,
+				 LanguageGroup group,
+				 gboolean elide_underscores,
+				 GList **encodings)
 {
 	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (shell);
-        return klass->get_charset_titles (shell, group, charsets);
+	return klass->get_encodings (shell, group, elide_underscores, encodings);
 }
 
 gresult
-ephy_embed_single_get_charset_groups (EphyEmbedSingle *shell,
-			             GList **groups)
+ephy_embed_single_get_language_groups (EphyEmbedSingle *shell,
+			              GList **groups)
 {
 	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (shell);
-        return klass->get_charset_groups (shell, groups);
+	return klass->get_language_groups (shell, groups);
 }
 
 gresult
@@ -236,14 +256,13 @@ ephy_embed_single_show_file_picker (EphyEmbedSingle *shell,
 		                   const char *file,
 				   FilePickerMode mode,
 				   char **ret_fullpath,
-				   gboolean *ret_save_content,
 				   FileFormat *file_formats,
 				   int *ret_file_format)
 {
 	EphyEmbedSingleClass *klass = EPHY_EMBED_SINGLE_GET_CLASS (shell);
         return klass->show_file_picker (shell, parentWidget, title,
 					directory, file, mode,
-					ret_fullpath, ret_save_content,
+					ret_fullpath,
 					file_formats, ret_file_format);
 }
 
