@@ -41,6 +41,7 @@ struct EphyFaviconCachePrivate
 {
 	char *directory;
 	char *xml_file;
+	EphyNodeDb *db;
 	EphyNode *icons;
 	GHashTable *icons_hash;
 	GStaticRWLock *icons_hash_lock;
@@ -141,7 +142,7 @@ ephy_favicon_cache_load (EphyFaviconCache *eb)
 	{
 		EphyNode *node;
 
-		node = ephy_node_new_from_xml (child);
+		node = ephy_node_new_from_xml (eb->priv->db, child);
 	}
 
 	xmlFreeDoc (doc);
@@ -220,7 +221,7 @@ remove_obsolete_icons (EphyFaviconCache *eb)
 			path = g_build_filename (eb->priv->directory,
 						 filename, NULL);
 			gnome_vfs_unlink (path);
-			g_object_unref (kid);
+			ephy_node_unref (kid);
 		}
 	}
 }
@@ -252,14 +253,21 @@ ephy_favicon_cache_save (EphyFaviconCache *eb)
 	}
 	ephy_node_thaw (eb->priv->icons);
 
-	xmlSaveFormatFile (eb->priv->xml_file, doc, 1);
+	ephy_file_save_xml (eb->priv->xml_file, doc);
+
+	xmlFreeDoc (doc);
 }
 
 static void
 ephy_favicon_cache_init (EphyFaviconCache *cache)
 {
+	EphyNodeDb *db;
+
 	cache->priv = g_new0 (EphyFaviconCachePrivate, 1);
 
+	db = ephy_node_db_new ("EphyFaviconCache");
+	cache->priv->db = db;
+
 	cache->priv->xml_file = g_build_filename (ephy_dot_dir (),
 					          "ephy-favicon-cache.xml",
 					          NULL);
@@ -286,18 +294,16 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 							     NULL);
 
 	/* Icons */
-	cache->priv->icons = ephy_node_new_with_id (ICONS_NODE_ID);
+	cache->priv->icons = ephy_node_new_with_id (db, ICONS_NODE_ID);
 	ephy_node_ref (cache->priv->icons);
-	g_signal_connect_object (G_OBJECT (cache->priv->icons),
-				 "child_added",
-				 G_CALLBACK (icons_added_cb),
-				 G_OBJECT (cache),
-				 0);
-	g_signal_connect_object (G_OBJECT (cache->priv->icons),
-				 "child_removed",
-				 G_CALLBACK (icons_removed_cb),
-				 G_OBJECT (cache),
-				 0);
+	ephy_node_signal_connect_object (cache->priv->icons,
+				         EPHY_NODE_CHILD_ADDED,
+				         (EphyNodeCallback) icons_added_cb,
+				         G_OBJECT (cache));
+	ephy_node_signal_connect_object (cache->priv->icons,
+				         EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) icons_removed_cb,
+				         G_OBJECT (cache));
 
 	ephy_favicon_cache_load (cache);
 }
@@ -318,7 +324,7 @@ kill_download (gpointer key,
 	icon = g_hash_table_lookup (cache->priv->icons_hash, (char *)key);
 	g_static_rw_lock_reader_unlock (cache->priv->icons_hash_lock);
 
-	g_object_unref (icon);
+	ephy_node_unref (icon);
 
 	return TRUE;
 }
@@ -355,6 +361,8 @@ ephy_favicon_cache_finalize (GObject *object)
 	g_static_rw_lock_free (cache->priv->icons_hash_lock);
 	g_hash_table_destroy (cache->priv->downloads_hash);
 
+	g_object_unref (cache->priv->db);
+
 	g_free (cache->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -447,7 +455,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 
 		filename = favicon_name_build (url);
 
-		icon = ephy_node_new ();
+		icon = ephy_node_new (cache->priv->db);
 		g_value_init (&value, G_TYPE_STRING);
 		g_value_set_string (&value, url);
 		ephy_node_set_property (icon, EPHY_NODE_FAVICON_PROP_URL,
