@@ -42,6 +42,7 @@
 struct EphyHistoryPrivate
 {
 	char *xml_file;
+	EphyNodeDb *db;
 	EphyNode *hosts;
 	EphyNode *pages;
 	EphyNode *last_page;
@@ -142,7 +143,7 @@ ephy_history_autocompletion_source_foreach (EphyAutocompletionSource *source,
 		guint32 score;
 
 		kid = g_ptr_array_index (children, i);
-		g_assert (EPHY_IS_NODE (kid));
+		g_assert (kid != NULL);
 
 		url = ephy_node_get_property_string
 			(kid, EPHY_NODE_PAGE_PROP_LOCATION);
@@ -218,7 +219,7 @@ ephy_history_load (EphyHistory *eb)
 	{
 		EphyNode *node;
 
-		node = ephy_node_new_from_xml (child);
+		node = ephy_node_new_from_xml (eb->priv->db, child);
 	}
 
 	xmlFreeDoc (doc);
@@ -308,7 +309,7 @@ ephy_history_save (EphyHistory *eb)
 	}
 	ephy_node_thaw (eb->priv->pages);
 
-	xmlSaveFormatFile (eb->priv->xml_file, doc, 1);
+	ephy_file_save_xml (eb->priv->xml_file, doc);
 	xmlFreeDoc(doc);
 }
 
@@ -329,6 +330,7 @@ hosts_added_cb (EphyNode *node,
 static void
 hosts_removed_cb (EphyNode *node,
 		  EphyNode *child,
+		  guint old_index,
 		  EphyHistory *eb)
 {
 	g_static_rw_lock_writer_lock (eb->priv->hosts_hash_lock);
@@ -356,29 +358,15 @@ pages_added_cb (EphyNode *node,
 static void
 pages_removed_cb (EphyNode *node,
 		  EphyNode *child,
+		  guint old_index,
 		  EphyHistory *eb)
 {
-	EphyNode *host;
-	int host_id;
-	int children;
-
 	g_static_rw_lock_writer_lock (eb->priv->pages_hash_lock);
 
 	g_hash_table_remove (eb->priv->pages_hash,
 			     ephy_node_get_property_string (child, EPHY_NODE_PAGE_PROP_LOCATION));
 
 	g_static_rw_lock_writer_unlock (eb->priv->pages_hash_lock);
-
-	host_id = ephy_node_get_property_int (child, EPHY_NODE_PAGE_PROP_HOST_ID);
-	host = ephy_node_get_from_id (host_id);
-	children = ephy_node_get_n_children (host);
-
-	LOG ("Check host children: %d", children)
-
-	if (children == 0)
-	{
-		ephy_node_unref (host);
-	}
 }
 
 static gboolean
@@ -390,13 +378,50 @@ periodic_save_cb (EphyHistory *eh)
 	return TRUE;
 }
 
+static gboolean
+unref_empty_host (EphyNode *node)
+{
+	ephy_node_unref (node);
+
+	return FALSE;
+}
+
+static void
+page_removed_from_host_cb (EphyNode *node,
+		           EphyNode *child,
+		           guint old_index,
+		           EphyHistory *eb)
+{
+	if (ephy_node_get_n_children (node) == 0)
+	{
+		g_idle_add ((GSourceFunc)unref_empty_host, node);
+	}
+}
+
+static void
+connect_page_removed_from_host (char *url,
+                                EphyNode *node,
+                                EphyHistory *eb)
+{
+	if (node == eb->priv->pages) return;
+
+	ephy_node_signal_connect_object (node,
+					 EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) page_removed_from_host_cb,
+					 G_OBJECT (eb));
+}
+
 static void
 ephy_history_init (EphyHistory *eb)
 {
 	GValue value = { 0, };
+	EphyNodeDb *db;
 
         eb->priv = g_new0 (EphyHistoryPrivate, 1);
 
+	db = ephy_node_db_new ("EphyHistory");
+	eb->priv->db = db;
+
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
 					       "ephy-history.xml",
 					       NULL);
@@ -412,7 +437,7 @@ ephy_history_init (EphyHistory *eb)
 	g_static_rw_lock_init (eb->priv->hosts_hash_lock);
 
 	/* Pages */
-	eb->priv->pages = ephy_node_new_with_id (PAGES_NODE_ID);
+	eb->priv->pages = ephy_node_new_with_id (db, PAGES_NODE_ID);
 	ephy_node_ref (eb->priv->pages);
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, _("All"));
@@ -429,36 +454,36 @@ ephy_history_init (EphyHistory *eb)
 			        EPHY_NODE_PAGE_PROP_PRIORITY,
 			        &value);
 	g_value_unset (&value);
-	g_signal_connect_object (G_OBJECT (eb->priv->pages),
-				 "child_added",
-				 G_CALLBACK (pages_added_cb),
-				 G_OBJECT (eb),
-				 0);
-	g_signal_connect_object (G_OBJECT (eb->priv->pages),
-				 "child_removed",
-				 G_CALLBACK (pages_removed_cb),
-				 G_OBJECT (eb),
-				 0);
+	ephy_node_signal_connect_object (eb->priv->pages,
+					 EPHY_NODE_CHILD_ADDED,
+				         (EphyNodeCallback) pages_added_cb,
+					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->pages,
+					 EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) pages_removed_cb,
+					 G_OBJECT (eb));
 
 	/* Hosts */
-	eb->priv->hosts = ephy_node_new_with_id (HOSTS_NODE_ID);
+	eb->priv->hosts = ephy_node_new_with_id (db, HOSTS_NODE_ID);
 	ephy_node_ref (eb->priv->hosts);
-	g_signal_connect_object (G_OBJECT (eb->priv->hosts),
-				 "child_added",
-				 G_CALLBACK (hosts_added_cb),
-				 G_OBJECT (eb),
-				 0);
-	g_signal_connect_object (G_OBJECT (eb->priv->hosts),
-				 "child_removed",
-				 G_CALLBACK (hosts_removed_cb),
-				 G_OBJECT (eb),
-				 0);
+	ephy_node_signal_connect_object (eb->priv->hosts,
+					 EPHY_NODE_CHILD_ADDED,
+				         (EphyNodeCallback) hosts_added_cb,
+					 G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->hosts,
+					 EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) hosts_removed_cb,
+					 G_OBJECT (eb));
 
 	ephy_node_add_child (eb->priv->hosts, eb->priv->pages);
 
 	ephy_history_load (eb);
 	ephy_history_emit_data_changed (eb);
 
+	g_hash_table_foreach (eb->priv->hosts_hash,
+			      (GHFunc) connect_page_removed_from_host,
+			      eb);
+
 	/* setup the periodic history saving callback */
 	eb->priv->autosave_timeout =
 		g_timeout_add (HISTORY_SAVE_INTERVAL,
@@ -482,6 +507,8 @@ ephy_history_finalize (GObject *object)
 	ephy_node_unref (eb->priv->pages);
 	ephy_node_unref (eb->priv->hosts);
 
+	g_object_unref (eb->priv->db);
+
 	g_hash_table_destroy (eb->priv->pages_hash);
 	g_static_rw_lock_free (eb->priv->pages_hash_lock);
 	g_hash_table_destroy (eb->priv->hosts_hash);
@@ -610,7 +637,12 @@ ephy_history_add_host (EphyHistory *eh, EphyNode *page)
 
 	if (!host)
 	{
-		host = ephy_node_new ();
+		host = ephy_node_new (eh->priv->db);
+		ephy_node_signal_connect_object (host,
+						 EPHY_NODE_CHILD_REMOVED,
+					         (EphyNodeCallback) page_removed_from_host_cb,
+						 G_OBJECT (eh));
+
 		g_value_init (&value, G_TYPE_STRING);
 		g_value_set_string (&value, host_name);
 		ephy_node_set_property (host, EPHY_NODE_PAGE_PROP_TITLE,
@@ -656,7 +688,7 @@ ephy_history_visited (EphyHistory *eh, EphyNode *node)
 
 	now = time (NULL);
 
-	g_assert (EPHY_IS_NODE (node));
+	g_assert (node != NULL);
 
 	url = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_LOCATION);
@@ -686,7 +718,10 @@ ephy_history_visited (EphyHistory *eh, EphyNode *node)
 	host_id = ephy_node_get_property_int (node, EPHY_NODE_PAGE_PROP_HOST_ID);
 	if (host_id >= 0)
 	{
-		ephy_history_host_visited (eh, ephy_node_get_from_id (host_id), now);
+		EphyNode *host;
+
+		host = ephy_node_db_get_node_from_id (eh->priv->db, host_id);
+		ephy_history_host_visited (eh, host, now);
 	}
 
 	eh->priv->last_page = node;
@@ -727,7 +762,7 @@ ephy_history_add_page (EphyHistory *eb,
 		return;
 	}
 
-	bm = ephy_node_new ();
+	bm = ephy_node_new (eb->priv->db);
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, url);
@@ -825,12 +860,6 @@ ephy_history_clear (EphyHistory *gh)
 		ephy_node_unref (node);
 	}
 
-	/* The first node is All, dont unref it */
-	while ((node = ephy_node_get_nth_child (gh->priv->hosts, 1)) != NULL)
-	{
-		ephy_node_unref (node);
-	}
-
 	ephy_history_save (gh);
 }
 
