@@ -34,7 +34,6 @@
 
 #include "ephy-string.h"
 #include "ephy-gui.h"
-#include "eel-gconf-extensions.h"
 
 #include <glib/gconvert.h>
 #include <gtk/gtkmain.h>
@@ -76,9 +75,7 @@
 /* Implementation file */
 NS_IMPL_ISUPPORTS1(GFilePicker, nsIFilePicker)
 
-GFilePicker::GFilePicker(PRBool aShowContentCheck, FileFormat *aFileFormats) :
-			mShowContentCheck(aShowContentCheck),
-			mSaveContentCheck(NULL),
+GFilePicker::GFilePicker(FileFormat *aFileFormats) :
 			mFileFormats(aFileFormats)
 {
 	NS_INIT_ISUPPORTS();
@@ -143,9 +140,9 @@ NS_IMETHODIMP GFilePicker::SetFilterIndex(PRInt32 aFilterIndex)
 /* attribute wstring defaultString; */
 NS_IMETHODIMP GFilePicker::GetDefaultString(PRUnichar * *aDefaultString)
 {
-	guint bytesRead(0), bytesWritten(0);
+	gsize bytesWritten;
 	gchar *utf8DefaultString = g_filename_to_utf8(mDefaultString.get(), -1,
-						      &bytesRead,
+						      NULL,
 						      &bytesWritten, NULL);
 
 	*aDefaultString = ToNewUnicode(NS_ConvertUTF8toUCS2(utf8DefaultString));
@@ -157,10 +154,10 @@ NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar *aDefaultString)
 {
 	if (aDefaultString)
 	{
-		guint bytesRead(0), bytesWritten(0);
+		gsize bytesWritten;
 		gchar *localeDefaultString =
 			g_filename_from_utf8(NS_ConvertUCS2toUTF8(aDefaultString).get(),
-					     -1, &bytesRead,
+					     -1, NULL,
 					     &bytesWritten, NULL);
 		mDefaultString = localeDefaultString;					     
 		g_free(localeDefaultString);
@@ -246,24 +243,6 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 		gtk_window_set_transient_for(GTK_WINDOW(mFileSelector),
 					     GTK_WINDOW(mParentWidget));
 
-	if (mShowContentCheck)
-	{
-		GtkWidget *bbox = gtk_hbutton_box_new ();
-		gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox),
-					  GTK_BUTTONBOX_END);
-		gtk_box_set_spacing(GTK_BOX(bbox), 0);
-        	gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(mFileSelector)->action_area),
-				 bbox, TRUE, TRUE, 0);
-
-		mSaveContentCheck = 
-			gtk_check_button_new_with_label(_("Save with content"));
-
-		gtk_box_pack_start(GTK_BOX(bbox), mSaveContentCheck,
-				   FALSE, FALSE, 0);
-
-		gtk_widget_show_all(bbox);
-	}
-
 	if (mFileFormats)
 	{
 		mFormatChooser = gtk_option_menu_new();
@@ -301,11 +280,9 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 
 	gint retVal = gtk_dialog_run(GTK_DIALOG(mFileSelector));
 	
-	if (retVal == GTK_RESPONSE_OK)
-	{
-		HandleFilePickerResult(_retval);
-	}
-	else
+	HandleFilePickerResult(_retval);
+
+	if (retVal != GTK_RESPONSE_OK)
 	{
 		*_retval = returnCancel;
 	}
@@ -482,11 +459,7 @@ NS_METHOD GFilePicker::HandleFilePickerResult(PRInt16 *retval)
 		}
 	}
 
-	if (GTK_IS_TOGGLE_BUTTON(mSaveContentCheck))
-		*retval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mSaveContentCheck)) ?
-			  returnOKSaveContent : returnOK;
-	else
-		*retval = returnOK;
+	*retval = returnOK;
 
 	return NS_OK;
 }
