@@ -48,8 +48,15 @@ NS_GENERIC_FACTORY_CONSTRUCTOR(MozGlobalHistory)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GPrintingPromptService)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GIRCProtocolHandler)
 NS_GENERIC_FACTORY_CONSTRUCTOR(GFtpProtocolHandler)
+NS_GENERIC_FACTORY_CONSTRUCTOR(GExternalProtocolService)
 
 static const nsModuleComponentInfo sAppComps[] = {
+	{
+		G_EXTERNALPROTOCOLSERVICE_CLASSNAME,
+		G_EXTERNALPROTOCOLSERVICE_CID,
+		NS_EXTERNALPROTOCOLSERVICE_CONTRACTID,
+ 		GExternalProtocolServiceConstructor
+	},
 	{
 		G_PROGRESSDIALOG_CLASSNAME,
 		G_PROGRESSDIALOG_CID,
