@@ -16,15 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "glib.h"
 #include "ephy-string.h"
+#include "ephy-debug.h"
 #include "gtkmozembed.h"
 #include "mozilla-embed-single.h"
-#include "mozilla-prefs.h"
 #include "ephy-prefs.h"
 #include "ephy-file-helpers.h"
 #include "mozilla-notifiers.h"
-#include "mozilla-i18n.h"
+#include "ephy-langs.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-embed-prefs.h"
 #include "MozRegisterComponents.h"
@@ -35,6 +39,7 @@
 #include <string.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
+#include <nsIPrefService.h>
 #include <nsNetCID.h>
 #include <nsIServiceManager.h>
 #include <nsIIOService.h>
@@ -57,6 +62,137 @@
 #define MOZILLA_PROFILE_NAME "epiphany"
 #define MOZILLA_PROFILE_FILE "prefs.js"
 
+/* language groups names */
+static const
+struct
+{
+	gchar *title;
+	LanguageGroup group;
+}
+lang_groups[] =
+{
+        { N_("_Arabic"),		LG_ARABIC },
+        { N_("_Baltic"),		LG_BALTIC },
+        { N_("Central _European"),	LG_CENTRAL_EUROPEAN },
+        { N_("Chi_nese"),		LG_CHINESE },
+        { N_("_Cyrillic"),		LG_CYRILLIC },
+        { N_("_Greek"),			LG_GREEK },
+        { N_("_Hebrew"),		LG_HEBREW },
+        { N_("_Indian"),		LG_INDIAN },
+        { N_("_Japanese"),		LG_JAPANESE },
+        { N_("_Korean"),		LG_KOREAN },
+        { N_("_Turkish"),		LG_TURKISH },
+        { N_("_Unicode"),		LG_UNICODE },
+        { N_("_Vietnamese"),		LG_VIETNAMESE },
+        { N_("_Western"),		LG_WESTERN },
+        { N_("_Other"),			LG_OTHER }
+};
+static const guint n_lang_groups = G_N_ELEMENTS (lang_groups);
+
+/**
+ * translatable encodings titles
+ * NOTE: if you add /change encodings, please also update the schema file
+ * epiphany.schemas.in
+ */
+static const 
+struct
+{
+	gchar *title;
+	gchar *name;
+	LanguageGroup group;
+}
+encodings[] =
+{ 
+	{ N_("Arabic (_IBM-864)"),                  "IBM864",                LG_ARABIC },
+	{ N_("Arabic (I_BM-864-I)"),                "IBM864i",               LG_ARABIC },
+	{ N_("Arabic (ISO-_8859-6)"),               "ISO-8859-6",            LG_ARABIC },
+	{ N_("Arabic (I_SO-8859-6-E)"),             "ISO-8859-6-E",          LG_ARABIC },
+	{ N_("Arabic (IS_O-8859-6-I)"),             "ISO-8859-6-I",          LG_ARABIC },
+	{ N_("Arabic (_MacArabic)"),                "x-mac-arabic",          LG_ARABIC },
+	{ N_("Arabic (_Windows-1256)"),             "windows-1256",          LG_ARABIC },
+	{ N_("_Armenian (ARMSCII-8)"),              "armscii-8",             LG_OTHER },
+	{ N_("Baltic (_ISO-8859-13)"),              "ISO-8859-13",           LG_BALTIC },
+	{ N_("Baltic (I_SO-8859-4)"),               "ISO-8859-4",            LG_BALTIC },
+	{ N_("Baltic (_Windows-1257)"),             "windows-1257",          LG_BALTIC },
+	{ N_("_Celtic (ISO-8859-14)"),              "ISO-8859-14",           LG_OTHER },
+	{ N_("Central European (_IBM-852)"),        "IBM852",                LG_CENTRAL_EUROPEAN },
+	{ N_("Central European (I_SO-8859-2)"),     "ISO-8859-2",	     LG_CENTRAL_EUROPEAN },
+	{ N_("Central European (_MacCE)"),          "x-mac-ce",              LG_CENTRAL_EUROPEAN },
+	{ N_("Central European (_Windows-1250)"),   "windows-1250",          LG_CENTRAL_EUROPEAN },
+	{ N_("Chinese Simplified (_GB18030)"),      "gb18030",               LG_CHINESE },
+	{ N_("Chinese Simplified (G_B2312)"),       "GB2312",                LG_CHINESE },
+	{ N_("Chinese Simplified (GB_K)"),          "x-gbk",                 LG_CHINESE },
+	{ N_("Chinese Simplified (_HZ)"),           "HZ-GB-2312",	     LG_CHINESE },
+	{ N_("Chinese Simplified (_ISO-2022-CN)"),  "ISO-2022-CN",           LG_CHINESE },
+	{ N_("Chinese Simplified (_Windows-936)"),  "windows-936",           LG_CHINESE },
+	{ N_("Chinese Traditional (Big_5)"),        "Big5",                  LG_CHINESE },
+	{ N_("Chinese Traditional (Big5-HK_SCS)"),  "Big5-HKSCS",	     LG_CHINESE },
+	{ N_("Chinese Traditional (_EUC-TW)"),      "x-euc-tw",              LG_CHINESE },
+	{ N_("Croatian (Mac_Croatian)"),            "x-mac-croatian",        LG_CENTRAL_EUROPEAN },
+	{ N_("Cyrillic (_IBM-855)"),                "IBM855",                LG_CYRILLIC },
+	{ N_("Cyrillic (I_SO-8859-5)"),             "ISO-8859-5",	     LG_CYRILLIC },
+	{ N_("Cyrillic (IS_O-IR-111)"),             "ISO-IR-111",	     LG_CYRILLIC },
+	{ N_("Cyrillic (_KOI8-R)"),                 "KOI8-R",                LG_CYRILLIC },
+	{ N_("Cyrillic (_MacCyrillic)"),            "x-mac-cyrillic",        LG_CYRILLIC },
+	{ N_("Cyrillic (_Windows-1251)"),           "windows-1251",          LG_CYRILLIC },
+	{ N_("Cyrillic/Russian (_CP-866)"),         "IBM866",                LG_CYRILLIC },
+	{ N_("Cyrillic/Ukrainian (_KOI8-U)"),       "KOI8-U",                LG_CYRILLIC },
+	{ N_("Cyrillic/Ukrainian (Mac_Ukrainian)"), "x-mac-ukrainian",       LG_CYRILLIC },
+	{ N_("English (_US-ASCII)"),                "us-ascii",              LG_WESTERN },
+	{ N_("_Farsi (MacFarsi)"),                  "x-mac-farsi",           LG_OTHER },
+	{ N_("_Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER },
+	{ N_("Greek (_ISO-8859-7)"),                "ISO-8859-7",            LG_GREEK },
+	{ N_("Greek (_MacGreek)"),                  "x-mac-greek",           LG_GREEK },
+	{ N_("Greek (_Windows-1253)"),              "windows-1253",          LG_GREEK },
+	{ N_("Gujarati (_MacGujarati)"),            "x-mac-gujarati",        LG_INDIAN },
+	{ N_("Gurmukhi (Mac_Gurmukhi)"),            "x-mac-gurmukhi",        LG_INDIAN },
+	{ N_("Hebrew (_IBM-862)"),                  "IBM862",                LG_HEBREW },
+	{ N_("Hebrew (I_SO-8859-8-E)"),             "ISO-8859-8-E",          LG_HEBREW },
+	{ N_("Hebrew (IS_O-8859-8-I)"),             "ISO-8859-8-I",          LG_HEBREW },
+	{ N_("Hebrew (_MacHebrew)"),                "x-mac-hebrew",          LG_HEBREW },
+	{ N_("Hebrew (_Windows-1255)"),             "windows-1255",          LG_HEBREW },
+	{ N_("Hindi (Mac_Devanagari)"),             "x-mac-devanagari",      LG_INDIAN },
+	{ N_("_Icelandic (MacIcelandic)"),          "x-mac-icelandic",       LG_OTHER },
+	{ N_("Japanese (_EUC-JP)"),                 "EUC-JP",                LG_JAPANESE },
+	{ N_("Japanese (_ISO-2022-JP)"),            "ISO-2022-JP",           LG_JAPANESE },
+	{ N_("Japanese (_Shift_JIS)"),              "Shift_JIS",             LG_JAPANESE },
+	{ N_("Korean (_EUC-KR)"),                   "EUC-KR",                LG_KOREAN },
+	{ N_("Korean (_ISO-2022-KR)"),              "ISO-2022-KR",           LG_KOREAN },
+	{ N_("Korean (_JOHAB)"),                    "x-johab",               LG_KOREAN },
+	{ N_("Korean (_UHC)"),                      "x-windows-949",         LG_KOREAN },
+	{ N_("_Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER },
+	{ N_("_Romanian (_MacRomanian)"),            "x-mac-romanian",        LG_OTHER },
+	{ N_("R_omanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER },
+	{ N_("South European (_ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER },
+	{ N_("_Thai (TIS-620)"),                    "TIS-620",               LG_OTHER },
+	{ N_("Turkish (_IBM-857)"),                 "IBM857",                LG_TURKISH },
+	{ N_("Turkish (I_SO-8859-9)"),              "ISO-8859-9",            LG_TURKISH },
+	{ N_("Turkish (_MacTurkish)"),              "x-mac-turkish",         LG_TURKISH },
+	{ N_("Turkish (_Windows-1254)"),            "windows-1254",          LG_TURKISH },
+	{ N_("Unicode (UTF-_7)"),                   "UTF-7",                 LG_UNICODE },
+	{ N_("Unicode (UTF-_8)"),                   "UTF-8",                 LG_UNICODE },
+	{ N_("Unicode (UTF-1_6BE)"),                "UTF-16BE",              LG_UNICODE },
+	{ N_("Unicode (UTF-16_LE)"),                "UTF-16LE",              LG_UNICODE },
+	{ N_("Unicode (UTF-3_2BE)"),                "UTF-32BE",              LG_UNICODE },
+	{ N_("Unicode (UTF-32L_E)"),                "UTF-32LE",              LG_UNICODE },
+	{ N_("_User Defined"),                      "x-user-defined",        LG_OTHER },
+	{ N_("Vietnamese (_TCVN)"),                 "x-viet-tcvn5712",       LG_VIETNAMESE },
+	{ N_("Vietnamese (_VISCII)"),               "VISCII",                LG_VIETNAMESE },
+	{ N_("Vietnamese (V_PS)"),                  "x-viet-vps",            LG_VIETNAMESE },
+	{ N_("Vietnamese (_Windows-1258)"),         "windows-1258",          LG_VIETNAMESE },
+	{ N_("_Visual Hebrew (ISO-8859-8)"),        "ISO-8859-8",            LG_HEBREW },
+	{ N_("Western (_IBM-850)"),                 "IBM850",                LG_WESTERN },
+	{ N_("Western (I_SO-8859-1)"),              "ISO-8859-1",            LG_WESTERN },
+	{ N_("Western (IS_O-8859-15)"),             "ISO-8859-15",           LG_WESTERN },
+	{ N_("Western (_MacRoman)"),                "x-mac-roman",           LG_WESTERN },
+	{ N_("Western (_Windows-1252)"),            "windows-1252",          LG_WESTERN },
+	/* encodings without translatable names */
+	{ "T._61-8bit",                             "T.61-8bit",             LG_OTHER },
+	{ "x-_imap4-modified-utf7",                 "x-imap4-modified-utf7", LG_UNICODE },
+	{ "x-u-_escaped",                           "x-u-escaped",           LG_OTHER }
+};
+static const guint n_encodings = G_N_ELEMENTS (encodings);
+
 static void
 mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass);
 static void
@@ -74,12 +210,13 @@ static gresult
 impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 			  const char* url);
 static gresult           
-impl_get_charset_titles (EphyEmbedSingle *shell,
-			 const char *group,
-			 GList **charsets);
+impl_get_encodings (EphyEmbedSingle *shell,
+		    LanguageGroup group,
+		    gboolean elide_underscores,
+		    GList **encodings_list);
 static gresult           
-impl_get_charset_groups (EphyEmbedSingle *shell,
-		         GList **groups);
+impl_get_language_groups (EphyEmbedSingle *shell,
+		          GList **groups);
 static gresult
 impl_get_font_list (EphyEmbedSingle *shell,
 		    const char *langGroup,
@@ -108,7 +245,6 @@ impl_show_file_picker (EphyEmbedSingle *shell,
 		       const char *file, 
 		       FilePickerMode mode,
                        char **ret_fullpath, 
-		       gboolean *ret_save_content, 
                        FileFormat *file_formats, 
 		       int *ret_file_format);
 
@@ -117,11 +253,16 @@ static void mozilla_embed_single_new_window_orphan_cb (GtkMozEmbedSingle *embed,
 					              guint chrome_mask,
                            		              EphyEmbedSingle *shell);
 
+#ifdef GNOME_ENABLE_DEBUG
+static gresult control_encodings_list (void);
+#endif
+
 struct MozillaEmbedSinglePrivate
 {
 	char *user_prefs;
-	GHashTable *charsets_hash;
-	GList *sorted_charsets_titles;
+	
+	/* monitor this widget for theme changes*/
+	GtkWidget *theme_window;
 };
 
 static NS_DEFINE_CID(kJVMManagerCID, NS_JVMMANAGER_CID);
@@ -170,8 +311,8 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 	shell_class->clear_cache = impl_clear_cache;
 	shell_class->set_offline_mode = impl_set_offline_mode;
 	shell_class->load_proxy_autoconf = impl_load_proxy_autoconf;
-	shell_class->get_charset_titles = impl_get_charset_titles;
-	shell_class->get_charset_groups = impl_get_charset_groups;
+	shell_class->get_encodings = impl_get_encodings;
+	shell_class->get_language_groups = impl_get_language_groups;
 	shell_class->get_font_list = impl_get_font_list;
 	shell_class->list_cookies = impl_list_cookies;
 	shell_class->remove_cookies = impl_remove_cookies;
@@ -181,78 +322,140 @@ mozilla_embed_single_class_init (MozillaEmbedSingleClass *klass)
 }
 
 static void
-mozilla_load_proxy_prefs (MozillaEmbedSingle *shell)
+mozilla_set_default_prefs (MozillaEmbedSingle *mes)
 {
-	char *tmp;
-	int i, mozilla_mode = 0;
-	
-	/* Proxy mode */
-	tmp = eel_gconf_get_string (CONF_NETWORK_PROXY_MODE);
-	g_return_if_fail (tmp != NULL);
-	
-	if (strcmp (tmp, "manual") == 0)
-	{
-		mozilla_mode = 1;
-	}
-	else if (strcmp (tmp, "auto") == 0)
-	{
-		mozilla_mode = 2;
-	}
+	nsCOMPtr<nsIPrefService> prefService;
 
-	mozilla_prefs_set_int ("network.proxy.type", mozilla_mode);
-	g_free (tmp);
+        prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
+	g_return_if_fail (prefService != NULL);
 
-	/* Http proxy */
-	tmp = eel_gconf_get_string (CONF_NETWORK_HTTP_PROXY);
-	g_return_if_fail (tmp != NULL);
-	mozilla_prefs_set_string ("network.proxy.http", tmp);
-	g_free (tmp);
+        nsCOMPtr<nsIPrefBranch> pref;
+        prefService->GetBranch ("", getter_AddRefs(pref));
+	g_return_if_fail (pref != NULL);
 
-	i = eel_gconf_get_integer (CONF_NETWORK_HTTP_PROXY_PORT);
-	mozilla_prefs_set_int ("network.proxy.http_port", i);
-	
-	/* Ftp proxy */
-	tmp = eel_gconf_get_string (CONF_NETWORK_FTP_PROXY);
-	g_return_if_fail (tmp != NULL);
-	mozilla_prefs_set_string ("network.proxy.ftp", tmp);
-	g_free (tmp);
+	/* Don't allow mozilla to raise window when setting focus (work around bugs) */
+	pref->SetBoolPref ("mozilla.widget.raise-on-setfocus", PR_FALSE);
 
-	i = eel_gconf_get_integer (CONF_NETWORK_FTP_PROXY_PORT);
-	mozilla_prefs_set_int ("network.proxy.ftp_port", i);
+	/* set default search engine */
+	pref->SetCharPref ("keyword.URL", "http://www.google.com/search?btnI=I%27m+Feeling+Lucky&q=");
+	pref->SetBoolPref ("keyword.enabled", PR_TRUE);
+	pref->SetBoolPref ("security.checkloaduri", PR_FALSE);
 
-	/* Secure proxy */
-	tmp = eel_gconf_get_string (CONF_NETWORK_SSL_PROXY);
-	g_return_if_fail (tmp != NULL);
-	mozilla_prefs_set_string ("network.proxy.ssl", tmp);
-	g_free (tmp);
+	/* dont allow xpi installs from epiphany, there are crashes */
+	pref->SetBoolPref ("xpinstall.enabled", PR_FALSE);
 
-	i = eel_gconf_get_integer (CONF_NETWORK_SSL_PROXY_PORT);
-	mozilla_prefs_set_int ("network.proxy.ssl_port", i);
+	/* deactivate mailcap and mime.types support */
+	pref->SetCharPref ("helpers.global_mailcap_file", "");
+	pref->SetCharPref ("helpers.global_mime_types_file", "");
+	pref->SetCharPref ("helpers.private_mailcap_file", "");
+	pref->SetCharPref ("helpers.private_mime_types_file", "");
 
-	/* Socks proxy */
-	tmp = eel_gconf_get_string (CONF_NETWORK_SOCKS_PROXY);
-	g_return_if_fail (tmp != NULL);
-	mozilla_prefs_set_string ("network.proxy.socks", tmp);
-	g_free (tmp);
+	/* disable sucky XUL ftp view, have nice ns4-like html page instead */
+	pref->SetBoolPref ("network.dir.generate_html", PR_TRUE);
 
-	i = eel_gconf_get_integer (CONF_NETWORK_SOCKS_PROXY_PORT);
-	mozilla_prefs_set_int ("network.proxy.socks_port", i);
+	/* disable usless security warnings */
+	pref->SetBoolPref ("security.warn_entering_secure", PR_FALSE);
+	pref->SetBoolPref ("security.warn_leaving_secure", PR_FALSE);
+	pref->SetBoolPref ("security.warn_submit_insecure", PR_FALSE);
 
-	/* Autoconfiguration */
-	tmp = eel_gconf_get_string (CONF_NETWORK_PROXY_AUTO_URL);
-	g_return_if_fail (tmp != NULL);
-	ephy_embed_single_load_proxy_autoconf 
-		(EPHY_EMBED_SINGLE (shell), tmp);
-	g_free (tmp);
+	/* FIXME: We should change this back to true when mozilla bugs
+         * 207000 and 207001 are fixed.
+	 */
+	/* Make sure this is off, we monitor for theme changes and set colors. */
+	pref->SetBoolPref ("browser.display.use_system_colors", PR_FALSE);
+
+	/* Smooth scrolling on */
+	pref->SetBoolPref ("general.smoothScroll", PR_TRUE);
+
+	/* Disable blinking text and marquee, its non-standard and annoying */
+	pref->SetBoolPref ("browser.blink_allowed", PR_FALSE);
+	pref->SetBoolPref ("browser.display.enable_marquee", PR_FALSE);
+
+	/* FIXME: We disable this for now, because enough people complained
+	 * and there are some moz bugs. We either need a user visible pref
+	 * or need to revert to enabling by default for a11y
+         */
+	/* Enable Browsing with the Caret */
+	/*pref->SetBoolPref ("accessibility.browsewithcaret", PR_TRUE);*/
+
+	/* Don't Fetch the Sidebar whats related information, since we don't use it */
+	pref->SetBoolPref ("browser.related.enabled", PR_FALSE);
+
+	/* Line Wrap View->Source */
+	pref->SetBoolPref ("view_source.wrap_long_lines", PR_TRUE);
+
+	/* CTRL-Mousewheel scrolls by one page */
+	pref->SetIntPref ("mousewheel.withcontrolkey.action", 1);
+	pref->SetIntPref ("mousewheel.withcontrolkey.numlines", 1);
+	pref->SetBoolPref ("mousewheel.withcontrolkey.sysnumlines", PR_FALSE);
+
+	/* Enable Image Auto-Resizing */
+	pref->SetBoolPref ("browser.enable_automatic_image_resizing", PR_TRUE);
+}
+
+static char *
+color_to_string (GdkColor color)
+{
+	return g_strdup_printf ("#%.2x%.2x%.2x",
+			       color.red >> 8,
+			       color.green >> 8,
+			       color.blue >> 8);
 }
 
 static void
-mozilla_set_default_prefs (MozillaEmbedSingle *mes)
+mozilla_update_colors (GtkWidget *window,
+                       GtkStyle *previous_style,
+                       MozillaEmbedSingle *mes)
 {
-	mozilla_prefs_load (ephy_file ("default-prefs.js"));
-	mozilla_prefs_save (mes->priv->user_prefs);
+	nsCOMPtr<nsIPrefService> prefService;
+	GdkColor color;
+	char *str;
+
+        prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
+	g_return_if_fail (prefService != NULL);
+
+        nsCOMPtr<nsIPrefBranch> pref;
+        prefService->GetBranch ("", getter_AddRefs(pref));
+	g_return_if_fail (pref != NULL);
+
+	/* Set the bg color to the text bg color*/
+	color = window->style->base[GTK_STATE_NORMAL];
+	str = color_to_string (color);
+	pref->SetCharPref ("browser.display.background_color", str);		
+	g_free (str);
+
+	/* Set the text color */
+	color = window->style->text[GTK_STATE_NORMAL];
+	str = color_to_string (color);	
+	pref->SetCharPref ("browser.display.foreground_color", str);	
+	g_free (str);
+
+	/* FIXME: We should probably monitor and set link color here too,
+	 * but i'm not really sure what to do about that yet
+	 */
 }
 
+static void
+mozilla_setup_colors (MozillaEmbedSingle *mes)
+{
+	GtkWidget *window;
+
+	/* Create a random window to monitor for theme changes */
+	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	mes->priv->theme_window = window;
+	gtk_widget_realize (window);
+	gtk_widget_ensure_style (window);
+
+	/* monitor theme changes*/
+	g_signal_connect (G_OBJECT (window), "style-set",
+			  G_CALLBACK (mozilla_update_colors), mes);
+	
+	/* Initialize the colors */
+	mozilla_update_colors (window, NULL, mes);
+
+	mes->priv->theme_window = window;
+}			
+
 static void
 mozilla_init_single (MozillaEmbedSingle *mes)
 {	
@@ -291,41 +494,6 @@ mozilla_init_profile (void)
         g_free (profile_path);
 }
 
-static gboolean
-is_new_build (MozillaEmbedSingle *mes)
-{
-	gboolean new_build = FALSE;
-	char *build_test;
-	char *prefs_file;
-	
-	prefs_file = g_build_filename (ephy_dot_dir (), 
-				       MOZILLA_PROFILE_DIR,
-				       MOZILLA_PROFILE_NAME,
-				       MOZILLA_PROFILE_FILE,
-				       NULL);
-
-	/* no mozilla prefs ? or new epiphany build */
-        build_test = eel_gconf_get_string ("/apps/epiphany/gconf_test");
-        if (!g_file_test(mes->priv->user_prefs, G_FILE_TEST_EXISTS) || 
-            build_test == NULL ||
-            strncmp (build_test, __TIME__, 8) != 0)
-        {
-                new_build = TRUE;
-                eel_gconf_set_string ("/apps/epiphany/gconf_test", __TIME__);
-        }
-	
-        g_free (build_test);
-
-	return new_build;
-}
-
-static void
-mozilla_init_prefs (MozillaEmbedSingle *mes)
-{
-	mozilla_set_default_prefs (mes);
-	mozilla_notifiers_set_defaults ();
-}
-
 static gboolean
 have_gnome_url_handler (const gchar *protocol)
 {
@@ -337,7 +505,7 @@ have_gnome_url_handler (const gchar *protocol)
 	cmd = eel_gconf_get_string (key);
 	g_free (key);
 
-	rv = (cmd != NULL);
+	rv = (cmd != NULL && strstr (cmd, "epiphany") == NULL);
 	g_free (cmd);
 
 	if (!rv) return rv;
@@ -369,11 +537,7 @@ mozilla_register_external_protocols (void)
 static void
 mozilla_embed_single_init (MozillaEmbedSingle *mes)
 {
-	gboolean new_build;
- 
  	mes->priv = g_new0 (MozillaEmbedSinglePrivate, 1);
-	mes->priv->charsets_hash = NULL;
-	mes->priv->sorted_charsets_titles = NULL;
 
 	mes->priv->user_prefs =
 		g_build_filename (ephy_dot_dir (), 
@@ -382,23 +546,23 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 				  MOZILLA_PROFILE_FILE,
 				  NULL);
 
-	new_build = is_new_build (mes);
-	
 	/* Pre initialization */
-	mozilla_notifiers_init (EPHY_EMBED_SINGLE (mes));
 	mozilla_init_home ();
 	mozilla_init_profile ();
 	
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
 
-	/* Post initialization */
-	if (new_build)
-	{
-		mozilla_init_prefs (mes);
-	}
+	mozilla_set_default_prefs (mes);
 
-	mozilla_load_proxy_prefs (mes);
+	/* FIXME: This should be removed when mozilla
+	 * bugs 207000 and 207001 are fixed.
+	 */
+	mozilla_setup_colors (mes);
+
+	START_PROFILER ("Mozilla prefs notifiers")
+	mozilla_notifiers_init (EPHY_EMBED_SINGLE (mes));
+	STOP_PROFILER ("Mozilla prefs notifiers")
 
 	mozilla_init_single (mes);
 	
@@ -406,6 +570,9 @@ mozilla_embed_single_init (MozillaEmbedSingle *mes)
 
 	mozilla_register_external_protocols ();
 
+#ifdef GNOME_ENABLE_DEBUG
+	control_encodings_list ();
+#endif
 	/* FIXME alert if fails */
 }
 
@@ -472,11 +639,11 @@ mozilla_embed_single_finalize (GObject *object)
 
 	mozilla_notifiers_free ();
 
-	mozilla_prefs_save (mes->priv->user_prefs);
-	
 	gtk_moz_embed_pop_startup ();
 
 	g_free (mes->priv->user_prefs);
+
+	gtk_widget_destroy (mes->priv->theme_window);
 	
         g_free (mes->priv);
 }
@@ -529,14 +696,15 @@ impl_load_proxy_autoconf (EphyEmbedSingle *shell,
 	return G_OK;
 }
 
+#ifdef GNOME_ENABLE_DEBUG
 static gresult
-fill_charsets_lists (MozillaEmbedSinglePrivate *priv)
+control_encodings_list (void)
 {
 	nsresult rv;
 	char *tmp;
         PRUint32 cscount;
-        PRUint32 translated_cscount = get_translated_cscount ();
-        char *charset_str, *charset_title_str;
+        char *encoding_str, *encoding_title_str;
+	gresult ret = G_OK;
 
         nsCOMPtr<nsIAtom> docCharsetAtom;
         nsCOMPtr<nsICharsetConverterManager2> ccm2 =
@@ -548,121 +716,151 @@ fill_charsets_lists (MozillaEmbedSinglePrivate *priv)
         if (!NS_SUCCEEDED(rv)) return G_FAILED;
 
         rv = cs_list->Count(&cscount);
-        priv->charsets_hash = g_hash_table_new (g_str_hash, g_str_equal);
 	for (PRUint32 i = 0; i < cscount; i++)
         {
                 nsCOMPtr<nsISupports> cssupports =
                                         (dont_AddRef)(cs_list->ElementAt(i));
                 nsCOMPtr<nsIAtom> csatom ( do_QueryInterface(cssupports) );
-                nsAutoString charset_ns, charset_title_ns;
+                nsAutoString encoding_ns, encoding_title_ns;
 
-                /* charset name */
-		rv = csatom->ToString(charset_ns);
-                tmp = ToNewCString (charset_ns);
+                /* encoding name */
+		rv = csatom->ToString(encoding_ns);
+                tmp = ToNewCString (encoding_ns);
                 if (tmp == NULL || strlen (tmp) == 0)
                 {
                         continue;
                 }
-		charset_str = g_strdup (tmp);
+		encoding_str = g_strdup (tmp);
 		nsMemory::Free (tmp);
 		tmp = nsnull;
 
-                /* charset readable title */
-                rv = ccm2->GetCharsetTitle2(csatom, &charset_title_ns);
-                tmp = ToNewCString (charset_title_ns);
+                /* encoding readable title */
+                rv = ccm2->GetCharsetTitle2(csatom, &encoding_title_ns);
+                tmp = ToNewCString (encoding_title_ns);
                 if (tmp == NULL || 
                     strlen (tmp) == 0)
                 {
-                        if (tmp) nsMemory::Free (tmp);
-                        charset_title_str = g_strdup (charset_str);
+                        encoding_title_str = g_strdup (encoding_str);
                 }
 		else
 		{
-			charset_title_str = g_strdup (tmp);
-			nsMemory::Free (tmp);
-			tmp = nsnull;
+			encoding_title_str = g_strdup (tmp);
 		}
 
-		for (PRUint32 j = 0; j < translated_cscount; j++)
+		if (tmp) nsMemory::Free (tmp);
+		tmp = nsnull;
+
+		gboolean found = FALSE;
+		for (PRUint32 j = 0; j < n_encodings; j++)
                 {
                         if (g_ascii_strcasecmp (
-                                charset_str, 
-                                charset_trans_array[j].charset_name) == 0)
+                                encoding_str, 
+                                encodings[j].name) == 0)
                         {
-                                g_free (charset_title_str);
-                                charset_title_str = (char *) 
-                                        _(charset_trans_array[j].charset_title);
+				LOG ("Mozilla reported encoding %s with title %s found in our list.",
+				     encoding_str, encoding_title_str)
+
+				found = TRUE;
                                 break;
                         }
                 }
 
-		/* fill the hash and the sorted list */
-		g_hash_table_insert (priv->charsets_hash, charset_title_str, charset_str);
-                priv->sorted_charsets_titles = 
-			g_list_insert_sorted (priv->sorted_charsets_titles,
-                                              (gpointer)charset_title_str,
-                                              (GCompareFunc)g_ascii_strcasecmp); 
+		if (found == FALSE)
+		{
+			g_warning ("Mozilla reported encoding %s with title %s NOT found in our list!",
+				   encoding_str, encoding_title_str);
+			ret = G_FAILED;
+		}
         }
 
-	return G_OK;
+	return ret;
 }
+#endif
 
-static void
-ensure_charsets_tables (MozillaEmbedSingle *shell)
+static gint
+encoding_info_cmp (const EncodingInfo *i1, const EncodingInfo *i2)
 {
-	if (!shell->priv->charsets_hash)
-	{
-		fill_charsets_lists (shell->priv);
-	}
+	return strcmp (i1->key, i2->key);
 }
 
 static gresult           
-impl_get_charset_titles (EphyEmbedSingle *shell,
-		         const char *group,
-			 GList **charsets)
+impl_get_encodings (EphyEmbedSingle *shell,
+		   LanguageGroup group,
+		   gboolean elide_underscores,
+		   GList **encodings_list)
 {
-	MozillaEmbedSingle *mshell = MOZILLA_EMBED_SINGLE(shell);
-	int count = get_translated_cscount ();
 	GList *l = NULL;
-	int j;
+	guint i;
 
-	ensure_charsets_tables (mshell);
-	g_return_val_if_fail (mshell->priv->charsets_hash != NULL, G_FAILED);
-
-	for (j = 0; j < count; j++)
-	{	
-		if (group == NULL ||
-		    strcmp (group, lgroups[charset_trans_array[j].lgroup]) == 0) 
+	for (i = 0; i < n_encodings; i++)
+	{
+		if (group == LG_ALL || group == encodings[i].group)
 		{
-			CharsetInfo *info;
-			info = g_new0 (CharsetInfo, 1);
-			info->name = charset_trans_array[j].charset_name;
-			info->title = charset_trans_array[j].charset_title;
-			l = g_list_append (l, info);
+			EncodingInfo *info;
+			gchar *elided = NULL;
 
-			/* FIXME check that the encoding exists in mozilla before
-			 * adding it */
+			info = g_new0 (EncodingInfo, 1);
+
+			info->encoding = g_strdup (encodings[i].name);
+			
+			elided = ephy_str_elide_underscores (_(encodings[i].title));
+
+			if (elide_underscores)
+			{
+				info->title = g_strdup (elided);
+			}
+			else
+			{
+				info->title = g_strdup (_(encodings[i].title));
+			}
+
+			/* collate without underscores */
+			info->key = g_utf8_collate_key (elided, -1);
+
+			info->group = encodings[i].group;
+
+			l = g_list_prepend (l, info);
+			g_free (elided);
 		}
 	}
 
-	*charsets = l;
+	*encodings_list = g_list_sort (l, (GCompareFunc) encoding_info_cmp);
 
 	return G_OK;
 }
 
+static gint
+language_group_info_cmp (const LanguageGroupInfo *i1, const LanguageGroupInfo *i2)
+{
+	return strcmp (i1->key, i2->key);
+}
+
 static gresult           
-impl_get_charset_groups (EphyEmbedSingle *shell,
-		         GList **groups)
+impl_get_language_groups (EphyEmbedSingle *shell,
+		          GList **groups)
 {
 	GList *l = NULL;
-	int i;
+	guint i;
 	
-	for (i = 0; lgroups[i] != NULL; i++)
+	for (i = 0; i < n_lang_groups; i++)
 	{
-		l = g_list_append (l, (gpointer)lgroups[i]);
+		LanguageGroupInfo *info;
+		gchar *elided = NULL;
+		
+		info = g_new0 (LanguageGroupInfo, 1);
+		
+		info->title = g_strdup (_(lang_groups[i].title));
+		info->group = lang_groups[i].group;
+		
+		/* collate without underscores */
+		elided = ephy_str_elide_underscores (info->title);
+		info->key = g_utf8_collate_key (elided, -1);
+		g_free (elided);
+
+		l = g_list_prepend (l, info);
 	}
 	
-	*groups = l;
+	*groups = g_list_sort (l, (GCompareFunc) language_group_info_cmp);
 	
 	return G_OK;
 }
@@ -701,17 +899,28 @@ impl_get_font_list (EphyEmbedSingle *shell,
 
 		char *gFontString;
 		gFontString = g_strdup(NS_ConvertUCS2toUTF8(fontString).get());
-		l = g_list_append(l, gFontString);
+		l = g_list_prepend (l, gFontString);
 	}
-	*fontList = l;
+	*fontList = g_list_reverse (l);
 
 	if (default_font != NULL)
 	{
-		char key [255];
+		char key[255];
+		char *value = NULL;
+		nsCOMPtr<nsIPrefService> prefService;
+
+	        prefService = do_GetService (NS_PREFSERVICE_CONTRACTID);
+		g_return_val_if_fail (prefService != NULL, G_FAILED);
+	
+	        nsCOMPtr<nsIPrefBranch> pref;
+	        prefService->GetBranch ("", getter_AddRefs(pref));
+		g_return_val_if_fail (pref != NULL, G_FAILED);
 
 		sprintf (key, "font.name.%s.%s", fontType, langGroup);
 		
-		*default_font = mozilla_prefs_get_string (key);
+		pref->GetCharPref (key, &value);
+		*default_font = g_strdup (value);
+		nsMemory::Free (value);
 	}
 
 	return G_OK;
@@ -887,20 +1096,13 @@ impl_show_file_picker (EphyEmbedSingle *shell,
 		       const char *file, 
 		       FilePickerMode mode,
                        char **ret_fullpath, 
-		       gboolean *ret_save_content, 
                        FileFormat *file_formats, 
 		       int *ret_file_format)
 {
-	PRBool showContentCheck;
-        gchar *expanded_directory;
+	gchar *expanded_directory;
+	gresult result;
 
-        if (ret_save_content == NULL)
-                showContentCheck = PR_FALSE;
-        else
-                showContentCheck = PR_TRUE;
-
-        GFilePicker *filePicker = new GFilePicker (showContentCheck, 
-                                                   file_formats);
+        GFilePicker *filePicker = new GFilePicker (file_formats);
 
 	/* FIXME sane path: expand tilde ... */
         expanded_directory = g_strdup (directory);
@@ -926,31 +1128,20 @@ impl_show_file_picker (EphyEmbedSingle *shell,
         PRInt16 retval;
         filePicker->Show (&retval);
 
-        if (ret_save_content != NULL)
-        {
-                if (retval == GFilePicker::returnOKSaveContent)
-                        *ret_save_content = TRUE;
-                else
-                        *ret_save_content = FALSE;
-        }
         if (ret_file_format != NULL)
         {
                 *ret_file_format = filePicker->mSelectedFileFormat;
         }
 
-        if (retval == nsIFilePicker::returnCancel)
-        {
-                delete filePicker;
-                return G_FAILED;
-        }
-        else
-        {
-                nsCOMPtr<nsILocalFile> file;
-                filePicker->GetFile (getter_AddRefs(file));
-		nsCAutoString tempFullPathStr;
-                file->GetNativePath (tempFullPathStr);
-                *ret_fullpath = g_strdup (tempFullPathStr.get());
-                delete filePicker;
-                return G_OK;
-        }
+        nsCOMPtr<nsILocalFile> local_file;
+	filePicker->GetFile (getter_AddRefs(local_file));
+	nsCAutoString tempFullPathStr;
+	local_file->GetNativePath (tempFullPathStr);
+	*ret_fullpath = g_strdup (tempFullPathStr.get());
+
+        result = (retval == nsIFilePicker::returnCancel) ? G_FAILED : G_OK;
+
+	delete filePicker;
+
+	return result;
 }
