@@ -25,6 +25,7 @@
 #include "MozillaPrivate.h"
 #include "EphyWrapper.h"
 #include "EventContext.h"
+#include "ephy-debug.h"
 
 #include <nsIWindowWatcher.h>
 #include <nsIURI.h>
@@ -37,8 +38,6 @@
 #include <nsIPrintOptions.h>
 #include <nsGfxCIID.h>
 
-#include <math.h>
-
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass);
 static void
@@ -104,11 +103,11 @@ impl_copy_page (EphyEmbed *dest,
 		EmbedDisplayType display_type);
 static gresult
 impl_zoom_set (EphyEmbed *embed, 
-               int zoom, 
+               float zoom, 
                gboolean reflow);
 static gresult
 impl_zoom_get (EphyEmbed *embed,
-               int *zoom);
+               float *zoom);
 static gresult 
 impl_selection_can_cut (EphyEmbed *embed);
 static gresult 
@@ -150,8 +149,8 @@ impl_find (EphyEmbed *embed,
            EmbedFindInfo *info);
 
 static gresult
-impl_set_charset (EphyEmbed *embed,
-                  const char *charset);
+impl_set_encoding (EphyEmbed *embed,
+                   const char *encoding);
 
 static gresult
 impl_print (EphyEmbed *embed, 
@@ -328,7 +327,7 @@ ephy_embed_init (EphyEmbedClass *embed_class)
 	embed_class->shistory_copy = impl_shistory_copy;
 	embed_class->get_security_level = impl_get_security_level;
 	embed_class->find = impl_find;
-	embed_class->set_charset = impl_set_charset;
+	embed_class->set_encoding = impl_set_encoding;
 	embed_class->select_all = impl_select_all;
 	embed_class->print = impl_print;
 	embed_class->print_preview_close = impl_print_preview_close;
@@ -438,7 +437,7 @@ impl_get_capabilities (EphyEmbed *embed,
 	        EMBED_PRINT_CAP |
 	        EMBED_FIND_CAP |
 	        EMBED_SECURITY_CAP |
-	        EMBED_CHARSET_CAP |
+	        EMBED_ENCODING_CAP |
 	        EMBED_SHISTORY_CAP );
 	
 	*caps = mozilla_caps;
@@ -772,7 +771,7 @@ impl_copy_page (EphyEmbed *dest,
 
 static gresult
 impl_zoom_set (EphyEmbed *embed, 
-               int zoom, 
+               float zoom, 
                gboolean reflow)
 {
 	EphyWrapper *wrapper;
@@ -781,7 +780,7 @@ impl_zoom_set (EphyEmbed *embed,
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
-	result = wrapper->SetZoom ((float)(zoom) / 100, reflow);
+	result = wrapper->SetZoom (zoom, reflow);
 
 	if (NS_SUCCEEDED (result))
 	{
@@ -793,7 +792,7 @@ impl_zoom_set (EphyEmbed *embed,
 
 static gresult
 impl_zoom_get (EphyEmbed *embed,
-               int *zoom)
+               float *zoom)
 {
 	float f;
 	EphyWrapper *wrapper;
@@ -801,15 +800,17 @@ impl_zoom_get (EphyEmbed *embed,
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	if (!wrapper)
 	{
-		*zoom = 100;
-		return G_OK;
+		LOG ("impl_zoom_get: wrapper == NULL")
+		
+		*zoom = 1.0;
+		return G_FAILED;
 	}
 	
 	nsresult result = wrapper->GetZoom (&f);
 	
 	if (NS_SUCCEEDED (result))
 	{
-		*zoom = (int) rint (f * 100);
+		*zoom = f;
 
 		return G_OK;
 	}
@@ -1154,8 +1155,8 @@ impl_find (EphyEmbed *embed,
 }
 
 static gresult
-impl_set_charset (EphyEmbed *embed,
-                  const char *charset)
+impl_set_encoding (EphyEmbed *embed,
+		   const char *encoding)
 {
 	nsresult result = NS_OK;
 	EphyWrapper *wrapper;
@@ -1163,7 +1164,7 @@ impl_set_charset (EphyEmbed *embed,
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
-	result = wrapper->ForceCharacterSet (charset);
+	result = wrapper->ForceEncoding (encoding);
 	if (NS_FAILED (result)) return G_FAILED;
 	
 	gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
@@ -1318,8 +1319,9 @@ mozilla_embed_destroy_brsr_cb (GtkMozEmbed *embed,
 }
 
 static gint
-mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, 
-				  MozillaEmbed *membed)
+mozilla_embed_emit_mouse_signal (MozillaEmbed *membed,
+				 gpointer dom_event,
+				 const char *signal_name)
 {
 	EphyEmbedEvent *info;
 	EventContext event_context;
@@ -1332,40 +1334,6 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
-
-        nsCOMPtr<nsIDOMDocument> domDoc;
-        result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
-        if (NS_SUCCEEDED(result))
-        {
-                result = wrapper->PushTargetDocument (domDoc);
-        }
-
-	g_signal_emit_by_name (membed, "ge_dom_mouse_click", info, &return_value); 
-	
-	wrapper->PopTargetDocument ();
-	
-	g_object_unref (info);
-	
-	return return_value;
-}
-
-static gint
-mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
-				 MozillaEmbed *membed)
-{
-	EphyEmbedEvent *info;
-	EventContext event_context;
-	gint return_value = 0;
-	EphyWrapper *wrapper;
-	nsresult result;
-	
-	info = ephy_embed_event_new ();
-	
-	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-	
 	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
         result = event_context.GetMouseEventInfo (info);
 	if (NS_SUCCEEDED(result))
@@ -1377,19 +1345,35 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 			result = wrapper->PushTargetDocument (domDoc);
 			if (NS_SUCCEEDED(result))
 			{
-				g_signal_emit_by_name (membed, "ge_dom_mouse_down", 
+				g_signal_emit_by_name (membed, signal_name, 
 						       info, &return_value); 
 				wrapper->PopTargetDocument ();
 			}
 		}
 
 	}
-	
+
 	g_object_unref (info);
 	
 	return return_value;
 }
 
+static gint
+mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, 
+				  MozillaEmbed *membed)
+{
+	return mozilla_embed_emit_mouse_signal
+		(membed, dom_event, "ge_dom_mouse_click");
+}
+
+static gint
+mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
+				 MozillaEmbed *membed)
+{
+	return mozilla_embed_emit_mouse_signal
+		(membed, dom_event, "ge_dom_mouse_down");
+}
+
 static void
 mozilla_embed_size_to_cb (GtkMozEmbed *embed, gint width, gint height, 
 			  MozillaEmbed *membed)
