@@ -16,17 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
 #include "mozilla-notifiers.h"
 #include "eel-gconf-extensions.h"
-#include "mozilla-prefs.h"
 #include "MozRegisterComponents.h"
 #include "ephy-prefs.h"
 #include "ephy-embed-prefs.h"
-#include "mozilla-i18n.h"
+#include "ephy-langs.h"
+#include "ephy-debug.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -35,24 +37,22 @@
 #include <stdlib.h>
 #include <sys/utsname.h>
 #include "nsBuildID.h"
+#include "nsCOMPtr.h"
+#include "nsIPrefService.h"
+#include "nsIServiceManager.h"
 
 static void
 mozilla_own_colors_notifier(GConfClient *client,
 			    guint cnxn_id,
 			    GConfEntry *entry,
-			    gpointer user_data);
+			    EphyEmbedSingle *single);
 static void
 mozilla_own_fonts_notifier(GConfClient *client,
 			   guint cnxn_id,
 			   GConfEntry *entry,
-			   gpointer user_data);
+			   EphyEmbedSingle *single);
 
 static void
-mozilla_animate_notifier(GConfClient *client,
-			 guint cnxn_id,
-			 GConfEntry *entry,
-			 gpointer user_data);
-static void
 generic_mozilla_string_notifier(GConfClient *client,
 				guint cnxn_id,
 				GConfEntry *entry,
@@ -71,53 +71,42 @@ static void
 mozilla_allow_popups_notifier(GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
-			      gpointer user_data);
+			      EphyEmbedSingle *single);
 
 static void
 mozilla_language_notifier(GConfClient *client,
 			  guint cnxn_id,
 			  GConfEntry *entry,
-			  gpointer user_data);
-
-static void
-mozilla_autodetect_charset_notifier(GConfClient *client,
-				    guint cnxn_id,
-				    GConfEntry *entry,
-				    EphyEmbedSingle *single);
+			  EphyEmbedSingle *single);
 
 static void
 mozilla_default_font_notifier(GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
-			      gpointer user_data);
+			      EphyEmbedSingle *single);
 
 static void
 mozilla_proxy_mode_notifier (GConfClient *client,
 			     guint cnxn_id,
 			     GConfEntry *entry,
-			     char *pref);
+			     EphyEmbedSingle *single);
 static void
 mozilla_proxy_autoconfig_notifier (GConfClient *client,
 			           guint cnxn_id,
 			           GConfEntry *entry,
-			           char *pref);
+			           EphyEmbedSingle *single);
 
 static void
 mozilla_user_agent_notifier(GConfClient *client,
 			    guint cnxn_id,
 			    GConfEntry *entry,
-			    gpointer user_data);
+			    EphyEmbedSingle *single);
 
-static void 
-mozilla_default_charset_notifier (GConfClient *client,
-				  guint cnxn_id,
-				  GConfEntry *entry,
-				  EphyEmbedSingle *single);
 static void
 mozilla_socks_version_notifier (GConfClient *client,
 				guint cnxn_id,
 				GConfEntry *entry,
-				gpointer user_data);
+				EphyEmbedSingle *single);
 
 /* Keeps the list of the notifiers we installed for mozilla prefs */
 /* to be able to remove them when exiting */
@@ -139,13 +128,8 @@ static const struct
 }
 conversion_table [] =
 {
-	{ CONF_FILTERING_JAVA_ENABLED, BOOL_PREF, "security.enable_java"},
-	{ CONF_FILTERING_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled"},
-	{ CONF_FILTERING_IMAGE_LOADING_TYPE, INT_PREF, "network.image.imageBehavior"},
-	{ CONF_RENDERING_BG_COLOR, STRING_PREF, "browser.display.background_color"},
-	{ CONF_RENDERING_TEXT_COLOR, STRING_PREF, "browser.display.foreground_color"},
-	{ CONF_RENDERING_UNVISITED_LINKS, STRING_PREF, "browser.anchor_color"},
-	{ CONF_RENDERING_VISITED_LINKS, STRING_PREF, "browser.visited_color"},
+	{ CONF_SECURITY_JAVA_ENABLED, BOOL_PREF, "security.enable_java"},
+	{ CONF_SECURITY_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled"},
 	{ CONF_RENDERING_UNDERLINE_LINKS, BOOL_PREF, "browser.underline_anchors"},
 	{ CONF_NETWORK_PROXY_AUTO_URL, STRING_PREF, "network.proxy.autoconfig_url"},
 	{ CONF_NETWORK_HTTP_PROXY, STRING_PREF, "network.proxy.http"},
@@ -160,14 +144,12 @@ conversion_table [] =
 	{ CONF_NETWORK_MEMORY_CACHE, INT_PREF, "browser.cache.memory.capacity"},
 	{ CONF_NETWORK_DISK_CACHE, INT_PREF, "browser.cache.disk.capacity"},
 	{ CONF_NETWORK_CACHE_COMPARE, INT_PREF, "browser.cache.check_doc_frequency"},
-	{ CONF_PERSISTENT_COOKIE_WARN, BOOL_PREF, "network.cookie.warnAboutCookies"},
-	{ CONF_PERSISTENT_COOKIES_BEHAVIOR, INT_PREF, "network.cookie.cookieBehavior"},
-	{ CONF_PERSISTENT_COOKIE_LIFETIME, BOOL_PREF, "network.cookie.lifetime.enabled"},
-	{ CONF_PERSISTENT_PASSWORDS_SAVE, BOOL_PREF, "signon.rememberSignons"},
-	{ CONF_RENDERING_USE_SYSTEM_COLORS, BOOL_PREF, "browser.display.use_system_colors"},
+	{ CONF_SECURITY_COOKIES_ACCEPT, BOOL_PREF, "network.cookie.warnAboutCookies"},
+	{ CONF_LANGUAGE_DEFAULT_ENCODING, STRING_PREF, "intl.charset.default" },
+	{ CONF_LANGUAGE_AUTODETECT_ENCODING, STRING_PREF, "intl.charset.detector" },
 	{ NULL, 0, NULL }
 };
-
+ 
 static const struct 
 {
 	const char *gconf_key;
@@ -177,20 +159,14 @@ custom_notifiers [] =
 {
 	{ CONF_NETWORK_USER_AGENT, 
 	  (GConfClientNotifyFunc) mozilla_user_agent_notifier },
-	{ CONF_FILTERING_ANIMATE_TYPE, 
-	  (GConfClientNotifyFunc) mozilla_animate_notifier },
 	{ CONF_RENDERING_USE_OWN_COLORS, 
 	  (GConfClientNotifyFunc) mozilla_own_colors_notifier },
 	{ CONF_RENDERING_USE_OWN_FONTS, 
 	  (GConfClientNotifyFunc) mozilla_own_fonts_notifier },
-	{ CONF_FILTERING_ALLOW_POPUPS, 
+	{ CONF_SECURITY_ALLOW_POPUPS, 
 	  (GConfClientNotifyFunc) mozilla_allow_popups_notifier },
-	{ CONF_LANGUAGE_DEFAULT_CHARSET, 
-	  (GConfClientNotifyFunc) mozilla_default_charset_notifier },
 	{ CONF_RENDERING_LANGUAGE, 
 	  (GConfClientNotifyFunc) mozilla_language_notifier },
-	{ CONF_LANGUAGE_AUTODETECT_CHARSET, 
-	  (GConfClientNotifyFunc) mozilla_autodetect_charset_notifier },
 	{ CONF_RENDERING_DEFAULT_FONT, 
 	  (GConfClientNotifyFunc) mozilla_default_font_notifier },
 	{ CONF_NETWORK_SOCKS_PROXY_VERSION, 
@@ -202,6 +178,64 @@ custom_notifiers [] =
 	{NULL, NULL}
 };
 
+static gboolean
+mozilla_prefs_set_string(const char *preference_name, const char *new_value)
+{
+        g_return_val_if_fail (preference_name != NULL, FALSE);
+        g_return_val_if_fail (new_value != NULL, FALSE);
+        nsCOMPtr<nsIPrefService> prefService = 
+                                do_GetService (NS_PREFSERVICE_CONTRACTID);
+        nsCOMPtr<nsIPrefBranch> pref;
+        prefService->GetBranch ("", getter_AddRefs(pref));
+
+        if (pref)
+        {
+		nsresult rv = pref->SetCharPref (preference_name, new_value);            
+                return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+        }
+
+        return FALSE;
+}
+
+static gboolean
+mozilla_prefs_set_boolean (const char *preference_name,
+                           gboolean new_boolean_value)
+{
+        g_return_val_if_fail (preference_name != NULL, FALSE);
+  
+        nsCOMPtr<nsIPrefService> prefService = 
+                                do_GetService (NS_PREFSERVICE_CONTRACTID);
+        nsCOMPtr<nsIPrefBranch> pref;
+        prefService->GetBranch ("", getter_AddRefs(pref));
+  
+        if (pref)
+        {
+                nsresult rv = pref->SetBoolPref (preference_name,
+                                new_boolean_value ? PR_TRUE : PR_FALSE);
+                return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+        }
+        return FALSE;
+}
+
+static gboolean
+mozilla_prefs_set_int (const char *preference_name, int new_int_value)
+{
+        g_return_val_if_fail (preference_name != NULL, FALSE);
+
+        nsCOMPtr<nsIPrefService> prefService = 
+                                do_GetService (NS_PREFSERVICE_CONTRACTID);
+        nsCOMPtr<nsIPrefBranch> pref;
+        prefService->GetBranch ("", getter_AddRefs(pref));
+
+        if (pref)
+        {
+                nsresult rv = pref->SetIntPref (preference_name, new_int_value);
+                return NS_SUCCEEDED (rv) ? TRUE : FALSE;
+        }
+
+        return FALSE;
+}
+
 static void
 mozilla_font_size_notifier (GConfClient *client,
 		            guint cnxn_id,
@@ -219,7 +253,7 @@ static void
 mozilla_proxy_mode_notifier (GConfClient *client,
 		             guint cnxn_id,
 		             GConfEntry *entry,
-		             char *pref)
+		             EphyEmbedSingle *single)
 {
 	const char *mode;
 	int mozilla_mode = 0;
@@ -255,18 +289,54 @@ static void
 mozilla_proxy_autoconfig_notifier (GConfClient *client,
 		       	           guint cnxn_id,
 		       		   GConfEntry *entry,
-		       		   char *pref)
+		       		   EphyEmbedSingle *single)
 {
-	EphyEmbedSingle *single;
-	single = ephy_embed_shell_get_embed_single (embed_shell);
 	ephy_embed_single_load_proxy_autoconf 
 		(single, gconf_value_get_string(entry->value));
 }
 
+static void
+add_notification_and_notify (GConfClient		*client,
+			     const char 		*key,
+			     GConfClientNotifyFunc	 func,
+			     gpointer			 user_data)
+{
+	GConfEntry *entry;
+	GError *error = NULL;
+	guint cnxn_id;
+
+	cnxn_id = gconf_client_notify_add (client, key, func, user_data, NULL, &error);
+	if (eel_gconf_handle_error (&error))
+	{
+		if (cnxn_id != EEL_GCONF_UNDEFINED_CONNECTION)
+		{
+			gconf_client_notify_remove (client, cnxn_id);
+		}
+		return;
+	}
+
+	mozilla_notifiers = g_list_append (mozilla_notifiers,
+			                   GUINT_TO_POINTER (cnxn_id));
+
+	entry = gconf_client_get_entry (client, key, NULL, TRUE, &error);
+	if (eel_gconf_handle_error (&error))
+	{
+		return;
+	}
+	g_return_if_fail (entry != NULL);
+
+	if (entry->value != NULL)
+	{
+		func (client, cnxn_id, entry, user_data);
+	}
+	gconf_entry_free (entry);
+}
+
 void 
 mozilla_notifiers_init(EphyEmbedSingle *single) 
 {
-	int i;
+	GConfClient *client = eel_gconf_client_get_global ();
+	guint i;
 
 	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
 	{
@@ -287,66 +357,62 @@ mozilla_notifiers_init(EphyEmbedSingle *single)
 
 		g_assert (func != NULL);
 		
-		ephy_notification_add(
-			conversion_table[i].gconf_key,
-			func,
-			(gpointer)conversion_table[i].mozilla_key,
-			&mozilla_notifiers);
+		add_notification_and_notify
+			(client,
+			 conversion_table[i].gconf_key,
+			 func,
+			 (gpointer)conversion_table[i].mozilla_key);
 	}
 
 	for (i = 0; custom_notifiers[i].gconf_key != NULL; i++)
 	{
-			ephy_notification_add(
-				custom_notifiers[i].gconf_key,
-				custom_notifiers[i].func,
-				(gpointer)single,
-				&mozilla_notifiers);
+		add_notification_and_notify
+			(client,
+			 custom_notifiers[i].gconf_key,
+			 custom_notifiers[i].func,
+			 (gpointer)single);
 	}
 
 	/* fonts notifiers */
-	for (i = 0; i < LANG_ENC_NUM; i++)
+	for (i = 0; i < n_fonts_languages; i++)
 	{
-		int k;
+		guint k;
 		char *types [] = { "serif", "sans-serif", "cursive", "fantasy", "monospace" };
 		char key[255];
 		char *info;
 		
-		for (k = 0; k < 5; k++)
+		for (k = 0; k < G_N_ELEMENTS (types); k++)
 		{
-			info = g_strconcat (types[k], ".", lang_encode_item[i], NULL);
+			info = g_strconcat (types[k], ".", fonts_language[i].code, NULL);
 			
-			sprintf (key, "%s_%s_%s", CONF_RENDERING_FONT, 
+			snprintf (key, 255, "%s_%s_%s", CONF_RENDERING_FONT, 
 			 	 types[k], 
-                 	 	 lang_encode_item[i]);
-			ephy_notification_add (key,
-						 (GConfClientNotifyFunc)mozilla_font_notifier,
-						  info,
-						  &mozilla_notifiers);
+                 	 	 fonts_language[i].code);
+			add_notification_and_notify (client, key,
+						     (GConfClientNotifyFunc)mozilla_font_notifier,
+						     info);
 			font_infos = g_list_append (font_infos, info);
 		}
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("minimum-size", ".", lang_encode_item[i], NULL);
-		ephy_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_MIN_SIZE, fonts_language[i].code);
+		info = g_strconcat ("minimum-size", ".", fonts_language[i].code, NULL);
+		add_notification_and_notify (client, key,
+					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("size.fixed", ".", lang_encode_item[i], NULL);
-		ephy_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_FIXED_SIZE, fonts_language[i].code);
+		info = g_strconcat ("size.fixed", ".", fonts_language[i].code, NULL);
+		add_notification_and_notify (client, key,
+					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);
 
-		sprintf (key, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, lang_encode_item[i]);
-		info = g_strconcat ("size.variable", ".", lang_encode_item[i], NULL);
-		ephy_notification_add (key,
-					 (GConfClientNotifyFunc)mozilla_font_size_notifier,
-					 info,
-				         &mozilla_notifiers);
+		snprintf (key, 255, "%s_%s", CONF_RENDERING_FONT_VAR_SIZE, fonts_language[i].code);
+		info = g_strconcat ("size.variable", ".", fonts_language[i].code, NULL);
+		add_notification_and_notify (client, key,
+					     (GConfClientNotifyFunc)mozilla_font_size_notifier,
+					     info);
 		font_infos = g_list_append (font_infos, info);		
 	}
 }
@@ -366,40 +432,6 @@ mozilla_notifiers_free (void)
 	g_list_free (font_infos);
 }
 
-void 
-mozilla_notifiers_set_defaults(void) 
-{
-	GConfClient* client = eel_gconf_client_get_global();
-	GConfValue* value;
-	int i;
-
-	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
-	{
-		value = gconf_client_get 
-			(client, conversion_table[i].gconf_key, NULL);
-		if (value)
-		{
-			gconf_client_set (client, 
-					  conversion_table[i].gconf_key,
-					  value, NULL);
-			gconf_value_free (value);
-		}
-	}
-
-	for (i = 0; custom_notifiers[i].gconf_key != NULL; i++)
-	{
-		value = gconf_client_get 
-			(client, custom_notifiers[i].gconf_key, NULL);
-		if (value)
-		{
-			gconf_client_set (client, 
-					  custom_notifiers[i].gconf_key,
-					  value, NULL);
-			gconf_value_free (value);
-		}
-	}
-}
-	
 /**
  * generic_mozilla_string_notify: update mozilla pref to match epiphany prefs.
  * 	user_data should match the mozilla key
@@ -481,21 +513,11 @@ generic_mozilla_bool_notifier(GConfClient *client,
 	}
 }
 
-static void 
-mozilla_default_charset_notifier(GConfClient *client,
-				 guint cnxn_id,
-				 GConfEntry *entry,
-				 EphyEmbedSingle *single)
-{
-	/* FIXME */
-}
-
-
 static void
 mozilla_own_colors_notifier(GConfClient *client,
 			    guint cnxn_id,
 			    GConfEntry *entry,
-			    gpointer user_data)
+			    EphyEmbedSingle *single)
 {
 	mozilla_prefs_set_boolean("browser.display.use_document_colors",
 				       !gconf_value_get_bool(entry->value));
@@ -505,118 +527,94 @@ static void
 mozilla_own_fonts_notifier(GConfClient *client,
 			   guint cnxn_id,
 			   GConfEntry *entry,
-			   gpointer user_data)
+			   EphyEmbedSingle *single)
 {
 	mozilla_prefs_set_int("browser.display.use_document_fonts",
 				   !gconf_value_get_bool(entry->value));
 }
 
-static void
-mozilla_animate_notifier(GConfClient *client,
-			 guint cnxn_id,
-			 GConfEntry *entry,
-			 gpointer user_data)
-{
-	static const gchar *type[] =
-	{
-		"normal",
-		"once",
-		"none"
-	};
-
-	mozilla_prefs_set_string ("image.animation_mode",
-				  type[gconf_value_get_int(entry->value)]);
-}
-
 static void
 mozilla_allow_popups_notifier(GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
-			      gpointer user_data)
+			      EphyEmbedSingle *single)
 {
-	gboolean new_val = eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS);
+	gboolean new_val = eel_gconf_get_boolean(CONF_SECURITY_ALLOW_POPUPS);
 	mozilla_prefs_set_boolean ("dom.disable_open_during_load", 
 					!new_val);
 }
 
+static char *
+get_system_language ()
+{
+	const GList *sys_langs;
+
+	sys_langs = gnome_i18n_get_language_list ("LC_MESSAGES");
+
+	if (sys_langs)
+	{
+		char *lang = (char *)sys_langs->data;
+
+		/* FIXME this probably need to be smarter */
+		if (strcmp (lang, "C") != 0)
+		{
+			return g_strndup (lang, 2);
+		}
+	}
+
+	return NULL;
+}
+
 static void
 mozilla_language_notifier(GConfClient *client,
 			  guint cnxn_id,
 			  GConfEntry *entry,
-			  gpointer user_data)
+			  EphyEmbedSingle *single)
 {
-	gchar *languages;
-	GSList *language_list ,*cur_lang_list;
-	
-	language_list = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
+	GSList *languages, *l;
+	GString *result;
 
-	languages = NULL;
-	cur_lang_list = language_list;
-	while (cur_lang_list != NULL) {
-		char *lang, *tmp;
+	languages = eel_gconf_get_string_list (CONF_RENDERING_LANGUAGE);
+	g_return_if_fail (languages != NULL);
 
-		lang = g_strdup((char *)cur_lang_list->data);
-		
-		if (languages == NULL)
-			languages = lang;
-		else {
-			tmp = languages;
-			languages = g_strconcat(languages, ",", lang, NULL);
-			g_free(lang);
-			g_free(tmp);
+	result = g_string_new ("");
+
+	for (l = languages; l != NULL; l = l->next)
+	{
+		char *lang = (char *)l->data;
+
+		if (strcmp (lang, "system") == 0)
+		{
+			char *sys_lang;
+			
+			sys_lang = get_system_language ();
+			if (sys_lang)
+			{
+				g_string_append (result, sys_lang);
+				g_free (sys_lang);
+			}
 		}
-		g_free(cur_lang_list->data);
-		cur_lang_list = cur_lang_list->next;
+		else
+		{
+			g_string_append (result, (char *)l->data);
+		}
+
+		if (l->next) g_string_append (result, ",");
 	}
+	
+	mozilla_prefs_set_string ("intl.accept_languages", result->str);
 
-	if (languages == NULL)
-	{
-		languages = g_strdup ("");
-	}
-
-	mozilla_prefs_set_string ("intl.accept_languages", languages);
-	g_free (languages);
-
-	g_slist_free(language_list);
-}
-
-static char *autodetect_charset_prefs[] =
-{
-        "",
-        "zh_parallel_state_machine",
-        "cjk_parallel_state_machine",
-        "ja_parallel_state_machine",
-        "ko_parallel_state_machine",
-        "ruprob",
-        "zhcn_parallel_state_machine",
-        "zhtw_parallel_state_machine",
-        "ukprob"
-};
-
-static void
-mozilla_autodetect_charset_notifier(GConfClient *client,
-				    guint cnxn_id,
-				    GConfEntry *entry,
-				    EphyEmbedSingle *single)
-{
-	int charset = eel_gconf_get_integer (CONF_LANGUAGE_AUTODETECT_CHARSET);
-	if (charset < 0 || 
-	    charset >= (int)(sizeof(autodetect_charset_prefs)
-		             / sizeof(autodetect_charset_prefs[0])))
-	{
-		g_warning ("mozilla_autodetect_charset_notifier: "
-			   "unsupported value: %d", charset);
-		return;
-	}
-	mozilla_prefs_set_string ("intl.charset.detector", 
-				  autodetect_charset_prefs[charset]);
+	g_string_free (result, TRUE);
+	
+	g_slist_foreach (languages, (GFunc) g_free, NULL);
+	g_slist_free (languages);
 }
 
 static void
 mozilla_default_font_notifier(GConfClient *client,
 			      guint cnxn_id,
 			      GConfEntry *entry,
-			      gpointer user_data)
+			      EphyEmbedSingle *single)
 {
 	const gchar *font_types [] = {"serif","sans-serif"};
 	int default_font;
@@ -686,7 +684,7 @@ static void
 mozilla_user_agent_notifier (GConfClient *client,
 			     guint cnxn_id,
 			     GConfEntry *entry,
-			     gpointer user_data)
+			     EphyEmbedSingle *single)
 {
 	switch (entry->value->type)
 	{
@@ -704,7 +702,7 @@ static void
 mozilla_socks_version_notifier (GConfClient *client,
 				guint cnxn_id,
 				GConfEntry *entry,
-				gpointer user_data)
+				EphyEmbedSingle *single)
 {
 	int version;
 	version = gconf_value_get_int(entry->value) + 4;
