@@ -31,6 +31,7 @@ enum
   ITEM_ADDED,
   ITEM_REMOVED,
   TOOLBAR_ADDED,
+  TOOLBAR_CHANGED,
   TOOLBAR_REMOVED,
   LAST_SIGNAL
 };
@@ -213,6 +214,9 @@ egg_toolbars_model_set_flags (EggToolbarsModel *t,
   toolbar = toolbar_node->data;
 
   toolbar->flags = flags;
+
+  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_CHANGED],
+		 0, toolbar_position);
 }
 
 void
@@ -299,16 +303,22 @@ parse_item_list (EggToolbarsModel *t,
 }
 
 int
-egg_toolbars_model_add_toolbar (EggToolbarsModel *t, const char *name)
+egg_toolbars_model_add_toolbar (EggToolbarsModel *t,
+				int               position,
+				const char       *name)
 {
   GNode *node;
+  int real_position;
 
   g_return_val_if_fail (IS_EGG_TOOLBARS_MODEL (t), -1);
 
   node = g_node_new (toolbars_toolbar_new (name));
-  g_node_insert (t->priv->toolbars, -1, node);
+  g_node_insert (t->priv->toolbars, position, node);
 
-  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_ADDED], 0);
+  real_position = g_node_child_position (t->priv->toolbars, node);
+
+  g_signal_emit (G_OBJECT (t), egg_toolbars_model_signals[TOOLBAR_ADDED],
+		 0, real_position);
 
   return g_node_child_position (t->priv->toolbars, node);
 }
@@ -325,7 +335,7 @@ parse_toolbars (EggToolbarsModel *t,
 	  int position;
 
 	  name = xmlGetProp (child, "name");
-	  position = egg_toolbars_model_add_toolbar (t, name);
+	  position = egg_toolbars_model_add_toolbar (t, -1, name);
 	  xmlFree (name);
 
 	  parse_item_list (t, child->children, position);
@@ -392,6 +402,13 @@ egg_toolbars_model_class_init (EggToolbarsModelClass *klass)
 		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_removed),
 		  NULL, NULL, g_cclosure_marshal_VOID__INT,
 		  G_TYPE_NONE, 1, G_TYPE_INT);
+  egg_toolbars_model_signals[TOOLBAR_CHANGED] =
+    g_signal_new ("toolbar_changed",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (EggToolbarsModelClass, toolbar_changed),
+		  NULL, NULL, g_cclosure_marshal_VOID__INT,
+		  G_TYPE_NONE, 1, G_TYPE_INT);
 }
 
 static void
