@@ -16,12 +16,21 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <bonobo/bonobo-i18n.h>
+
 #include "ephy-bookmark-action.h"
 #include "ephy-bookmarks.h"
 #include "ephy-shell.h"
+#include "ephy-string.h"
 #include "eggtoolitem.h"
 #include "ephy-debug.h"
 
+#define MAX_LABEL_LENGTH 30
+
 static void ephy_bookmark_action_init       (EphyBookmarkAction *action);
 static void ephy_bookmark_action_class_init (EphyBookmarkActionClass *class);
 
@@ -79,6 +88,16 @@ ephy_bookmark_action_get_type (void)
 	return type;
 }
 
+static GtkWidget *
+create_menu_item (EggAction *action)
+{
+	GtkWidget *item;
+
+	item = gtk_image_menu_item_new ();
+
+	return item;
+}
+
 static GtkWidget *
 create_tool_item (EggAction *action)
 {
@@ -105,6 +124,7 @@ create_tool_item (EggAction *action)
 		GtkWidget *entry;
 
 		entry = gtk_entry_new ();
+		gtk_widget_set_size_request (entry, 120, -1);
 		gtk_widget_show (entry);
 		gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
 		g_object_set_data (G_OBJECT (item), "entry", entry);
@@ -131,13 +151,9 @@ static void
 ephy_bookmark_action_sync_icon (EggAction *action, GParamSpec *pspec, GtkWidget *proxy)
 {
 	char *icon_location;
-	GtkImage *icon;
 	EphyFaviconCache *cache;
 	GdkPixbuf *pixbuf = NULL;
 
-	icon = GTK_IMAGE (g_object_get_data (G_OBJECT (proxy), "icon"));
-	g_return_if_fail (icon != NULL);
-
 	icon_location = EPHY_BOOKMARK_ACTION (action)->priv->icon;
 
 	cache = ephy_embed_shell_get_favicon_cache
@@ -148,30 +164,70 @@ ephy_bookmark_action_sync_icon (EggAction *action, GParamSpec *pspec, GtkWidget
 		pixbuf = ephy_favicon_cache_get (cache, icon_location);
 	}
 
-	if (pixbuf)
+	if (pixbuf == NULL) return;
+
+	if (EGG_IS_TOOL_ITEM (proxy))
 	{
+		GtkImage *icon;
+
+		icon = GTK_IMAGE (g_object_get_data (G_OBJECT (proxy), "icon"));
+		g_return_if_fail (icon != NULL);
+
 		gtk_image_set_from_pixbuf (icon, pixbuf);
-		g_object_unref (pixbuf);
 	}
-	else
+	else if (GTK_IS_MENU_ITEM (proxy))
 	{
-		gtk_image_set_from_stock (icon,
-					  GTK_STOCK_JUMP_TO,
-					  GTK_ICON_SIZE_MENU);
+		GtkWidget *image;
+
+		image = gtk_image_new_from_pixbuf (pixbuf);
+		gtk_widget_show (image);
+
+		gtk_image_menu_item_set_image
+			(GTK_IMAGE_MENU_ITEM (proxy), image);
 	}
+
+	g_object_unref (pixbuf);
 }
 
 static void
 ephy_bookmark_action_sync_label (EggAction *action, GParamSpec *pspec, GtkWidget *proxy)
 {
-	GtkLabel *label;
+	GtkWidget *label;
+	char *label_text;
+	char *title;
 
 	LOG ("Set bookmark action proxy label to %s", action->label)
 
-	label = GTK_LABEL (g_object_get_data (G_OBJECT (proxy), "label"));
-	g_return_if_fail (label != NULL);
+	title = ephy_string_shorten (action->label, MAX_LABEL_LENGTH);
 
-	gtk_label_set_label (label, action->label);
+	if (EPHY_BOOKMARK_ACTION (action)->priv->smart_url)
+	{
+		label_text = g_strdup_printf (_("%s:"), title);
+	}
+	else
+	{
+		label_text = g_strdup (title);
+	}
+
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		label = g_object_get_data (G_OBJECT (proxy), "label");
+		g_return_if_fail (label != NULL);
+	}
+	else if (GTK_IS_MENU_ITEM (proxy))
+	{
+		label = GTK_BIN (proxy)->child;
+	}
+	else
+	{
+		g_warning ("Unkown widget");
+		return;
+	}
+
+	gtk_label_set_label (GTK_LABEL (label), label_text);
+
+	g_free (label_text);
+	g_free (title);
 }
 
 static void
@@ -199,7 +255,7 @@ entry_activated_cb (GtkWidget *entry, EggAction *action)
 }
 
 static void
-button_clicked_cb (GtkWidget *button, EggAction *action)
+activate_cb (GtkWidget *widget, EggAction *action)
 {
 	g_signal_emit (action,
 		       ephy_bookmark_action_signals[GO_LOCATION],
@@ -221,10 +277,18 @@ connect_proxy (EggAction *action, GtkWidget *proxy)
 	g_signal_connect_object (action, "notify::icon",
 			         G_CALLBACK (ephy_bookmark_action_sync_icon), proxy, 0);
 
-	button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
-	g_signal_connect (button, "clicked", G_CALLBACK (button_clicked_cb), action);
+	if (EGG_IS_TOOL_ITEM (proxy))
+	{
+		button = GTK_WIDGET (g_object_get_data (G_OBJECT (proxy), "button"));
+		g_signal_connect (button, "clicked", G_CALLBACK (activate_cb), action);
+	}
+	else if (GTK_IS_MENU_ITEM (proxy))
+	{
+		g_signal_connect (proxy, "activate", G_CALLBACK (activate_cb), action);
+	}
 
-	if (EPHY_BOOKMARK_ACTION (action)->priv->smart_url)
+	if (EPHY_BOOKMARK_ACTION (action)->priv->smart_url &&
+	    EGG_IS_TOOL_ITEM (proxy))
 	{
 		GtkWidget *entry;
 
@@ -315,6 +379,7 @@ ephy_bookmark_action_class_init (EphyBookmarkActionClass *class)
 
 	action_class->toolbar_item_type = EGG_TYPE_TOOL_ITEM;
 	action_class->create_tool_item = create_tool_item;
+	action_class->create_menu_item = create_menu_item;
 	action_class->connect_proxy = connect_proxy;
 
 	object_class->finalize = ephy_bookmark_action_finalize;
@@ -415,9 +480,9 @@ ephy_bookmark_action_init (EphyBookmarkAction *action)
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 	node = ephy_bookmarks_get_bookmarks (bookmarks);
-	g_signal_connect_object (node, "child_changed",
-				 G_CALLBACK (bookmarks_child_changed_cb),
-				 action, 0);
+	ephy_node_signal_connect_object (node, EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) bookmarks_child_changed_cb,
+				         G_OBJECT (action));
 }
 
 EggAction *
@@ -429,7 +494,7 @@ ephy_bookmark_action_new (const char *name, guint id)
 
 	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
 
-	bmk = ephy_node_get_from_id (id);
+	bmk = ephy_bookmarks_get_from_id (bookmarks, id);
 	g_return_val_if_fail (bmk != NULL, NULL);
 
 	action =  EGG_ACTION (g_object_new (EPHY_TYPE_BOOKMARK_ACTION,
