@@ -122,11 +122,10 @@ ephy_bookmark_properties_class_init (EphyBookmarkPropertiesClass *klass)
 
 	g_object_class_install_property (object_class,
 					 PROP_BOOKMARK,
-					 g_param_spec_object ("bookmark",
-							      "Bookmark",
-							      "Bookmark",
-							      EPHY_TYPE_NODE,
-							      G_PARAM_READWRITE));
+					 g_param_spec_pointer ("bookmark",
+							       "Bookmark",
+							       "Bookmark",
+							       G_PARAM_READWRITE));
 }
 
 static void
@@ -172,7 +171,7 @@ ephy_bookmark_properties_set_property (GObject *object,
 		break;
 	case PROP_BOOKMARK:
 		ephy_bookmark_properties_set_bookmark
-			(selector, g_value_get_object (value));
+			(selector, g_value_get_pointer (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -336,12 +335,13 @@ build_ui (EphyBookmarkProperties *editor)
 	set_window_icon (editor);
 
 	gtk_dialog_set_has_separator (GTK_DIALOG (editor), FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (editor), 6);
-	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (editor)->vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (editor), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (editor)->vbox), 2);
 
 	table = gtk_table_new (2, 2, FALSE);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-	gtk_table_set_col_spacings (GTK_TABLE (table), 6);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_TABLE (table)), 5);
 	gtk_widget_show (table);
 
 	entry = gtk_entry_new ();
