@@ -30,7 +30,6 @@
 #include <gtk/gtkvbox.h>
 #include <gdk/gdkkeysyms.h>
 #include <bonobo/bonobo-i18n.h>
-#include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
 
@@ -53,6 +52,7 @@
 #include "window-commands.h"
 #include "ephy-debug.h"
 #include "ephy-gui.h"
+#include "ephy-stock-icons.h"
 
 static GtkTargetEntry topic_drag_dest_types [] =
 {
@@ -315,7 +315,7 @@ cmd_show_in_bookmarks_bar (EggAction *action,
 		return;
 	}
 
-	node = EPHY_NODE (selection->data);
+	node = selection->data;
 	id = ephy_node_get_id (node);
 	state = EGG_TOGGLE_ACTION (action)->active;
 
@@ -346,14 +346,14 @@ cmd_open_bookmarks_in_tabs (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_BMK_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
-			EPHY_NEW_TAB_APPEND_GROUPED | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+			EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	}
 
 	g_list_free (selection);
@@ -372,7 +372,7 @@ cmd_open_bookmarks_in_browser (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
@@ -400,7 +400,7 @@ cmd_delete (EggAction *action,
 		EphyNode *node;
 
 		selected = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->key_view));
-		node = EPHY_NODE (selected->data);
+		node = selected->data;
 		priority = ephy_node_get_property_int (node, EPHY_NODE_KEYWORD_PROP_PRIORITY);
 
 		if (priority == -1) priority = EPHY_NODE_NORMAL_PRIORITY;
@@ -452,7 +452,7 @@ cmd_bookmark_properties (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		show_properties_dialog (editor, node);
 	}
 
@@ -491,7 +491,7 @@ cmd_copy (EggAction *action,
 		if (g_list_length (selection) == 1)
 		{
 			const char *tmp;
-			EphyNode *node = EPHY_NODE (selection->data);
+			EphyNode *node = selection->data;
 			tmp = ephy_node_get_property_string (node, EPHY_NODE_BMK_PROP_LOCATION);
 			gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), tmp, -1);
 		}
@@ -632,7 +632,6 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 	const char *location;
 	EphyWindow *window;
 
-	g_return_if_fail (EPHY_IS_NODE (node));
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_BMK_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
@@ -647,6 +646,7 @@ ephy_bookmarks_editor_node_activated_cb (GtkWidget *view,
 	{
 		/* We have to create a browser window */
 		ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
+				    EPHY_NEW_TAB_OPEN_PAGE |
 				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
 }
@@ -707,7 +707,7 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	selected = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->key_view));
 	if (key_focus && selected)
 	{
-		EphyNode *node = EPHY_NODE (selected->data);
+		EphyNode *node = selected->data;
 		EphyNodePriority priority;
 		gulong id;
 
@@ -726,9 +726,11 @@ ephy_bookmarks_editor_update_menu (EphyBookmarksEditor *editor)
 	selected = ephy_node_view_get_selection (EPHY_NODE_VIEW (editor->priv->bm_view));
 	if (bmk_focus && selected)
 	{
-		EphyNode *node = EPHY_NODE (selected->data);
+		EphyNode *node = selected->data;
 		gulong id;
 
+		g_return_if_fail (node != NULL);
+
 		id = ephy_node_get_id (node);
 		show_in_bookmarks_bar = ephy_toolbars_model_has_bookmark
 			(editor->priv->tb_model, FALSE, id);
@@ -874,7 +876,7 @@ ephy_bookmarks_editor_dispose (GObject *object)
 			return;
 		}
 
-		selected_id = ephy_node_get_id (EPHY_NODE (selection->data));
+		selected_id = ephy_node_get_id (selection->data);
 		if (selected_id >= 0)
 		{
 			selected_id_str = g_strdup_printf ("%ld", selected_id);
@@ -1065,11 +1067,9 @@ node_dropped_cb (EphyNodeView *view, EphyNode *node,
 {
 	GList *l;
 
-	g_return_if_fail (EPHY_IS_NODE (node));
-
 	for (l = nodes; l != NULL; l = l->next)
 	{
-		EphyNode *bmk = EPHY_NODE (l->data);
+		EphyNode *bmk = l->data;
 
 		ephy_bookmarks_set_keyword (editor->priv->bookmarks, node, bmk);
 	}
@@ -1117,14 +1117,16 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 	EggMenuMerge *ui_merge;
 	EggActionGroup *action_group;
 	EggAction *action;
-	const char *icon_path;
+	GdkPixbuf *icon;
 	int i;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("Bookmarks"));
 
-	icon_path = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_PIXMAP,
-					       "epiphany-bookmarks.png", TRUE, NULL);
-	gtk_window_set_icon_from_file (GTK_WINDOW (editor), icon_path, NULL);
+	icon = gtk_widget_render_icon (GTK_WIDGET (editor), 
+				       EPHY_STOCK_BOOKMARKS,
+				       GTK_ICON_SIZE_MENU,
+				       NULL);
+	gtk_window_set_icon (GTK_WINDOW(editor), icon);
 
 	g_signal_connect (editor, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
@@ -1290,7 +1292,7 @@ ephy_bookmarks_editor_construct (EphyBookmarksEditor *editor)
 		return;
 	}
 
-	selected_node = ephy_node_get_from_id (selected_id);
+	selected_node = ephy_bookmarks_get_from_id (editor->priv->bookmarks, selected_id);
 	if (selected_node != NULL)
 	{
 		ephy_node_view_select_node (EPHY_NODE_VIEW (key_view), selected_node);
