@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.41 2003/05/17 12:43:34 cneumair Exp $
+ *  $Id: ephy-bookmarks.c,v 1.48 2003/06/04 15:27:13 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,7 +42,10 @@
 
 struct EphyBookmarksPrivate
 {
+	EphyToolbarsModel *toolbars_model;
+	gboolean init_defaults;
 	char *xml_file;
+	EphyNodeDb *db;
 	EphyNode *bookmarks;
 	EphyNode *keywords;
 	EphyNode *favorites;
@@ -80,6 +83,21 @@ static const char *default_topics [] =
 };
 static int n_default_topics = G_N_ELEMENTS (default_topics);
 
+enum
+{
+	PROP_0,
+	PROP_TOOLBARS_MODEL
+};
+
+/* Signals */
+enum
+{
+	TREE_CHANGED,
+	LAST_SIGNAL
+};
+
+static guint ephy_bookmarks_signals[LAST_SIGNAL] = { 0 };
+
 static void
 ephy_bookmarks_class_init (EphyBookmarksClass *klass);
 static void
@@ -89,17 +107,8 @@ ephy_bookmarks_finalize (GObject *object);
 static void
 ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface);
 
-enum
-{
-	BOOKMARK_REMOVE,
-	TOPIC_REMOVE,
-	LAST_SIGNAL
-};
-
 static GObjectClass *parent_class = NULL;
 
-static guint ephy_bookmarks_signals[LAST_SIGNAL] = { 0 };
-
 GType
 ephy_bookmarks_get_type (void)
 {
@@ -201,45 +210,11 @@ ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
 	iface->set_basic_key = ephy_bookmarks_autocompletion_source_set_basic_key;
 }
 
-static void
-ephy_bookmarks_class_init (EphyBookmarksClass *klass)
-{
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
-        parent_class = g_type_class_peek_parent (klass);
-
-        object_class->finalize = ephy_bookmarks_finalize;
-
-	ephy_bookmarks_signals[BOOKMARK_REMOVE] =
-                g_signal_new ("bookmark_remove",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyBookmarksClass, bookmark_remove),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_INT);
-	ephy_bookmarks_signals[TOPIC_REMOVE] =
-                g_signal_new ("topic_remove",
-                              G_OBJECT_CLASS_TYPE (object_class),
-                              G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (EphyBookmarksClass, topic_remove),
-                              NULL, NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE,
-                              1,
-			      G_TYPE_INT);
-}
-
 static void
 ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 {
 	int i, id;
 	EphyNode *node;
-	EphyToolbarsModel *model;
-
-	model = ephy_shell_get_toolbars_model (ephy_shell);
 
 	for (i = 0; i < n_default_topics; i++)
 	{
@@ -255,12 +230,93 @@ ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 		node = ephy_bookmarks_find_bookmark (eb, default_bookmarks[i].location);
 		if (node == NULL) break;
 		id = ephy_node_get_id (node);
-		ephy_toolbars_model_add_bookmark (model, FALSE, id);
+		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE, id);
 	}
 
 	ephy_bookmarks_save (eb);
 }
 
+static void
+ephy_bookmarks_set_toolbars_model (EphyBookmarks *eb, EphyToolbarsModel *model)
+{
+	eb->priv->toolbars_model = model;
+	g_object_add_weak_pointer (G_OBJECT(eb->priv->toolbars_model),
+				   (gpointer *)&eb->priv->toolbars_model);
+
+	if (eb->priv->init_defaults)
+	{
+		ephy_bookmarks_init_defaults (eb);
+	}
+}
+
+static void
+ephy_bookmarks_set_property (GObject *object,
+                             guint prop_id,
+                             const GValue *value,
+                             GParamSpec *pspec)
+{
+	EphyBookmarks *eb;
+
+	eb = EPHY_BOOKMARKS (object);
+
+	switch (prop_id)
+	{
+		case PROP_TOOLBARS_MODEL:
+			ephy_bookmarks_set_toolbars_model (eb, g_value_get_object (value));
+			break;
+	}
+}
+
+static void
+ephy_bookmarks_get_property (GObject *object,
+                             guint prop_id,
+                             GValue *value,
+                             GParamSpec *pspec)
+{
+	EphyBookmarks *eb;
+
+	eb = EPHY_BOOKMARKS (object);
+
+	switch (prop_id)
+	{
+		case PROP_TOOLBARS_MODEL:
+			g_value_set_object (value, eb->priv->toolbars_model);
+			break;
+	}
+}
+
+
+static void
+ephy_bookmarks_class_init (EphyBookmarksClass *klass)
+{
+        GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+        parent_class = g_type_class_peek_parent (klass);
+
+        object_class->finalize = ephy_bookmarks_finalize;
+	object_class->set_property = ephy_bookmarks_set_property;
+	object_class->get_property = ephy_bookmarks_get_property;
+
+	ephy_bookmarks_signals[TREE_CHANGED] =
+		g_signal_new ("tree_changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyBookmarksClass, tree_changed),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
+	g_object_class_install_property (object_class,
+                                         PROP_TOOLBARS_MODEL,
+                                         g_param_spec_object ("toolbars_model",
+                                                              "Toolbars model",
+                                                              "Toolbars model",
+                                                              EPHY_TOOLBARS_MODEL_TYPE,
+                                                              G_PARAM_READWRITE));
+
+}
+
 static gboolean
 ephy_bookmarks_load (EphyBookmarks *eb)
 {
@@ -284,7 +340,7 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	{
 		EphyNode *node;
 
-		node = ephy_node_new_from_xml (child);
+		node = ephy_node_new_from_xml (eb->priv->db, child);
 	}
 
 	xmlFreeDoc (doc);
@@ -338,7 +394,7 @@ ephy_bookmarks_save (EphyBookmarks *eb)
 	}
 	ephy_node_thaw (eb->priv->bookmarks);
 
-	xmlSaveFormatFile (eb->priv->xml_file, doc, 1);
+	ephy_file_save_xml (eb->priv->xml_file, doc);
 	xmlFreeDoc(doc);
 
 	/* Export bookmarks in rdf */
@@ -481,15 +537,11 @@ bookmarks_changed_cb (EphyNode *node,
 static void
 bookmarks_removed_cb (EphyNode *node,
 		      EphyNode *child,
+		      guint old_index,
 		      EphyBookmarks *eb)
 {
-	long id;
-
-	id = ephy_node_get_id (child);
-	g_signal_emit (eb, ephy_bookmarks_signals[BOOKMARK_REMOVE],
-		       0, id);
-
 	ephy_bookmarks_emit_data_changed (eb);
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
 static char *
@@ -543,9 +595,9 @@ update_topics_list (EphyNode *bookmark, const char *list)
 static void
 topics_removed_cb (EphyNode *node,
 		   EphyNode *child,
+		   guint old_index,
 		   EphyBookmarks *eb)
 {
-	long id;
 	GPtrArray *children;
 	int i;
 
@@ -572,24 +624,27 @@ topics_removed_cb (EphyNode *node,
 	}
 	ephy_node_thaw (child);
 
-	id = ephy_node_get_id (child);
-	g_signal_emit (eb, ephy_bookmarks_signals[TOPIC_REMOVE],
-		       0, id);
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
 static void
 ephy_bookmarks_init (EphyBookmarks *eb)
 {
 	GValue value = { 0, };
+	EphyNodeDb *db;
 
         eb->priv = g_new0 (EphyBookmarksPrivate, 1);
+	eb->priv->toolbars_model = NULL;
+
+	db = ephy_node_db_new ("EphyBookmarks");
+	eb->priv->db = db;
 
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
 					       "bookmarks.xml",
 					       NULL);
 
 	/* Bookmarks */
-	eb->priv->bookmarks = ephy_node_new_with_id (BOOKMARKS_NODE_ID);
+	eb->priv->bookmarks = ephy_node_new_with_id (db, BOOKMARKS_NODE_ID);
 	ephy_node_ref (eb->priv->bookmarks);
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, _("All"));
@@ -597,19 +652,17 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 			        EPHY_NODE_KEYWORD_PROP_NAME,
 			        &value);
 	g_value_unset (&value);
-	g_signal_connect_object (G_OBJECT (eb->priv->bookmarks),
-				 "child_removed",
-				 G_CALLBACK (bookmarks_removed_cb),
-				 G_OBJECT (eb),
-				 0);
-	g_signal_connect_object (G_OBJECT (eb->priv->bookmarks),
-				 "child_changed",
-				 G_CALLBACK (bookmarks_changed_cb),
-				 G_OBJECT (eb),
-				 0);
+	ephy_node_signal_connect_object (eb->priv->bookmarks,
+				         EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) bookmarks_removed_cb,
+				         G_OBJECT (eb));
+	ephy_node_signal_connect_object (eb->priv->bookmarks,
+				         EPHY_NODE_CHILD_CHANGED,
+				         (EphyNodeCallback) bookmarks_changed_cb,
+				         G_OBJECT (eb));
 
 	/* Keywords */
-	eb->priv->keywords = ephy_node_new_with_id (KEYWORDS_NODE_ID);
+	eb->priv->keywords = ephy_node_new_with_id (db, KEYWORDS_NODE_ID);
 	ephy_node_ref (eb->priv->keywords);
 	g_value_init (&value, G_TYPE_INT);
 	g_value_set_int (&value, EPHY_NODE_ALL_PRIORITY);
@@ -617,17 +670,16 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 			        EPHY_NODE_KEYWORD_PROP_PRIORITY,
 			        &value);
 	g_value_unset (&value);
-	g_signal_connect_object (G_OBJECT (eb->priv->keywords),
-				 "child_removed",
-				 G_CALLBACK (topics_removed_cb),
-				 G_OBJECT (eb),
-				 0);
+	ephy_node_signal_connect_object (eb->priv->keywords,
+				         EPHY_NODE_CHILD_REMOVED,
+				         (EphyNodeCallback) topics_removed_cb,
+				         G_OBJECT (eb));
 
 	ephy_node_add_child (eb->priv->keywords,
 			     eb->priv->bookmarks);
 
 	/* Favorites */
-	eb->priv->favorites = ephy_node_new_with_id (FAVORITES_NODE_ID);
+	eb->priv->favorites = ephy_node_new_with_id (db, FAVORITES_NODE_ID);
 	ephy_node_ref (eb->priv->favorites);
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, _("Most Visited"));
@@ -644,7 +696,7 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	ephy_node_add_child (eb->priv->keywords, eb->priv->favorites);
 
 	/* Not categorized */
-	eb->priv->notcategorized = ephy_node_new_with_id (BMKS_NOTCATEGORIZED_NODE_ID);
+	eb->priv->notcategorized = ephy_node_new_with_id (db, BMKS_NOTCATEGORIZED_NODE_ID);
 	ephy_node_ref (eb->priv->notcategorized);
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, _("Not Categorized"));
@@ -660,10 +712,7 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	g_value_unset (&value);
 	ephy_node_add_child (eb->priv->keywords, eb->priv->notcategorized);
 
-	if (!ephy_bookmarks_load (eb))
-	{
-		ephy_bookmarks_init_defaults (eb);
-	}
+	eb->priv->init_defaults = !ephy_bookmarks_load (eb);
 
 	ephy_bookmarks_emit_data_changed (eb);
 
@@ -689,6 +738,14 @@ ephy_bookmarks_finalize (GObject *object)
 	ephy_node_unref (eb->priv->favorites);
 	ephy_node_unref (eb->priv->notcategorized);
 
+	g_object_unref (eb->priv->db);
+
+	if (eb->priv->toolbars_model)
+	{
+		g_object_remove_weak_pointer (G_OBJECT(eb->priv->toolbars_model),
+					      (gpointer *)&eb->priv->toolbars_model);
+	}
+
         g_free (eb->priv);
 
 	LOG ("Bookmarks finalized")
@@ -715,7 +772,7 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 	EphyNode *bm;
 	GValue value = { 0, };
 
-	bm = ephy_node_new ();
+	bm = ephy_node_new (eb->priv->db);
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, title);
@@ -739,6 +796,7 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 	ephy_node_add_child (eb->priv->notcategorized, bm);
 
 	ephy_bookmarks_emit_data_changed (eb);
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 
 	return bm;
 }
@@ -952,7 +1010,7 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 	EphyNode *key;
 	GValue value = { 0, };
 
-	key = ephy_node_new ();
+	key = ephy_node_new (eb->priv->db);
 
 	g_value_init (&value, G_TYPE_STRING);
 	g_value_set_string (&value, name);
@@ -1048,6 +1106,8 @@ ephy_bookmarks_set_keyword (EphyBookmarks *eb,
 
 	update_topics_list (bookmark, list);
 	g_free (list);
+
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
 void
@@ -1074,6 +1134,8 @@ ephy_bookmarks_unset_keyword (EphyBookmarks *eb,
 
 	update_topics_list (bookmark, list);
 	g_free (list);
+
+	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
 }
 
 EphyNode *
@@ -1094,3 +1156,14 @@ ephy_bookmarks_get_favorites (EphyBookmarks *eb)
 	return eb->priv->favorites;
 }
 
+EphyNode *
+ephy_bookmarks_get_not_categorized (EphyBookmarks *eb)
+{
+	return eb->priv->notcategorized;
+}
+
+EphyNode *
+ephy_bookmarks_get_from_id (EphyBookmarks *eb, long id)
+{
+	return ephy_node_db_get_node_from_id (eb->priv->db, id);
+}
