@@ -47,13 +47,6 @@ enum
 	EPHY_NODE_KEYWORD_PROP_PRIORITY = 8
 };
 
-typedef enum
-{
-	EPHY_BOOKMARKS_KEYWORD_ALL_PRIORITY,
-	EPHY_BOOKMARKS_KEYWORD_SPECIAL_PRIORITY,
-	EPHY_BOOKMARKS_KEYWORD_NORMAL_PRIORITY
-} EphyBookmarksKeywordPriority;
-
 struct EphyBookmarks
 {
         GObject parent;
@@ -64,24 +57,26 @@ struct EphyBookmarksClass
 {
         GObjectClass parent_class;
 
-	void (* bookmark_remove) (EphyBookmarks *eb,
-			          long id);
-	void (* topic_remove)    (EphyBookmarks *eb,
-			          long id);
+	void (* tree_changed) (EphyBookmarks *eb);
 };
 
 GType		ephy_bookmarks_get_type		(void);
 
 EphyBookmarks  *ephy_bookmarks_new		(void);
 
+EphyNode	*ephy_bookmarks_get_from_id     (EphyBookmarks *eb,
+						 long id);
+
 /* Bookmarks */
 
-EphyNode       *ephy_bookmarks_add		(EphyBookmarks *eb,
+void		 ephy_bookmarks_save		(EphyBookmarks *eb);
+
+EphyNode        *ephy_bookmarks_add		(EphyBookmarks *eb,
 						 const char *title,
 						 const char *url,
 						 const char *smart_url);
 
-guint		 ephy_bookmarks_get_bookmark_id (EphyBookmarks *eb,
+EphyNode*	 ephy_bookmarks_find_bookmark   (EphyBookmarks *eb,
 						 const char *url);
 
 void		 ephy_bookmarks_set_icon	(EphyBookmarks *eb,
@@ -126,6 +121,8 @@ EphyNode       *ephy_bookmarks_get_keywords	(EphyBookmarks *eb);
 
 EphyNode       *ephy_bookmarks_get_bookmarks	(EphyBookmarks *eb);
 
+EphyNode       *ephy_bookmarks_get_not_categorized (EphyBookmarks *eb);
+
 G_END_DECLS
 
 #endif
