@@ -17,6 +17,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <gtk/gtktable.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkstock.h>
@@ -24,7 +28,7 @@
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
 #include <gdk/gdkkeysyms.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
 
@@ -43,7 +47,9 @@
 #include "ephy-debug.h"
 #include "ephy-new-bookmark.h"
 #include "ephy-stock-icons.h"
+#include "ephy-gui.h"
 #include "toolbar.h"
+#include "ephy-stock-icons.h"
 
 static GtkTargetEntry page_drag_types [] =
 {
@@ -58,13 +64,14 @@ static void ephy_history_window_class_init (EphyHistoryWindowClass *klass);
 static void ephy_history_window_init (EphyHistoryWindow *editor);
 static void ephy_history_window_finalize (GObject *object);
 static void ephy_history_window_set_property (GObject *object,
-		                                guint prop_id,
-		                                const GValue *value,
-		                                GParamSpec *pspec);
+		                              guint prop_id,
+		                              const GValue *value,
+		                              GParamSpec *pspec);
 static void ephy_history_window_get_property (GObject *object,
-						guint prop_id,
-						GValue *value,
-						GParamSpec *pspec);
+					      guint prop_id,
+					      GValue *value,
+					      GParamSpec *pspec);
+static void ephy_history_window_dispose      (GObject *object);
 
 static void search_entry_changed_cb       (GtkWidget *entry,
 					   EphyHistoryWindow *editor);
@@ -74,7 +81,7 @@ static void cmd_open_bookmarks_in_browser (EggAction *action,
 					   EphyHistoryWindow *editor);
 static void cmd_delete                    (EggAction *action,
                                            EphyHistoryWindow *editor);
-static void cmd_bookmark_page             (EggAction *action,
+static void cmd_bookmark_link             (EggAction *action,
                                            EphyHistoryWindow *editor);
 static void cmd_clear			  (EggAction *action,
 				           EphyHistoryWindow *editor);
@@ -102,6 +109,7 @@ struct EphyHistoryWindowPrivate
 	GtkWidget *window;
 	EggMenuMerge *ui_merge;
 	EggActionGroup *action_group;
+	GtkWidget *confirmation_dialog;
 };
 
 enum
@@ -122,52 +130,52 @@ static EggActionGroupEntry ephy_history_ui_entries [] = {
 
 	/* File Menu */
 	{ "OpenInWindow", N_("_Open in New Window"), GTK_STOCK_OPEN, "<control>O",
-	  N_("Open the selected history item in a new window"), 
+	  N_("Open the selected history link in a new window"),
 	  G_CALLBACK (cmd_open_bookmarks_in_browser), NULL },
 	{ "OpenInTab", N_("Open in New _Tab"), NULL, "<shift><control>O",
-	  N_("Open the selected history item in a new tab"), 
+	  N_("Open the selected history link in a new tab"),
 	  G_CALLBACK (cmd_open_bookmarks_in_tabs), NULL },
 	{ "Delete", N_("_Delete"), GTK_STOCK_DELETE, NULL,
-	  N_("Delete the selected history item"), 
+	  N_("Delete the selected history link"),
 	  G_CALLBACK (cmd_delete), NULL },
-	{ "BookmarkPage", N_("Boo_kmark Page..."), EPHY_STOCK_BOOKMARK_PAGE, "<control>D",
-	  N_("Bookmark the selected history item"), 
-	  G_CALLBACK (cmd_bookmark_page), NULL },
+	{ "BookmarkLink", N_("Boo_kmark Link..."), EPHY_STOCK_BOOKMARK_PAGE, "<control>D",
+	  N_("Bookmark the selected history link"),
+	  G_CALLBACK (cmd_bookmark_link), NULL },
 	{ "Close", N_("_Close"), GTK_STOCK_CLOSE, "<control>W",
-	  N_("Close the history window"), 
+	  N_("Close the history window"),
 	  G_CALLBACK (cmd_close), NULL },
-	
+
 	/* Edit Menu */
 	{ "Cut", N_("Cu_t"), GTK_STOCK_CUT, "<control>X",
-	  N_("Cut the selection"), 
+	  N_("Cut the selection"),
 	  G_CALLBACK (cmd_cut), NULL },
 	{ "Copy", N_("_Copy"), GTK_STOCK_COPY, "<control>C",
-	  N_("Copy the selection"), 
+	  N_("Copy the selection"),
 	  G_CALLBACK (cmd_copy), NULL },
 	{ "Paste", N_("_Paste"), GTK_STOCK_PASTE, "<control>V",
-	  N_("Paste the clipboard"), 
+	  N_("Paste the clipboard"),
 	  G_CALLBACK (cmd_paste), NULL },
 	{ "SelectAll", N_("Select _All"), NULL, "<control>A",
-	  N_("Select all history items or text"), 
+	  N_("Select all history links or text"),
 	  G_CALLBACK (cmd_select_all), NULL },
 	{ "Clear", N_("C_lear History"), GTK_STOCK_CLEAR, NULL,
-	  N_("Clear your browsing history"), 
+	  N_("Clear your browsing history"),
 	  G_CALLBACK (cmd_clear), NULL },
-	
+
 	/* View Menu */
 	{ "ViewTitle", N_("_Title"), NULL, NULL,
-	  N_("Show only the title column"), 
+	  N_("Show only the title column"),
 	  NULL, NULL, RADIO_ACTION, NULL },
-	{ "ViewLocation", N_("_Location"), NULL, NULL,
-	  N_("Show only the location column"), 
+	{ "ViewLocation", N_("_Address"), NULL, NULL,
+	  N_("Show only the address column"),
 	  NULL, NULL, RADIO_ACTION, "ViewTitle" },
-	{ "ViewTitleLocation", N_("T_itle and Location"), NULL, NULL,
-	  N_("Show both the title and location columns"), 
+	{ "ViewTitleLocation", N_("T_itle and Address"), NULL, NULL,
+	  N_("Show both the title and address columns"),
 	  NULL, NULL, RADIO_ACTION, "ViewTitle" },
-		
-	/* Help Menu */	
+
+	/* Help Menu */
 	{ "HelpContents", N_("_Contents"), GTK_STOCK_HELP, "F1",
-	  N_("Display history help"), 
+	  N_("Display history help"),
 	  G_CALLBACK (cmd_help_contents), NULL },
 	{ "HelpAbout", N_("_About"), GNOME_STOCK_ABOUT, NULL,
 	  N_("Display credits for the web browser creators"),
@@ -176,15 +184,20 @@ static EggActionGroupEntry ephy_history_ui_entries [] = {
 static guint ephy_history_ui_n_entries = G_N_ELEMENTS (ephy_history_ui_entries);
 
 static void
-cmd_clear (EggAction *action,
-	   EphyHistoryWindow *editor)
+confirmation_dialog_response_cb (GtkDialog *dialog, gint response,
+				 EphyHistoryWindow *editor)
 {
 	const GList *windows;
 	Session *session;
-	
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+
+	if (response != GTK_RESPONSE_OK)
+		return;
+
 	session = ephy_shell_get_session (ephy_shell);
 	windows = session_get_windows (session);
-               
+
 	for (; windows != NULL; windows = windows->next)
 	{
 		Toolbar *t;
@@ -196,10 +209,90 @@ cmd_clear (EggAction *action,
 	ephy_history_clear (editor->priv->history);
 }
 
+static GtkWidget *
+confirmation_dialog_construct (EphyHistoryWindow *editor)
+{
+	GtkWidget *dialog;
+	GtkWidget *label;
+	GtkWidget *vbox;
+	GtkWidget *hbox;
+	GtkWidget *image;
+	char *str;
+
+	dialog = gtk_dialog_new_with_buttons (_("Clear history"),
+					     GTK_WINDOW (editor),
+					     GTK_DIALOG_DESTROY_WITH_PARENT |
+					     GTK_DIALOG_NO_SEPARATOR,
+					     GTK_STOCK_CANCEL,
+					     GTK_RESPONSE_CANCEL,
+					     GTK_STOCK_CLEAR,
+					     GTK_RESPONSE_OK,
+					     NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_widget_show (hbox);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
+			    TRUE, TRUE, 0);
+
+	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
+					  GTK_ICON_SIZE_DIALOG);
+	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
+	gtk_widget_show (image);
+	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 0);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
+
+	label = gtk_label_new (NULL);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	str = g_strconcat ("<b><big>", _("Clear browsing history?"),
+			   "</big></b>", NULL);
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	g_free (str);
+	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+	gtk_widget_show (label);
+
+	label = gtk_label_new (_("Clearing the browsing history will cause all"
+				 " history links to be permanently deleted."));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
+	gtk_widget_show (label);
+
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (confirmation_dialog_response_cb),
+			  editor);
+
+	return dialog;
+}
+
+static void
+cmd_clear (EggAction *action,
+	   EphyHistoryWindow *editor)
+{
+	if (editor->priv->confirmation_dialog == NULL)
+	{
+		editor->priv->confirmation_dialog = confirmation_dialog_construct (editor);
+		g_object_add_weak_pointer (G_OBJECT(editor->priv->confirmation_dialog),
+					   (gpointer *)&editor->priv->confirmation_dialog);
+	}
+
+	gtk_widget_show (editor->priv->confirmation_dialog);
+}
+
 static void
 cmd_close (EggAction *action,
 	   EphyHistoryWindow *editor)
 {
+	if (editor->priv->confirmation_dialog != NULL)
+	{
+		gtk_widget_destroy (editor->priv->confirmation_dialog);
+	}
 	gtk_widget_hide (GTK_WIDGET (editor));
 }
 
@@ -229,14 +322,14 @@ cmd_open_bookmarks_in_tabs (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_PAGE_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
-			EPHY_NEW_TAB_APPEND_GROUPED | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
+			EPHY_NEW_TAB_OPEN_PAGE | EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	}
 
 	g_list_free (selection);
@@ -255,13 +348,14 @@ cmd_open_bookmarks_in_browser (EggAction *action,
 
 	for (l = selection; l; l = l->next)
 	{
-		EphyNode *node = EPHY_NODE (l->data);
+		EphyNode *node = l->data;
 		const char *location;
 
 		location = ephy_node_get_property_string (node,
 						EPHY_NODE_PAGE_PROP_LOCATION);
 
 		ephy_shell_new_tab (ephy_shell, window, NULL, location,
+				    EPHY_NEW_TAB_OPEN_PAGE |
 				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
 
@@ -300,7 +394,7 @@ cmd_copy (EggAction *action,
 		if (g_list_length (selection) == 1)
 		{
 			const char *tmp;
-			EphyNode *node = EPHY_NODE (selection->data);
+			EphyNode *node = selection->data;
 			tmp = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 			gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), tmp, -1);
 		}
@@ -352,7 +446,7 @@ cmd_delete (EggAction *action,
 }
 
 static void
-cmd_bookmark_page (EggAction *action,
+cmd_bookmark_link (EggAction *action,
                    EphyHistoryWindow *editor)
 {
         GtkWindow *window;
@@ -371,17 +465,18 @@ cmd_bookmark_page (EggAction *action,
 		const char *title;
 		EphyNode *node;
 
-		node = EPHY_NODE (selection->data);
+		node = selection->data;
 		location = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_LOCATION);
 		title = ephy_node_get_property_string (node, EPHY_NODE_PAGE_PROP_TITLE);
-		new_bookmark = ephy_new_bookmark_new
-			(bookmarks, window, location);
-		ephy_new_bookmark_set_title
-			(EPHY_NEW_BOOKMARK (new_bookmark), title);
-		g_signal_connect (G_OBJECT (new_bookmark), "response",
-			  	  G_CALLBACK (ephy_new_bookmark_response_cb),
-			  	  NULL);
-		gtk_widget_show (new_bookmark);
+		if (ephy_new_bookmark_is_unique (bookmarks, GTK_WINDOW (window),
+						 location))
+		{
+			new_bookmark = ephy_new_bookmark_new
+				(bookmarks, window, location);
+			ephy_new_bookmark_set_title
+				(EPHY_NEW_BOOKMARK (new_bookmark), title);
+			gtk_widget_show (new_bookmark);
+		}
 	}
 	g_list_free (selection);
 }
@@ -390,7 +485,9 @@ static void
 cmd_help_contents (EggAction *action,
 		   EphyHistoryWindow *editor)
 {
-	/*FIXME: Implement help.*/
+	ephy_gui_help (GTK_WINDOW (editor),
+		       "epiphany", 
+		       "ephy-managing-history");
 }
 
 GType
@@ -432,6 +529,7 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 
 	object_class->set_property = ephy_history_window_set_property;
 	object_class->get_property = ephy_history_window_get_property;
+	object_class->dispose  = ephy_history_window_dispose;
 
 	g_object_class_install_property (object_class,
 					 PROP_HISTORY,
@@ -481,7 +579,6 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 	const char *location;
 	EphyWindow *window;
 
-	g_return_if_fail (EPHY_IS_NODE (node));
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
@@ -556,7 +653,7 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 
 	if (pages_focus)
 	{
-		copy_label = _("_Copy Location");
+		copy_label = _("_Copy Address");
 	}
 	else
 	{
@@ -587,10 +684,60 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	g_object_set (action, "sensitive", select_all, NULL);
 	action = egg_action_group_get_action (action_group, "Delete");
 	g_object_set (action, "sensitive", delete, NULL);
-	action = egg_action_group_get_action (action_group, "BookmarkPage");
+	action = egg_action_group_get_action (action_group, "BookmarkLink");
 	g_object_set (action, "sensitive", bookmark_page, NULL);
 }
 
+static void
+entry_selection_changed_cb (GtkWidget *widget, GParamSpec *pspec, EphyHistoryWindow *editor)
+{
+	ephy_history_window_update_menu (editor);
+}
+
+static void
+add_entry_monitor (EphyHistoryWindow *editor, GtkWidget *entry)
+{
+        g_signal_connect (G_OBJECT (entry),
+                          "notify::selection-bound",
+                          G_CALLBACK (entry_selection_changed_cb),
+                          editor);
+        g_signal_connect (G_OBJECT (entry),
+                          "notify::cursor-position",
+                          G_CALLBACK (entry_selection_changed_cb),
+                          editor);
+}
+
+static gboolean
+view_focus_cb (EphyNodeView *view,
+               GdkEventFocus *event,
+               EphyHistoryWindow *editor)
+{
+       ephy_history_window_update_menu (editor);
+
+       return FALSE;
+}
+
+static void
+add_focus_monitor (EphyHistoryWindow *editor, GtkWidget *widget)
+{
+       g_signal_connect (G_OBJECT (widget),
+                         "focus_in_event",
+                         G_CALLBACK (view_focus_cb),
+                         editor);
+       g_signal_connect (G_OBJECT (widget),
+                         "focus_out_event",
+                         G_CALLBACK (view_focus_cb),
+                         editor);
+}
+
+static void
+remove_focus_monitor (EphyHistoryWindow *editor, GtkWidget *widget)
+{
+       g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
+                                             G_CALLBACK (view_focus_cb),
+                                             editor);
+}
+
 static void
 ephy_history_window_show_popup_cb (GtkWidget *view,
 				   EphyHistoryWindow *editor)
@@ -599,7 +746,6 @@ ephy_history_window_show_popup_cb (GtkWidget *view,
 
 	widget = egg_menu_merge_get_widget (editor->priv->ui_merge,
 					    "/popups/EphyHistoryWindowPopup");
-	ephy_history_window_update_menu (editor);
 	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, 2,
 			gtk_get_current_event_time ());
 }
@@ -721,6 +867,8 @@ build_search_box (EphyHistoryWindow *editor)
 	gtk_widget_show (box);
 
 	entry = gtk_entry_new ();
+	add_focus_monitor (editor, entry);
+	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
 	gtk_widget_show (entry);
 	g_signal_connect (G_OBJECT (entry), "changed",
@@ -759,13 +907,6 @@ delete_event_cb (EphyHistoryWindow *editor)
 	return TRUE;
 }
 
-static void
-menu_activate_cb (EphyNodeView *view,
-	          EphyHistoryWindow *editor)
-{
-	ephy_history_window_update_menu (editor);
-}
-
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
@@ -788,6 +929,12 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	g_value_set_object (value, pixbuf);
 }
 
+static void
+view_selection_changed_cb (GtkWidget *view, EphyHistoryWindow *editor)
+{
+	ephy_history_window_update_menu (editor);
+}
+
 static void
 ephy_history_window_construct (EphyHistoryWindow *editor)
 {
@@ -796,18 +943,20 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	GtkWidget *vbox, *hpaned;
 	GtkWidget *pages_view, *sites_view;
 	GtkWidget *scrolled_window;
-	GtkWidget *menu;
 	EphyNode *node;
 	EggMenuMerge *ui_merge;
 	EggActionGroup *action_group;
 	EggAction *action;
-	const char *icon_path;
+	GdkPixbuf *icon;
 	int i;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("History"));
 
-	icon_path = ephy_file ("epiphany-history.png");
-	gtk_window_set_icon_from_file (GTK_WINDOW (editor), icon_path, NULL);
+	icon = gtk_widget_render_icon (GTK_WIDGET (editor), 
+				       EPHY_STOCK_HISTORY,
+				       GTK_ICON_SIZE_MENU,
+				       NULL);
+	gtk_window_set_icon (GTK_WINDOW(editor), icon);
 
 	g_signal_connect (editor, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
@@ -836,12 +985,6 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	editor->priv->ui_merge = ui_merge;
 	editor->priv->action_group = action_group;
 
-	/* Update menu sensitivity before showing them */
-	menu = egg_menu_merge_get_widget (ui_merge, "/menu/FileMenu");
-	g_signal_connect (menu, "activate", G_CALLBACK (menu_activate_cb), editor);
-	menu = egg_menu_merge_get_widget (ui_merge, "/menu/EditMenu");
-	g_signal_connect (menu, "activate", G_CALLBACK (menu_activate_cb), editor);
-
 	/* Fixme: We should implement gconf prefs for monitoring this setting */
 	action = egg_action_group_get_action (action_group, "ViewTitle");
 	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action), TRUE);
@@ -865,6 +1008,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	gtk_paned_add1 (GTK_PANED (hpaned), scrolled_window);
 	gtk_widget_show (scrolled_window);
 	sites_view = ephy_node_view_new (node, NULL);
+	add_focus_monitor (editor, sites_view);
 	ephy_node_view_select_node (EPHY_NODE_VIEW (sites_view),
 				    ephy_history_get_pages (editor->priv->history));
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (sites_view),
@@ -886,6 +1030,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  "node_selected",
 			  G_CALLBACK (site_node_selected_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (selection),
+			  "changed",
+			  G_CALLBACK (view_selection_changed_cb),
+			  editor);
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_paned_add2 (GTK_PANED (hpaned), vbox);
@@ -908,6 +1056,8 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	node = ephy_history_get_pages (editor->priv->history);
 	editor->priv->pages_filter = ephy_node_filter_new ();
 	pages_view = ephy_node_view_new (node, editor->priv->pages_filter);
+	add_focus_monitor (editor, pages_view);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pages_view));
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (pages_view), TRUE);
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (pages_view),
 					   page_drag_types,
@@ -917,7 +1067,7 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_TITLE,
 				         -1, EPHY_NODE_VIEW_USER_SORT, NULL);
 	gtk_tree_view_column_set_max_width (col, 250);
-	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Location"),
+	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
 				         -1, EPHY_NODE_VIEW_USER_SORT, NULL);
 	gtk_tree_view_column_set_max_width (col, 200);
@@ -936,10 +1086,15 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 			  "key_press_event",
 			  G_CALLBACK (key_pressed_cb),
 			  editor);
+	g_signal_connect (G_OBJECT (selection),
+			  "changed",
+			  G_CALLBACK (view_selection_changed_cb),
+			  editor);
 
 	ephy_state_add_window (GTK_WIDGET (editor),
 			       "history_window",
-		               450, 400);
+		               450, 400,
+			       EPHY_STATE_WINDOW_SAVE_SIZE | EPHY_STATE_WINDOW_SAVE_POSITION);
 	ephy_state_add_paned  (GTK_WIDGET (hpaned),
 			       "history_paned",
 		               130);
@@ -1024,3 +1179,26 @@ ephy_history_window_init (EphyHistoryWindow *editor)
 {
 	editor->priv = g_new0 (EphyHistoryWindowPrivate, 1);
 }
+
+static void
+ephy_history_window_dispose (GObject *object)
+{
+	EphyHistoryWindow *editor;
+
+	g_return_if_fail (object != NULL);
+	g_return_if_fail (EPHY_IS_HISTORY_WINDOW (object));
+
+	editor = EPHY_HISTORY_WINDOW (object);
+
+	if (editor->priv->sites_view != NULL)
+	{
+		remove_focus_monitor (editor, editor->priv->pages_view);
+		remove_focus_monitor (editor, editor->priv->sites_view);
+		remove_focus_monitor (editor, editor->priv->search_entry);
+
+		editor->priv->sites_view = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
