@@ -16,15 +16,24 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "ephy-notebook.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-marshal.h"
 #include "ephy-file-helpers.h"
+#include "ephy-dnd.h"
+#include "ephy-embed.h"
+#include "ephy-window.h"
+#include "ephy-shell.h"
 
 #include <gtk/gtk.h>
 #include <glib-object.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 
 #define AFTER_ALL_TABS -1
 #define NOT_IN_APP_WINDOWS -2
@@ -38,6 +47,8 @@ struct EphyNotebookPrivate
 
 	EphyNotebookPageLoadStatus current_status;
 
+	GtkTooltips *title_tips;
+
 	/* Used during tab drag'n'drop */
 	gulong motion_notify_handler_id;
 	gint x_start, y_start;
@@ -55,6 +66,12 @@ static void ephy_notebook_finalize     (GObject *object);
 static GdkCursor *cursor = NULL;
 static GList *notebooks  = NULL;
 
+static GtkTargetEntry url_drag_types [] = 
+{
+        { EPHY_DND_URI_LIST_TYPE,   0, 0 },
+        { EPHY_DND_URL_TYPE,        0, 1 }
+};
+static guint n_url_drag_types = G_N_ELEMENTS (url_drag_types);
 
 /* Local functions */
 static void drag_start (EphyNotebook *notebook,
@@ -71,6 +88,7 @@ enum
 {
 	TAB_DROPPED,
 	TAB_DETACHED,
+	TABS_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -115,8 +133,7 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 		g_signal_new ("tab_dropped",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EphyNotebookClass,
-					       tab_dropped),
+			      G_STRUCT_OFFSET (EphyNotebookClass, tab_dropped),
 			      NULL, NULL,
 			      ephy_marshal_VOID__OBJECT_OBJECT_INT,
 			      G_TYPE_NONE,
@@ -128,8 +145,7 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 		g_signal_new ("tab_detached",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EphyNotebookClass,
-					       tab_detached),
+			      G_STRUCT_OFFSET (EphyNotebookClass, tab_detached),
 			      NULL, NULL,
 			      ephy_marshal_VOID__INT_INT_INT,
 			      G_TYPE_NONE,
@@ -137,7 +153,14 @@ ephy_notebook_class_init (EphyNotebookClass *klass)
 			      G_TYPE_INT,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
-
+	ephy_notebook_signals[TABS_CHANGED] =
+		g_signal_new ("tabs_changed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (EphyNotebookClass, tabs_changed),
+			      NULL, NULL,
+			      ephy_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
 }
 
 static gboolean
@@ -288,18 +311,6 @@ tab_label_set_size (GtkWidget *window, GtkWidget *label)
 	gtk_widget_set_size_request (label, label_width, -1);
 }
 
-static GtkWidget *
-tab_get_label (EphyNotebook *nb, GtkWidget *child)
-{
-	GtkWidget *hbox, *label;
-
-	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
-					   child);
-	label = g_object_get_data (G_OBJECT (hbox), "label");
-
-	return label;
-}
-
 static void
 tab_label_size_request_cb (GtkWidget *window,
 			   GtkRequisition *requisition,
@@ -318,21 +329,29 @@ tab_label_size_request_cb (GtkWidget *window,
 
 void
 ephy_notebook_move_page (EphyNotebook *src, EphyNotebook *dest,
-			GtkWidget *src_page,  gint dest_page)
+			 GtkWidget *src_page,  gint dest_page)
 {
 	GtkWidget *tab_label;
 
-	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (src), src_page);
+	if (dest == NULL || src == dest)
+	{
+		gtk_notebook_reorder_child (GTK_NOTEBOOK (src), src_page, dest_page);
+		g_signal_emit (G_OBJECT (src), ephy_notebook_signals[TABS_CHANGED], 0);
+	}
+	else
+	{
+		tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (src), src_page);
 
-	/* We don't want gtk to destroy tab and src_page behind our back */
-	g_object_ref (G_OBJECT (src_page));
-	g_object_ref (G_OBJECT (tab_label));
-	ephy_notebook_remove_page (EPHY_NOTEBOOK (src), src_page);
-	ephy_notebook_insert_page (EPHY_NOTEBOOK (dest), src_page,
-				  dest_page, TRUE);
-	gtk_notebook_set_tab_label (GTK_NOTEBOOK (dest), src_page, tab_label);
-	g_object_unref (G_OBJECT (src_page));
-	g_object_unref (G_OBJECT (tab_label));
+		/* We don't want gtk to destroy tab and src_page behind our back */
+		g_object_ref (G_OBJECT (src_page));
+		g_object_ref (G_OBJECT (tab_label));
+		ephy_notebook_remove_page (EPHY_NOTEBOOK (src), src_page);
+		ephy_notebook_insert_page (EPHY_NOTEBOOK (dest), src_page,
+					  dest_page, TRUE);
+		gtk_notebook_set_tab_label (GTK_NOTEBOOK (dest), src_page, tab_label);
+		g_object_unref (G_OBJECT (src_page));
+		g_object_unref (G_OBJECT (tab_label));
+	}
 }
 
 
@@ -384,8 +403,8 @@ move_tab (EphyNotebook *notebook, gint dest_page_num)
 		GtkWidget *cur_page;
 		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
 						      cur_page_num);
-		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), cur_page,
-					    dest_page_num);
+		ephy_notebook_move_page (EPHY_NOTEBOOK (notebook), NULL, cur_page,
+					 dest_page_num);
 
 		/* Reset the list of newly opened tabs when moving tabs. */
 		g_list_free (notebook->priv->opened_tabs);
@@ -585,6 +604,91 @@ ephy_notebook_switch_page_cb (GtkNotebook *notebook,
 	nb->priv->opened_tabs = NULL;
 }
 
+#define INSANE_NUMBER_OF_URLS	20
+
+static void
+notebook_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
+				gint x, gint y, GtkSelectionData *selection_data,
+				guint info, guint time, GtkWidget *child)
+{
+	EphyEmbed *embed = NULL;
+	EphyWindow *window = NULL;
+	EphyTab *tab = NULL;
+	GtkWidget *toplevel;
+	GList *uris = NULL, *l;
+	GnomeVFSURI *uri;
+	gchar *url = NULL;
+	guint num = 0;
+	gchar **tmp;
+
+	g_signal_stop_emission_by_name (widget, "drag_data_received");
+
+	if (selection_data->length <= 0 || selection_data->data == NULL) return;
+
+	if (selection_data->target == gdk_atom_intern (EPHY_DND_URL_TYPE, FALSE))
+	{
+		/* URL_TYPE has format: url \n title */
+		tmp = g_strsplit (selection_data->data, "\n", 2);
+		if (tmp)
+		{
+			uri = gnome_vfs_uri_new (tmp[0]);
+			if (uri) uris = g_list_append (uris, uri);
+			g_strfreev (tmp);
+		}
+	}
+	else if (selection_data->target == gdk_atom_intern (EPHY_DND_URI_LIST_TYPE, FALSE))
+	{
+		uris = gnome_vfs_uri_list_parse (selection_data->data);
+	}
+	else
+	{
+		g_assert_not_reached ();
+	}
+
+	if (uris == NULL) return;
+
+	toplevel = gtk_widget_get_toplevel (widget);
+	g_return_if_fail (IS_EPHY_WINDOW (toplevel));
+	window = EPHY_WINDOW (toplevel);
+
+	if (child)
+	{
+		embed = EPHY_EMBED (child);
+		tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "EphyTab"));
+	}
+
+	l = uris;
+	while (l != NULL && num < INSANE_NUMBER_OF_URLS)
+	{
+		uri = (GnomeVFSURI*) l->data;
+		url = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
+
+		if (num == 0 && embed != NULL)
+		{
+			/**
+			 * The first url is special: if the drag was to an
+			 * existing tab, load it there
+			 */
+			ephy_embed_load_url (embed, url);
+		}
+		else
+		{
+			tab = ephy_shell_new_tab (ephy_shell, window,
+						  tab, url,
+						  EPHY_NEW_TAB_OPEN_PAGE |
+						  EPHY_NEW_TAB_IN_EXISTING_WINDOW |
+						  EPHY_NEW_TAB_APPEND_LAST);
+		}
+
+		g_free (url);
+		url = NULL;
+		l = l->next;
+		++num;
+	}
+
+	gnome_vfs_uri_list_free (uris);
+}
+
 static void
 ephy_notebook_init (EphyNotebook *notebook)
 {
@@ -592,6 +696,10 @@ ephy_notebook_init (EphyNotebook *notebook)
 
 	notebook->priv->current_status = EPHY_NOTEBOOK_TAB_LOAD_NORMAL;
 
+	notebook->priv->title_tips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (notebook->priv->title_tips));
+	gtk_object_sink (GTK_OBJECT (notebook->priv->title_tips));
+
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
 	notebook->priv->src_notebook = NULL;
@@ -610,6 +718,15 @@ ephy_notebook_init (EphyNotebook *notebook)
 	g_signal_connect_after (G_OBJECT (notebook), "switch_page",
                                 G_CALLBACK (ephy_notebook_switch_page_cb),
                                 NULL);
+
+	/* Set up drag-and-drop target */
+	g_signal_connect (G_OBJECT(notebook), "drag_data_received",
+			  G_CALLBACK(notebook_drag_data_received_cb),
+			  NULL);
+        gtk_drag_dest_set (GTK_WIDGET(notebook), GTK_DEST_DEFAULT_MOTION |
+			   GTK_DEST_DEFAULT_DROP,
+                           url_drag_types,n_url_drag_types,
+                           GDK_ACTION_MOVE | GDK_ACTION_COPY);
 }
 
 static void
@@ -624,6 +741,7 @@ ephy_notebook_finalize (GObject *object)
 		g_list_free (notebook->priv->focused_pages);
 	}
 	g_list_free (notebook->priv->opened_tabs);
+	g_object_unref (notebook->priv->title_tips);
 
 	g_free (notebook->priv);
 }
@@ -650,7 +768,7 @@ ephy_notebook_set_page_status (EphyNotebook *nb,
 	image  = g_object_get_data (G_OBJECT (tab), "loading-image");
 
 	g_return_if_fail (image != NULL);
-	
+
 	icon = g_object_get_data (G_OBJECT (tab), "icon");
 
 	g_return_if_fail (icon != NULL);
@@ -710,6 +828,7 @@ tab_build_label (EphyNotebook *nb, GtkWidget *child)
 	GClosure *closure;
 	GtkWidget *window;
 	GtkWidget *loading_image, *icon;
+	GtkWidget *label_ebox;
 	GdkPixbufAnimation *loading_pixbuf;
 
 	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
@@ -741,10 +860,12 @@ tab_build_label (EphyNotebook *nb, GtkWidget *child)
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
 
 	/* setup label */
+	label_ebox = gtk_event_box_new ();
         label = gtk_label_new (_("Untitled"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
         gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (label_ebox), label);
 
 	tab_label_set_size (GTK_WIDGET (window), hbox);
 
@@ -766,11 +887,13 @@ tab_build_label (EphyNotebook *nb, GtkWidget *child)
                           child);
 
 	gtk_widget_show (hbox);
+	gtk_widget_show (label_ebox);
 	gtk_widget_show (label);
 	gtk_widget_show (image);
 	gtk_widget_show (close_button);
 
 	g_object_set_data (G_OBJECT (hbox), "label", label);
+	g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
 	g_object_set_data (G_OBJECT (hbox), "loading-image", loading_image);
 	g_object_set_data (G_OBJECT (hbox), "icon", icon);
 
@@ -791,7 +914,7 @@ update_tabs_visibility (EphyNotebook *nb, gboolean before_inserting)
 
 	if (before_inserting) tabs_num--;
 
-	show_tabs = eel_gconf_get_boolean (CONF_TABS_TABBED) || 
+	show_tabs = eel_gconf_get_boolean (CONF_TABS_TABBED) ||
 		    gtk_notebook_get_nth_page (GTK_NOTEBOOK (nb), tabs_num) > 0;
 
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
@@ -799,9 +922,9 @@ update_tabs_visibility (EphyNotebook *nb, gboolean before_inserting)
 
 void
 ephy_notebook_insert_page (EphyNotebook *nb,
-			  GtkWidget *child,
-			  int position,
-			  gboolean jump_to)
+			   GtkWidget *child,
+			   int position,
+			   gboolean jump_to)
 {
 	GtkWidget *tab_hbox;
 
@@ -835,6 +958,13 @@ ephy_notebook_insert_page (EphyNotebook *nb,
 				  child,
 				  tab_hbox, position);
 
+	/* Set up drag-and-drop target */
+	g_signal_connect (G_OBJECT(tab_hbox), "drag_data_received",
+			  G_CALLBACK(notebook_drag_data_received_cb), child);
+	gtk_drag_dest_set (tab_hbox, GTK_DEST_DEFAULT_ALL,
+			   url_drag_types,n_url_drag_types,
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
+
 	if (jump_to)
 	{
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
@@ -842,6 +972,8 @@ ephy_notebook_insert_page (EphyNotebook *nb,
 		g_object_set_data (G_OBJECT (child), "jump_to",
 				   GINT_TO_POINTER (jump_to));
 	}
+
+	g_signal_emit (G_OBJECT (nb), ephy_notebook_signals[TABS_CHANGED], 0);
 }
 
 static void
@@ -879,6 +1011,11 @@ ephy_notebook_remove_page (EphyNotebook *nb,
 {
 	int position, cur;
 	gboolean last_tab;
+	GtkWidget *hbox, *ebox;
+
+	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label-ebox"));
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, NULL, NULL);
 
 	last_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (nb), 1) == NULL;
 	if (last_tab)
@@ -905,6 +1042,7 @@ ephy_notebook_remove_page (EphyNotebook *nb,
 	}
 
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
+	g_signal_emit (G_OBJECT (nb), ephy_notebook_signals[TABS_CHANGED], 0);
 
 	update_tabs_visibility (nb, FALSE);
 }
@@ -914,8 +1052,15 @@ ephy_notebook_set_page_title (EphyNotebook *nb,
 			     GtkWidget *child,
 			     const char *title)
 {
-	GtkWidget *label;
+	GtkWidget *hbox, *ebox, *label;
 
-	label = tab_get_label (nb, child);
+	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+
+	label = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label"));
 	gtk_label_set_label (GTK_LABEL (label), title);
+
+	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (hbox), "label-ebox"));
+	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, title, NULL);
+
+	g_signal_emit (G_OBJECT (nb), ephy_notebook_signals[TABS_CHANGED], 0);
 }
