@@ -63,13 +63,6 @@ struct EphyShellPrivate
 	GtkWidget *history_window;
 };
 
-enum
-{
-        STARTPAGE_HOME,
-        STARTPAGE_LAST,
-        STARTPAGE_BLANK,
-};
-
 static void
 ephy_shell_class_init (EphyShellClass *klass);
 static void
@@ -248,7 +241,6 @@ ephy_shell_init (EphyShell *gs)
 
 	ephy_debug_init ();
 	ephy_thread_helpers_init ();
-	ephy_node_system_init (EPHY_NODE_RESERVED_IDS);
 	ephy_file_helpers_init ();
 	ephy_stock_icons_init ();
 	ephy_ensure_dir_exists (ephy_dot_dir ());
@@ -342,7 +334,6 @@ ephy_shell_finalize (GObject *object)
 
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
-	ephy_node_system_shutdown ();
 
 	LOG ("Ephy shell finalized")
 
@@ -363,7 +354,6 @@ ephy_init_services (EphyShell *gs)
 	/* preload the prefs */
 	/* it also enables notifiers support */
 	eel_gconf_monitor_add ("/apps/epiphany");
-	eel_gconf_monitor_add ("/apps/nautilus/preferences");
 	eel_gconf_monitor_add ("/system/proxy");
 
 #ifdef ENABLE_NAUTILUS_VIEW
@@ -374,58 +364,21 @@ ephy_init_services (EphyShell *gs)
 
 }
 
-static char *
-build_homepage_url (EphyShell *gs,
-		    EphyEmbed *previous_embed)
+static void
+load_homepage (EphyEmbed *embed)
 {
-        const gchar *last_page_url;
-        gchar *home_page_url;
-        gint page_type;
-        EphyHistory *gh;
-	char *result = NULL;
+	char *home;
 
-        if (previous_embed == NULL)
-        {
-                page_type = STARTPAGE_HOME;
-        }
-        else
-        {
-                page_type = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
-        }
+	home = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
 
-        /* return the appropriate page */
-        if (page_type == STARTPAGE_HOME)
-        {
-                /* get location of home page */
-                home_page_url = eel_gconf_get_string(CONF_GENERAL_HOMEPAGE);
-		result = home_page_url;
-        }
-	else if (page_type == STARTPAGE_LAST)
+	if (home == NULL)
 	{
-		if (previous_embed != NULL)
-		{
-			ephy_embed_get_location (previous_embed,
-						 TRUE,
-						 &result);
-		}
-
-		if (result == NULL)
-		{
-			/* get location of last page */
-			gh = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (gs));
-			last_page_url = ephy_history_get_last_page (gh);
-			result = g_strdup (last_page_url);
-		}
+		home = g_strdup ("about:blank");
 	}
 
-	if (result == NULL)
-	{
-		/* even in case of error, it's a good default */
-		result = g_strdup ("about:blank");
-	}
+	ephy_embed_load_url (embed, home);
 
-	return result;
+	g_free (home);
 }
 
 /**
@@ -476,21 +429,19 @@ ephy_shell_new_tab (EphyShell *shell,
 	EphyTab *tab;
 	EphyEmbed *embed;
 	gboolean in_new_window;
-	gboolean grouped = FALSE;
+	gboolean grouped;
 	gboolean jump_to;
 	EphyEmbed *previous_embed = NULL;
 	GtkWidget *nb;
 	gint position;
+	Toolbar *toolbar;
 
 	in_new_window = !eel_gconf_get_boolean (CONF_TABS_TABBED);
 
 	if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
 	if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
 
-	jump_to = eel_gconf_get_boolean (CONF_TABS_TABBED_AUTOJUMP);
-
-	if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
-	if (flags & EPHY_NEW_TAB_DONT_JUMP_TO) jump_to = FALSE;
+	jump_to = (flags & EPHY_NEW_TAB_JUMP);
 
 	if (!in_new_window && parent_window != NULL)
 	{
@@ -501,30 +452,26 @@ ephy_shell_new_tab (EphyShell *shell,
 		window = ephy_window_new ();
 	}
 
+	toolbar = ephy_window_get_toolbar (window);
+
 	if (previous_tab)
 	{
 		previous_embed = ephy_tab_get_embed (previous_tab);
 	}
 
-	if (url != NULL || flags & EPHY_NEW_TAB_IS_A_COPY ||
-	    flags & EPHY_NEW_TAB_VIEW_SOURCE)
-	{
-		grouped = TRUE;
-	}
+	grouped = (flags & EPHY_NEW_TAB_OPEN_PAGE ||
+	           flags & EPHY_NEW_TAB_APPEND_GROUPED ||
+		   flags & EPHY_NEW_TAB_CLONE_PAGE);
 
-	if (flags & EPHY_NEW_TAB_APPEND_GROUPED) grouped = TRUE;
-	if (flags & EPHY_NEW_TAB_APPEND_LAST) grouped = FALSE;
-
-	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
+	if (flags & EPHY_NEW_TAB_APPEND_AFTER)
 	{
+		g_assert (previous_embed != NULL);
 		nb = ephy_window_get_notebook (window);
-		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), 
+		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 						  GTK_WIDGET (previous_embed)) + 1;
 	}
-	else
-	{	
-		position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
-	}
+
+	position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
 
 	tab = ephy_tab_new ();
 	embed = ephy_tab_get_embed (tab);
@@ -532,46 +479,43 @@ ephy_shell_new_tab (EphyShell *shell,
 	ephy_window_add_tab (window, tab,
 			     position,
 			     jump_to);
+	gtk_widget_show (GTK_WIDGET(window));
 
-	if (flags & EPHY_NEW_TAB_RAISE_WINDOW)
+	if (flags & EPHY_NEW_TAB_HOME_PAGE)
 	{
-		gtk_window_present (GTK_WINDOW(window));
-	}
-	else
-	{
-		gtk_widget_show (GTK_WIDGET(window));
-	}
-
-	if (flags & EPHY_NEW_TAB_HOMEPAGE)
-	{
-		Toolbar *toolbar;
-		char *homepage;
-
-		homepage = build_homepage_url (shell, previous_embed);
-		g_assert (homepage != NULL);
-
-		toolbar = ephy_window_get_toolbar (window);
 		toolbar_edit_location (toolbar);
 
-		ephy_embed_load_url (embed, homepage);
-
-		g_free (homepage);
+		load_homepage (embed);
 	}
-	else if ((flags & EPHY_NEW_TAB_IS_A_COPY) ||
-		 (flags & EPHY_NEW_TAB_VIEW_SOURCE))
+	else if (flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		EmbedDisplayType display_type =
-			(flags & EPHY_NEW_TAB_VIEW_SOURCE) ?
-			 DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
+		char *previous_address = NULL;
+
+		g_return_val_if_fail (previous_embed != NULL, NULL);
+		ephy_embed_get_location (previous_embed, TRUE,
+					 &previous_address);
+		toolbar_set_location (toolbar, previous_address);
+		toolbar_edit_location (toolbar);
+
+		load_homepage (embed);
+	}
+	else if (flags & EPHY_NEW_TAB_OPEN_PAGE)
+	{
+		g_assert (url != NULL);
+		ephy_embed_load_url (embed, url);
+	}
+	else if (flags & EPHY_NEW_TAB_CLONE_PAGE)
+	{
+		EmbedDisplayType display_type;
+
+                display_type = (flags & EPHY_NEW_TAB_SOURCE_MODE) ?
+                                DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
+
 		ephy_embed_load_url (embed, "about:blank");
 		ephy_embed_copy_page (embed, previous_embed, display_type);
 	}
-	else if (url)
-	{
-		ephy_embed_load_url (embed, url);
-	}
 
-	if (flags & EPHY_NEW_TAB_FULLSCREEN)
+	if (flags & EPHY_NEW_TAB_FULLSCREEN_MODE)
 	{
 		ephy_window_set_chrome (window, EMBED_CHROME_OPENASFULLSCREEN |
 				        EMBED_CHROME_DEFAULT);
@@ -668,8 +612,11 @@ ephy_shell_get_toolbars_model (EphyShell *gs)
 	{
 		char *xml_file;
 		EggToolbarsModel *model;
+		EphyBookmarks *bookmarks;
 
-		gs->priv->toolbars_model = ephy_toolbars_model_new ();
+		bookmarks = ephy_shell_get_bookmarks (gs);
+
+		gs->priv->toolbars_model = ephy_toolbars_model_new (bookmarks);
 		model = EGG_TOOLBARS_MODEL (gs->priv->toolbars_model);
 
 		xml_file = g_build_filename (ephy_dot_dir (),
@@ -687,6 +634,9 @@ ephy_shell_get_toolbars_model (EphyShell *gs)
 			egg_toolbars_model_load (model, default_xml);
 		}
 		g_free (xml_file);
+
+		g_object_set (bookmarks, "toolbars_model",
+			      gs->priv->toolbars_model, NULL);
 	}
 
 	return gs->priv->toolbars_model;
