@@ -14,6 +14,8 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *  $Id: ephy-window.c,v 1.70 2003/06/05 18:16:07 mpeseng Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -21,6 +23,7 @@
 #endif
 
 #include "ephy-window.h"
+#include "ephy-bookmarks-menu.h"
 #include "ephy-favorites-menu.h"
 #include "ephy-state.h"
 #include "ephy-gobject-misc.h"
@@ -30,16 +33,18 @@
 #include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
+#include "ephy-zoom.h"
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
 #include "statusbar.h"
 #include "toolbar.h"
 #include "popup-commands.h"
 #include "ephy-encoding-menu.h"
+#include "ephy-tabs-menu.h"
 #include "ephy-stock-icons.h"
 
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <gtk/gtk.h>
@@ -57,6 +62,7 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	{ "File", N_("_File"), NULL, NULL, NULL, NULL, NULL },
 	{ "Edit", N_("_Edit"), NULL, NULL, NULL, NULL, NULL },
 	{ "View", N_("_View"), NULL, NULL, NULL, NULL, NULL },
+	{ "Bookmarks", N_("_Bookmarks"), NULL, NULL, NULL, NULL, NULL },
 	{ "Go", N_("_Go"), NULL, NULL, NULL, NULL, NULL },
 	{ "Tabs", N_("_Tabs"), NULL, NULL, NULL, NULL, NULL },
 	{ "Help", N_("_Help"), NULL, NULL, NULL, NULL, NULL },
@@ -80,9 +86,6 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	{ "FileSendTo", N_("S_end To..."), EPHY_STOCK_SEND_LINK, NULL,
 	  N_("Send a link of the current page"),
 	  G_CALLBACK (window_cmd_file_send_to), NULL },
-	{ "FileBookmarkPage", N_("Boo_kmark Page..."), EPHY_STOCK_BOOKMARK_PAGE, "<control>D",
-	  N_("Add a bookmark for the current page"),
-	  G_CALLBACK (window_cmd_file_bookmark_page), NULL },
 	{ "FileCloseWindow", N_("_Close"), GTK_STOCK_CLOSE, "<control>W",
 	  N_("Close this window"),
 	  G_CALLBACK (window_cmd_file_close_window), NULL },
@@ -148,10 +151,18 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	  N_("Show the contents at the normal size"),
 	  G_CALLBACK (window_cmd_view_zoom_normal), NULL },
 	{ "ViewEncoding", N_("_Encoding"), NULL, NULL, NULL, NULL, NULL },
-	{ "ViewPageSource", N_("_Page Source"), EPHY_STOCK_VIEWSOURCE, NULL,
+	{ "ViewPageSource", N_("_Page Source"), EPHY_STOCK_VIEWSOURCE, "<control>U",
 	  N_("View the source code of the page"),
 	  G_CALLBACK (window_cmd_view_page_source), NULL },
 
+	/* Bookmarks menu */
+	{ "FileBookmarkPage", N_("_Add Bookmark"), EPHY_STOCK_BOOKMARK_PAGE, "<control>D",
+	  N_("Add a bookmark for the current page"),
+	  G_CALLBACK (window_cmd_file_bookmark_page), NULL },
+	{ "GoBookmarks", N_("_Edit Bookmarks"), EPHY_STOCK_BOOKMARKS, "<control>B",
+	  N_("Go to a bookmark"),
+	  G_CALLBACK (window_cmd_go_bookmarks), NULL },
+
 	/* Go menu */
 	{ "GoBack", N_("_Back"), GTK_STOCK_GO_BACK, "<alt>Left",
 	  N_("Go to the previous visited page"),
@@ -171,9 +182,6 @@ static EggActionGroupEntry ephy_menu_entries [] = {
 	{ "GoHistory", N_("H_istory"), EPHY_STOCK_HISTORY, "<control>H",
 	  N_("Go to an already visited page"),
 	  G_CALLBACK (window_cmd_go_history), NULL },
-	{ "GoBookmarks", N_("Boo_kmarks"), EPHY_STOCK_BOOKMARKS, "<control>B",
-	  N_("Go to a bookmark"),
-	  G_CALLBACK (window_cmd_go_bookmarks), NULL },
 
 	/* Tabs menu */
 	{ "TabsPrevious", N_("_Previous Tab"), NULL, "<control>Page_Up",
@@ -229,7 +237,7 @@ static EggActionGroupEntry ephy_popups_entries [] = {
 	  NULL, G_CALLBACK (popup_cmd_download_link), NULL },
 	{ "BookmarkLink", N_("_Bookmark Link..."), EPHY_STOCK_BOOKMARK_PAGE, NULL,
 	  NULL, G_CALLBACK (popup_cmd_bookmark_link), NULL },
-	{ "CopyLinkLocation", N_("_Copy Link Location"), NULL, NULL,
+	{ "CopyLinkLocation", N_("_Copy Link Address"), NULL, NULL,
 	  NULL, G_CALLBACK (popup_cmd_copy_link_location), NULL },
 	{ "CopyEmail", N_("Copy _Email"), NULL, NULL,
 	  NULL, G_CALLBACK (popup_cmd_copy_email), NULL },
@@ -245,7 +253,7 @@ static EggActionGroupEntry ephy_popups_entries [] = {
 	  NULL, G_CALLBACK (popup_cmd_save_image_as), NULL },
 	{ "SetImageAsBackground", N_("Use Image As _Background"), NULL, NULL,
 	  NULL, G_CALLBACK (popup_cmd_set_image_as_background), NULL },
-	{ "CopyImageLocation", N_("_Copy Image Location"), GTK_STOCK_COPY, NULL,
+	{ "CopyImageLocation", N_("_Copy Image Address"), GTK_STOCK_COPY, NULL,
 	  NULL, G_CALLBACK (popup_cmd_copy_image_location), NULL },
 };
 static guint ephy_popups_n_entries = G_N_ELEMENTS (ephy_popups_entries);
@@ -255,12 +263,15 @@ struct EphyWindowPrivate
 	GtkWidget *main_vbox;
 	GtkWidget *menu_dock;
 	GtkWidget *menubar;
+	GtkWidget *exit_fullscreen_popup;
 	Toolbar *toolbar;
 	GtkWidget *statusbar;
 	EggActionGroup *action_group;
 	EggActionGroup *popups_action_group;
 	EphyFavoritesMenu *fav_menu;
 	EphyEncodingMenu *enc_menu;
+	EphyTabsMenu *tabs_menu;
+	EphyBookmarksMenu *bmk_menu;
 	PPViewToolbar *ppview_toolbar;
 	GtkNotebook *notebook;
 	EphyTab *active_tab;
@@ -284,9 +295,8 @@ ephy_window_notebook_switch_page_cb (GtkNotebook *notebook,
 				     EphyWindow *window);
 
 static void
-ephy_window_tab_detached_cb    (EphyNotebook *notebook, gint page,
-				gint x, gint y, gpointer data);
-
+ephy_window_tab_detached_cb	(EphyNotebook *notebook, gint page,
+				 gint x, gint y, gpointer data);
 
 static GObjectClass *parent_class = NULL;
 
@@ -330,34 +340,6 @@ ephy_window_class_init (EphyWindowClass *klass)
 	widget_class->show = ephy_window_show;
 }
 
-static gboolean
-ephy_window_key_press_event_cb (GtkWidget *widget,
-				GdkEventKey *event,
-				EphyWindow *window)
-{
-        int page;
-
-	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
-                return FALSE;
-
-        if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
-        {
-                page = event->keyval - GDK_0 -1;
-
-                if (page == -1) page = 9;
-
-                if (page>=-1 && page<=9)
-                {
-                        gtk_notebook_set_current_page
-				(GTK_NOTEBOOK (window->priv->notebook),
-                                 page == -1 ? -1 : page);
-                        return TRUE;
-                }
-        }
-
-        return FALSE;
-}
-
 static void
 ephy_window_selection_received_cb (GtkWidget *widget,
 				   GtkSelectionData *selection_data,
@@ -376,7 +358,8 @@ ephy_window_selection_received_cb (GtkWidget *widget,
 	if (location != NULL && strcmp (location, "about:blank") != 0)
 	{
 		ephy_shell_new_tab (ephy_shell, window, tab,
-					selection_data->data, 0);
+					selection_data->data, 
+					EPHY_NEW_TAB_OPEN_PAGE);
 	}
 	else
 	{
@@ -496,8 +479,8 @@ setup_window (EphyWindow *window)
 	g_object_set (action, "short_label", _("Print"), NULL);
 	action = egg_action_group_get_action (action_group, "FileBookmarkPage");
 	g_object_set (action, "short_label", _("Bookmark"), NULL);
-	action = egg_action_group_get_action (action_group, "ViewZoomNormal");
-	g_object_set (action, "sensitive", FALSE, NULL);
+	action = egg_action_group_get_action (action_group, "GoBookmarks");
+	g_object_set (action, "short_label", _("Bookmarks"), NULL);
 
 	action_group = egg_action_group_new ("PopupsActions");
 	egg_action_group_add_actions (action_group, ephy_popups_entries,
@@ -521,16 +504,50 @@ setup_window (EphyWindow *window)
 			    GTK_WIDGET (window->priv->toolbar),
 			    FALSE, FALSE, 0);
 
-	g_signal_connect(window,
-			 "key-press-event",
-                         G_CALLBACK(ephy_window_key_press_event_cb),
-                         window);
 	g_signal_connect (window,
 			  "selection-received",
 			  G_CALLBACK (ephy_window_selection_received_cb),
 			  window);
 }
 
+static void
+update_tabs_menu_sensitivity (EphyWindow *window)
+{
+	gboolean prev_tab, next_tab, move_left, move_right, detach;
+	EggActionGroup *action_group;
+	EggAction *action;
+	int current;
+	int last;
+
+	current = gtk_notebook_get_current_page
+		(GTK_NOTEBOOK (window->priv->notebook));
+	last = gtk_notebook_get_n_pages
+		(GTK_NOTEBOOK (window->priv->notebook)) - 1;
+	prev_tab = move_left = (current > 0);
+	next_tab = move_right = (current < last);
+	detach = gtk_notebook_get_n_pages
+		(GTK_NOTEBOOK (window->priv->notebook)) > 1;
+
+	action_group = window->priv->action_group;
+	action = egg_action_group_get_action (action_group, "TabsPrevious");
+	g_object_set (action, "sensitive", prev_tab, NULL);
+	action = egg_action_group_get_action (action_group, "TabsNext");
+	g_object_set (action, "sensitive", next_tab, NULL);
+	action = egg_action_group_get_action (action_group, "TabsMoveLeft");
+	g_object_set (action, "sensitive", move_left, NULL);
+	action = egg_action_group_get_action (action_group, "TabsMoveRight");
+	g_object_set (action, "sensitive", move_right, NULL);
+	action = egg_action_group_get_action (action_group, "TabsDetach");
+	g_object_set (action, "sensitive", detach, NULL);
+}
+
+static void
+ephy_window_tabs_changed_cb (EphyNotebook *notebook, EphyWindow *window)
+{
+	update_tabs_menu_sensitivity (window);
+	ephy_tabs_menu_update (window->priv->tabs_menu);
+}
+
 static GtkNotebook *
 setup_notebook (EphyWindow *window)
 {
@@ -549,6 +566,9 @@ setup_notebook (EphyWindow *window)
 	g_signal_connect (G_OBJECT (notebook), "tab_detached",
 			  G_CALLBACK (ephy_window_tab_detached_cb),
 			  NULL);
+	g_signal_connect (G_OBJECT (notebook), "tabs_changed",
+			  G_CALLBACK (ephy_window_tabs_changed_cb),
+			  window);
 
 	gtk_widget_show (GTK_WIDGET (notebook));
 
@@ -567,13 +587,11 @@ ephy_window_init (EphyWindow *window)
 	window->priv->chrome_mask = 0;
 	window->priv->closing = FALSE;
 	window->priv->ppview_toolbar = NULL;
+	window->priv->exit_fullscreen_popup = NULL;
 
 	/* Setup the window and connect verbs */
 	setup_window (window);
 
-	window->priv->fav_menu = ephy_favorites_menu_new (window);
-	window->priv->enc_menu = ephy_encoding_menu_new (window);
-
 	/* Setup window contents */
 	window->priv->notebook = setup_notebook (window);
 	gtk_box_pack_start (GTK_BOX (window->priv->main_vbox),
@@ -588,6 +606,12 @@ ephy_window_init (EphyWindow *window)
 
 	g_object_ref (ephy_shell);
 
+	/* Initializ the menus */
+	window->priv->tabs_menu = ephy_tabs_menu_new (window);
+	window->priv->fav_menu = ephy_favorites_menu_new (window);
+	window->priv->enc_menu = ephy_encoding_menu_new (window);
+	window->priv->bmk_menu = ephy_bookmarks_menu_new (window);
+
 	/* Once window is fully created, add it to the session list*/
 	session_add_window (session, window);
 }
@@ -597,13 +621,9 @@ save_window_chrome (EphyWindow *window)
 {
 	EmbedChromeMask flags = window->priv->chrome_mask;
 
-	if (flags & EMBED_CHROME_OPENASPOPUP)
-	{
-	}
-	else if (flags & EMBED_CHROME_PPVIEWTOOLBARON)
-	{
-	}
-	else
+	if (!(flags & EMBED_CHROME_OPENASPOPUP) &&
+	    !(flags & EMBED_CHROME_PPVIEWTOOLBARON) &&
+	    !(flags & EMBED_CHROME_OPENASFULLSCREEN))
 	{
 		eel_gconf_set_boolean (CONF_WINDOWS_SHOW_BOOKMARKS_BAR,
 				       flags & EMBED_CHROME_BOOKMARKSBARON);
@@ -645,6 +665,8 @@ ephy_window_finalize (GObject *object)
 
 	g_object_unref (window->priv->fav_menu);
 	g_object_unref (window->priv->enc_menu);
+	g_object_unref (window->priv->tabs_menu);
+	g_object_unref (window->priv->bmk_menu);
 
 	if (window->priv->ppview_toolbar)
 	{
@@ -677,57 +699,6 @@ ephy_window_get_chrome (EphyWindow *window)
 	return window->priv->chrome_mask;
 }
 
-static void
-wmspec_change_state (gboolean   add,
-                     GdkWindow *window,
-                     GdkAtom    state1,
-                     GdkAtom    state2)
-{
-	XEvent xev;
-
-	#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
-	#define _NET_WM_STATE_ADD           1    /* add/set property */
-	#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */
-
-	xev.xclient.type = ClientMessage;
-	xev.xclient.serial = 0;
-	xev.xclient.send_event = True;
-	xev.xclient.display = gdk_display;
-	xev.xclient.window = GDK_WINDOW_XID (window);
-	xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
-	xev.xclient.format = 32;
-	xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
-	xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1);
-	xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2);
-
-	XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
-                    False,
-                    SubstructureRedirectMask | SubstructureNotifyMask,
-                    &xev);
-}
-
-static void
-window_set_fullscreen_mode (EphyWindow *window, gboolean active)
-{
-        GdkWindow *gdk_window;
-
-        gdk_window = GTK_WIDGET(window)->window;
-
-        if (gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                  FALSE)))
-        {
-                wmspec_change_state (active,
-                                     gdk_window,
-                                     gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                      FALSE),
-                                     GDK_NONE);
-        }
-        else
-        {
-                g_warning ("NET_WM_STATE_FULLSCREEN not supported");
-	}
-}
-
 static void
 translate_default_chrome (EmbedChromeMask *chrome_mask)
 {
@@ -741,7 +712,8 @@ translate_default_chrome (EmbedChromeMask *chrome_mask)
 			 EMBED_CHROME_OPENASFULLSCREEN);
 
 	/* Load defaults */
-	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR))
+	if (eel_gconf_get_boolean (CONF_WINDOWS_SHOW_STATUSBAR) &&
+	    !(*chrome_mask & EMBED_CHROME_OPENASFULLSCREEN))
 	{
 		*chrome_mask |= EMBED_CHROME_STATUSBARON;
 	}
@@ -785,6 +757,104 @@ update_layout_toggles (EphyWindow *window)
 				      mask & EMBED_CHROME_OPENASFULLSCREEN);
 }
 
+static void
+update_exit_fullscreen_popup_position (EphyWindow *window)
+{
+	GdkRectangle screen_rect;
+	int popup_height;
+
+	gtk_window_get_size (GTK_WINDOW (window->priv->exit_fullscreen_popup),
+                             NULL, &popup_height);
+
+	gdk_screen_get_monitor_geometry (gdk_screen_get_default (),
+                        gdk_screen_get_monitor_at_window
+                        (gdk_screen_get_default (),
+                         GTK_WIDGET (window)->window),
+                         &screen_rect);
+
+	gtk_window_move (GTK_WINDOW (window->priv->exit_fullscreen_popup),
+                        screen_rect.x, screen_rect.height - popup_height);
+}
+
+static void
+size_changed_cb (GdkScreen *screen, EphyWindow *window)
+{
+	update_exit_fullscreen_popup_position (window);
+}
+
+static void
+exit_fullscreen_button_clicked_cb (GtkWidget *button, EphyWindow *window)
+{
+	EmbedChromeMask mask;
+
+	mask = ephy_window_get_chrome (window);
+
+	mask ^= EMBED_CHROME_OPENASFULLSCREEN;
+	mask |= EMBED_CHROME_DEFAULT;
+
+	ephy_window_set_chrome (window, mask);
+}
+
+static void
+ephy_window_fullscreen (EphyWindow *window)
+{
+	GtkWidget *popup, *button, *icon, *label, *hbox;
+	EphyToolbarsModel *tmodel;
+
+	gtk_window_fullscreen (GTK_WINDOW (window));
+
+	tmodel = ephy_shell_get_toolbars_model (ephy_shell);
+	ephy_toolbars_model_set_flag (tmodel, EGG_TB_MODEL_ICONS_ONLY);
+
+	popup = gtk_window_new (GTK_WINDOW_POPUP);
+	window->priv->exit_fullscreen_popup = popup;
+
+	button = gtk_button_new ();
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (exit_fullscreen_button_clicked_cb),
+			  window);
+	gtk_widget_show (button);
+	gtk_container_add (GTK_CONTAINER (popup), button);
+
+	hbox = gtk_hbox_new (FALSE, 2);
+	gtk_widget_show (hbox);
+	gtk_container_add (GTK_CONTAINER (button), hbox);
+
+	icon = gtk_image_new_from_stock (GTK_STOCK_QUIT, GTK_ICON_SIZE_BUTTON);
+	gtk_widget_show (icon);
+	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+
+	label = gtk_label_new (_("Exit Fullscreen"));
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	update_exit_fullscreen_popup_position (window);
+
+	gtk_widget_show (popup);
+
+	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
+                          "size-changed", G_CALLBACK (size_changed_cb),
+			  popup);
+}
+
+static void
+ephy_window_unfullscreen (EphyWindow *window)
+{
+	EphyToolbarsModel *tmodel;
+
+	tmodel = ephy_shell_get_toolbars_model (ephy_shell);
+	ephy_toolbars_model_unset_flag (tmodel, EGG_TB_MODEL_ICONS_ONLY);
+
+	g_signal_handlers_disconnect_by_func (G_OBJECT (gdk_screen_get_default ()),
+                                              G_CALLBACK (size_changed_cb),
+					      window);
+
+	gtk_widget_destroy (window->priv->exit_fullscreen_popup);
+	window->priv->exit_fullscreen_popup = NULL;
+
+	gtk_window_unfullscreen (GTK_WINDOW (window));
+}
+
 void
 ephy_window_set_chrome (EphyWindow *window,
 			EmbedChromeMask flags)
@@ -837,8 +907,14 @@ ephy_window_set_chrome (EphyWindow *window,
 	    (flags & EMBED_CHROME_OPENASFULLSCREEN))
 	{
 		save_window_chrome (window);
-		window_set_fullscreen_mode (window,
-					    flags & EMBED_CHROME_OPENASFULLSCREEN);
+		if (flags & EMBED_CHROME_OPENASFULLSCREEN)
+		{
+			ephy_window_fullscreen (window);
+		}
+		else
+		{
+			ephy_window_unfullscreen (window);
+		}
 	}
 
 	window->priv->chrome_mask = flags;
@@ -857,11 +933,10 @@ ephy_window_get_notebook (EphyWindow *window)
 void
 ephy_window_add_tab (EphyWindow *window,
 		     EphyTab *tab,
-		     gboolean grouped,
+		     gint position,
 		     gboolean jump_to)
 {
 	GtkWidget *widget;
-	int position;
 
 	g_return_if_fail (IS_EPHY_WINDOW (window));
 	g_return_if_fail (IS_EPHY_TAB (tab));
@@ -870,7 +945,6 @@ ephy_window_add_tab (EphyWindow *window,
 
 	widget = GTK_WIDGET(ephy_tab_get_embed (tab));
 
-	position = grouped ? EPHY_NOTEBOOK_INSERT_GROUPED : EPHY_NOTEBOOK_INSERT_LAST;
 	ephy_notebook_insert_page (EPHY_NOTEBOOK (window->priv->notebook),
 				   widget, position, jump_to);
 }
@@ -970,7 +1044,8 @@ ephy_window_show (GtkWidget *widget)
 	{
 		ephy_state_add_window (widget,
 				       "main_window",
-			               600, 500);
+			               600, 500,
+				       EPHY_STATE_WINDOW_SAVE_SIZE);
 	}
 
 	if ((window->priv->chrome_mask & EMBED_CHROME_OPENASPOPUP) &&
@@ -1124,6 +1199,40 @@ update_nav_control (EphyWindow *window)
 					   back, forward, up);
 }
 
+static void
+update_zoom_control (EphyWindow *window)
+{
+	EphyEmbed *embed;
+	EggActionGroup *action_group;
+	EggAction *action;
+	gboolean can_zoom_in = TRUE, can_zoom_out = TRUE, can_zoom_normal = FALSE;
+	float zoom = 1.0;
+	gresult rv;
+
+	g_return_if_fail (window != NULL);
+	
+	embed = ephy_window_get_active_embed (window);	
+	g_return_if_fail (embed != NULL);
+
+	rv = ephy_embed_zoom_get (embed, &zoom);
+	if (rv == G_OK)
+	{
+		if (zoom >= ZOOM_MAXIMAL) can_zoom_in = FALSE; 
+		if (zoom <= ZOOM_MINIMAL) can_zoom_out = FALSE; 
+		if (zoom != 1.0) can_zoom_normal = TRUE;
+	}
+
+	toolbar_update_zoom (window->priv->toolbar, zoom);
+
+	action_group = window->priv->action_group;
+	action = egg_action_group_get_action (action_group, "ViewZoomIn");
+	g_object_set (action, "sensitive", can_zoom_in, NULL);
+	action = egg_action_group_get_action (action_group, "ViewZoomOut");
+	g_object_set (action, "sensitive", can_zoom_out, NULL);
+	action = egg_action_group_get_action (action_group, "ViewZoomNormal");
+	g_object_set (action, "sensitive", can_zoom_normal, NULL);
+}
+
 static void
 update_title_control (EphyWindow *window)
 {
@@ -1207,37 +1316,6 @@ update_find_control (EphyWindow *window)
 	}
 }
 
-static void
-update_tabs (EphyWindow *window)
-{
-	gboolean prev_tab, next_tab, move_left, move_right, detach;
-	EggActionGroup *action_group;
-	EggAction *action;
-	int current;
-	int last;
-
-	current = gtk_notebook_get_current_page
-		(GTK_NOTEBOOK (window->priv->notebook));
-	last = gtk_notebook_get_n_pages
-		(GTK_NOTEBOOK (window->priv->notebook)) - 1;
-	prev_tab = move_left = (current > 0);
-	next_tab = move_right = (current < last);
-	detach = gtk_notebook_get_n_pages
-		(GTK_NOTEBOOK (window->priv->notebook)) > 1;
-
-	action_group = window->priv->action_group;
-	action = egg_action_group_get_action (action_group, "TabsPrevious");
-	g_object_set (action, "sensitive", prev_tab, NULL);
-	action = egg_action_group_get_action (action_group, "TabsNext");
-	g_object_set (action, "sensitive", next_tab, NULL);
-	action = egg_action_group_get_action (action_group, "TabsMoveLeft");
-	g_object_set (action, "sensitive", move_left, NULL);
-	action = egg_action_group_get_action (action_group, "TabsMoveRight");
-	g_object_set (action, "sensitive", move_right, NULL);
-	action = egg_action_group_get_action (action_group, "TabsDetach");
-	g_object_set (action, "sensitive", detach, NULL);
-}
-
 static void
 update_window_visibility (EphyWindow *window)
 {
@@ -1293,9 +1371,6 @@ ephy_window_update_control (EphyWindow *window,
 
 	switch (control)
 	{
-	case TabsControl:
-		update_tabs (window);
-		break;
 	case StatusbarMessageControl:
 		update_status_message (window);
 		break;
@@ -1312,6 +1387,7 @@ ephy_window_update_control (EphyWindow *window,
 		 /* the zoom control is updated at the same time than the navigation
 		    controls. This keeps it synched most of the time, but not always,
 		    because we don't get a notification when zoom changes */
+		update_zoom_control (window);
 	case NavControl:
 		update_nav_control (window);
 		break;
@@ -1355,7 +1431,7 @@ ephy_window_update_all_controls (EphyWindow *window)
 		update_security (window);
 		update_find_control (window);
 		update_spinner_control (window);
-		update_tabs (window);
+		update_zoom_control (window);
 	}
 }
 
@@ -1397,11 +1473,11 @@ ephy_window_get_tabs (EphyWindow *window)
 		tab = g_object_get_data (G_OBJECT (w), "EphyTab");
 		g_return_val_if_fail (IS_EPHY_TAB (G_OBJECT (tab)), NULL);
 
-		tabs = g_list_append (tabs, tab);
+		tabs = g_list_prepend (tabs, tab);
 		i++;
 	}
 
-	return tabs;
+	return g_list_reverse (tabs);
 }
 
 static void
@@ -1462,6 +1538,7 @@ ephy_window_notebook_switch_page_cb (GtkNotebook *notebook,
 
 	/* update window controls */
 	ephy_window_update_all_controls (window);
+	update_tabs_menu_sensitivity (window);
 }
 
 static void
@@ -1497,27 +1574,31 @@ ephy_window_get_find_dialog (EphyWindow *window)
 
 void
 ephy_window_set_zoom (EphyWindow *window,
-		      gint zoom)
+		      float zoom)
 {
 	EphyEmbed *embed;
-	gboolean zoom_out, zoom_normal;
-	EggActionGroup *action_group;
-	EggAction *action;
+	float current_zoom = 1.0;
 
         g_return_if_fail (IS_EPHY_WINDOW (window));
 
 	embed = ephy_window_get_active_embed (window);
         g_return_if_fail (embed != NULL);
 
-        ephy_embed_zoom_set (embed, zoom, TRUE);
+	ephy_embed_zoom_get (embed, &current_zoom);
 
-	zoom_normal = (zoom != 100);
-	zoom_out = (zoom > 0);
-	action_group = window->priv->action_group;
-	action = egg_action_group_get_action (action_group, "ViewZoomOut");
-	g_object_set (action, "sensitive", zoom_out, NULL);
-	action = egg_action_group_get_action (action_group, "ViewZoomNormal");
-	g_object_set (action, "sensitive", zoom_normal, NULL);
+	if (zoom == ZOOM_IN)
+	{
+		zoom = ephy_zoom_get_changed_zoom_level (current_zoom, 1);
+	}
+	else if (zoom == ZOOM_OUT)
+	{
+		zoom = ephy_zoom_get_changed_zoom_level (current_zoom, -1);
+	}
+
+	if (zoom != current_zoom)
+	{
+		ephy_embed_zoom_set (embed, zoom, TRUE);
+	}
 }
 
 Toolbar *
@@ -1526,7 +1607,7 @@ ephy_window_get_toolbar (EphyWindow *window)
 	return window->priv->toolbar;
 }
 
-void
+static void
 ephy_window_tab_detached_cb (EphyNotebook *notebook, gint page,
 			     gint x, gint y, gpointer data)
 {
@@ -1543,3 +1624,4 @@ ephy_window_tab_detached_cb (EphyNotebook *notebook, gint page,
 	ephy_tab_set_window (tab, window);
 	gtk_widget_show (GTK_WIDGET (window));
 }
+
