@@ -36,6 +36,7 @@
 #include "toolbar.h"
 #include "ephy-state.h"
 #include "ephy-gui.h"
+#include "ephy-zoom.h"
 
 #include <string.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -207,11 +208,35 @@ window_cmd_view_reload (EggAction *action,
 		        EphyWindow *window)
 {
 	EphyEmbed *embed;
+	GdkEvent *event;
+	GdkEventType type;
+	guint state = 0;
+	gboolean force = FALSE;
 
 	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (embed != NULL);
 
-	ephy_embed_reload (embed, EMBED_RELOAD_NORMAL);
+	event = gtk_get_current_event ();
+	type = event->type;
+
+	if (type == GDK_BUTTON_RELEASE)
+	{
+		state = event->button.state; 
+	}
+	else if (type == GDK_KEY_RELEASE)
+	{
+		state = event->key.state;
+	}
+
+	gdk_event_free (event);
+
+	if (state & GDK_SHIFT_MASK)
+	{
+		force = TRUE;
+	}
+
+	ephy_embed_reload (embed, force ? EMBED_RELOAD_NORMAL
+					: EMBED_RELOAD_BYPASSCACHE);
 }
 
 void
@@ -223,9 +248,8 @@ window_cmd_file_new_window (EggAction *action,
 	tab = ephy_window_get_active_tab (window);
 
 	ephy_shell_new_tab (ephy_shell, NULL, tab, NULL,
-			    EPHY_NEW_TAB_HOMEPAGE |
-			    EPHY_NEW_TAB_IN_NEW_WINDOW |
-			    EPHY_NEW_TAB_JUMP);
+			    EPHY_NEW_TAB_NEW_PAGE |
+			    EPHY_NEW_TAB_IN_NEW_WINDOW);
 }
 
 void
@@ -237,7 +261,7 @@ window_cmd_file_new_tab (EggAction *action,
 	tab = ephy_window_get_active_tab (window);
 
 	ephy_shell_new_tab (ephy_shell, window, tab, NULL,
-			      EPHY_NEW_TAB_HOMEPAGE |
+			      EPHY_NEW_TAB_NEW_PAGE |
 			      EPHY_NEW_TAB_IN_EXISTING_WINDOW |
 			      EPHY_NEW_TAB_JUMP);
 }
@@ -565,38 +589,21 @@ void
 window_cmd_view_zoom_in	(EggAction *action,
 			 EphyWindow *window)
 {
-	EphyEmbed *embed;
-	int zoom;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_zoom_get (embed, &zoom);
-	ephy_window_set_zoom (window, zoom + 10);
+	ephy_window_set_zoom (window, ZOOM_IN);
 }
 
 void
 window_cmd_view_zoom_out (EggAction *action,
 			  EphyWindow *window)
 {
-	EphyEmbed *embed;
-	int zoom;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_zoom_get (embed, &zoom);
-	if (zoom >= 10)
-	{
-		ephy_window_set_zoom (window, zoom - 10);
-	}
+	ephy_window_set_zoom (window, ZOOM_OUT);
 }
 
 void
 window_cmd_view_zoom_normal (EggAction *action,
 			     EphyWindow *window)
 {
-	ephy_window_set_zoom (window, 100);
+	ephy_window_set_zoom (window, 1.0);
 }
 
 void
@@ -609,7 +616,8 @@ window_cmd_view_page_source (EggAction *action,
 	g_return_if_fail (tab != NULL);
 
 	ephy_shell_new_tab (ephy_shell, window, tab, NULL,
-			      EPHY_NEW_TAB_VIEW_SOURCE);
+			    EPHY_NEW_TAB_CLONE_PAGE |
+			    EPHY_NEW_TAB_SOURCE_MODE);
 }
 
 void
@@ -695,18 +703,25 @@ window_cmd_edit_toolbar (EggAction *action,
 		 EGG_TOOLBARS_MODEL (model));
 	egg_toolbar_editor_load_actions (EGG_TOOLBAR_EDITOR (editor),
 					 ephy_file ("epiphany-toolbar.xml"));
+	gtk_container_set_border_width (GTK_CONTAINER (EGG_TOOLBAR_EDITOR (editor)), 5);
+	gtk_box_set_spacing (GTK_BOX (EGG_TOOLBAR_EDITOR (editor)), 5);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), editor);
 	g_signal_connect (editor, "destroy",
 			  G_CALLBACK (toolbar_editor_destroy_cb),
 			  t);
 	gtk_widget_show (editor);
+	
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
 
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)), 5);
+	
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       _("_Add a New Toolbar"), RESPONSE_ADD_TOOLBAR);
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 	gtk_dialog_add_button (GTK_DIALOG (dialog),
 			       GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+	
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (toolbar_editor_response_cb), NULL);
 	ephy_state_add_window (dialog,
